#include <math.h> 
#include <stdlib.h> 
#include <stdio.h>
#include <memory.h>

#include "global.h"
#include "block.h" 

#define Q_BITS          15
#define DQ_BITS         6
#define DQ_ROUND        (1<<(DQ_BITS-1))

//#define FAST_INTEGER
#define max(a, b)  (((a) > (b)) ? (a) : (b)) 
#define min(a, b)  (((a) < (b)) ? (a) : (b)) 
static const int quant_coef[6][4][4] = {
  {{13107, 8066,13107, 8066},{ 8066, 5243, 8066, 5243},{13107, 8066,13107, 8066},{ 8066, 5243, 8066, 5243}},
  {{11916, 7490,11916, 7490},{ 7490, 4660, 7490, 4660},{11916, 7490,11916, 7490},{ 7490, 4660, 7490, 4660}},
  {{10082, 6554,10082, 6554},{ 6554, 4194, 6554, 4194},{10082, 6554,10082, 6554},{ 6554, 4194, 6554, 4194}},
  {{ 9362, 5825, 9362, 5825},{ 5825, 3647, 5825, 3647},{ 9362, 5825, 9362, 5825},{ 5825, 3647, 5825, 3647}},
  {{ 8192, 5243, 8192, 5243},{ 5243, 3355, 5243, 3355},{ 8192, 5243, 8192, 5243},{ 5243, 3355, 5243, 3355}},
  {{ 7282, 4559, 7282, 4559},{ 4559, 2893, 4559, 2893},{ 7282, 4559, 7282, 4559},{ 4559, 2893, 4559, 2893}}
};

static const int dequant_coef[6][4][4] = {
  {{10, 13, 10, 13},{ 13, 16, 13, 16},{10, 13, 10, 13},{ 13, 16, 13, 16}},
  {{11, 14, 11, 14},{ 14, 18, 14, 18},{11, 14, 11, 14},{ 14, 18, 14, 18}},
  {{13, 16, 13, 16},{ 16, 20, 16, 20},{13, 16, 13, 16},{ 16, 20, 16, 20}},
  {{14, 18, 14, 18},{ 18, 23, 18, 23},{14, 18, 14, 18},{ 18, 23, 18, 23}},
  {{16, 20, 16, 20},{ 20, 25, 20, 25},{16, 20, 16, 20},{ 20, 25, 20, 25}},
  {{18, 23, 18, 23},{ 23, 29, 23, 29},{18, 23, 18, 23},{ 23, 29, 23, 29}}
};
static const int A[4][4] = {
  { 16, 20, 16, 20},
  { 20, 25, 20, 25},
  { 16, 20, 16, 20},
  { 20, 25, 20, 25}
};
static int quant_coef1[6][16];
static int dequant_coef1[6][16];
extern char intra_mb_pos[72][92];
extern char intra_mb_pos1[72][92];

void InitQuantTable()
{
	int i,j,coeff_ctr,qp_rem;

	for(qp_rem=0;qp_rem<6;qp_rem++)
	  for (coeff_ctr=0;coeff_ctr < 16;coeff_ctr++)
	  { 
		i=SNGL_SCAN[coeff_ctr][0];
		j=SNGL_SCAN[coeff_ctr][1]; 
		quant_coef1[qp_rem][coeff_ctr]=quant_coef[qp_rem][i][j];
		dequant_coef1[qp_rem][coeff_ctr]=dequant_coef[qp_rem][i][j];
	  }
}

/************************************************************************
*
*  Routine:     intrapred_chroma()
*
*  Description: Chroma intra prediction.  There are 3 pred modes:
*				1. DC, Used if:
*					a) 16x16 based 'plane' luma prediction
*					b) 16x16 based vert/hor pred. impossible due to edges
*					b) if sum of 4x4 luma pred. is 'DC based' 
*			    2) Vertical, used if:
*					a) 16x16 based vertical luma pred. is used
*					b) if sum of 4x4 luma pred is 'vertical based'
*			    3) Horizontal, used if:
*					a) 16x16 based horizontal luma pred. is used
*					b) if sum of 4x4 luma pred is 'horizontal based'
*
*  Input:       Starting point of current chroma macro block image posision, chroma component and predmode
*
*  Output:      8x8 array with given intra chroma prediction and diff array   
*                    
************************************************************************/

void intrapred_chroma(struct img_par *img,int img_c_x,int img_c_y,int uv)
{
  int     s, s0, s1, s2, s3, i, j;
  int     block_x, block_y;
  int     img_cx            = img_c_x;//img->pix_c_x;
  int     img_cy            = img_c_y;//img->pix_c_y;
  int     img_cx_1          = img_c_x-1;//img->pix_c_x-1;
  int     img_cx_4          = img_c_x+4;//img->pix_c_x+4;
  int     img_cy_1          = img_c_y-1;//img->pix_c_y-1;
  int     img_cy_4          = img_c_y+4;//img->pix_c_y+4;
  int     *pp;
  int     width_cr=img->width_cr+IMG_PAD_SIZE;

  if(uv==0) pp=MbCb;
  else pp=MbCr;

  for (block_y=0; block_y<8; block_y+=4)
    for (block_x=0; block_x<8; block_x+=4)
    {
      s=128;
      s0=s1=s2=s3=0;
      //===== get prediction value =====
      switch ((block_y>>1) + (block_x>>2))
      {
      case 0:  //===== TOP LEFT =====
        if      (img_cy > 0)    for (i=0;i<4;i++)  s0 += imgUV[uv][img_cy_1*width_cr+img_cx  +i];
        if      (img_cx > 0)  for (i=0;i<4;i++)  s2 += imgUV[uv][(img_cy  +i)*width_cr+img_cx_1  ];
        if      ((img_cy > 0) && (img_cx > 0))  s  = (s0+s2+4) >> 3;
        else if (img_cy > 0)                       s  = (s0   +2) >> 2;
        else if (img_cx > 0)                     s  = (s2   +2) >> 2;
        img->s[0]=s;
		break;
      case 1: //===== TOP RIGHT =====
        if      (img_cy > 0)    for (i=0;i<4;i++)  s1 += imgUV[uv][img_cy_1*width_cr+img_cx_4+i];
        else if (img_cx > 0)  for (i=0;i<4;i++)  s2 += imgUV[uv][(img_cy  +i)*width_cr+img_cx_1  ];
        if      (img_cy > 0)                       s  = (s1   +2) >> 2;
        else if (img_cx > 0)                     s  = (s2   +2) >> 2;
        img->s[1]=s;
		break;
      case 2: //===== BOTTOM LEFT =====
        if      (img_cx > 0)  for (i=0;i<4;i++)  s3 += imgUV[uv][(img_cy_4+i)*width_cr+img_cx_1  ];
        else if (img_cy > 0)    for (i=0;i<4;i++)  s0 += imgUV[uv][img_cy_1*width_cr+img_cx  +i];
        if      (img_cx > 0)                     s  = (s3   +2) >> 2;
        else if (img_cy > 0)                       s  = (s0   +2) >> 2;
        img->s[2]=s;
		break;
      case 3: //===== BOTTOM RIGHT =====
        if      (img_cy > 0)    for (i=0;i<4;i++)  s1 += imgUV[uv][img_cy_1*width_cr+img_cx_4+i];
        if      (img_cx > 0)  for (i=0;i<4;i++)  s3 += imgUV[uv][(img_cy_4+i)*width_cr+img_cx_1  ];
        if      ((img_cy > 0) && (img_cx > 0))  s  = (s1+s3+4) >> 3;
        else if (img_cy > 0)                       s  = (s1   +2) >> 2;
        else if (img_cx > 0)                     s  = (s3   +2) >> 2;
        img->s[3]=s;
		break;
      }

      //===== prediction =====
      for (j=block_y; j<block_y+4; j++)
      for (i=block_x; i<block_x+4; i++)
      {
		pp[j*8+i]=s;
      }
    }   
}

/************************************************************************
*
*  Routine:     motion_search()
*
*  Description: In this routine motion search (integer pel+1/3 pel) and mode selection 
*               is performed. Since we treat all 4x4 blocks before coding/decoding the 
*               prediction may not be based on decoded pixels (except for some of the blocks). 
*               This will result in too good prediction.  To compensate for this the SAD for 
*               intra(tot_intra_sad) is given a 'handicap' depending on QP.
*
*  Input:       Best intra SAD value.
*
*  Output:      Reference image.  
*
************************************************************************/
#define IABS(x) absm((x))

int SAD_Macroblock(struct img_par *img,byte* ii, byte* kk,int Min_FRAME,int curr_sad)
{
	int sad = curr_sad;
#ifdef FAST_SAD
	int i;
	int x0,x1,x2,x3,x4,x5;
	int y0,y1,y2,y3,y4,y5;
	int lx=img->width+(img->width<<1);
	for(i=0;i<16;i+=3)
	{
		x0=ii[ 0];	y0=kk[ 0];
		x1=ii[ 3];	y1=kk[ 3];
		x2=ii[ 6];	y2=kk[ 6];
		x3=ii[ 9];	y3=kk[ 9];
		x4=ii[12];	y4=kk[12];
		x5=ii[15];	y5=kk[15];
		sad+=IABS(x0-y0)+IABS(x1-y1)+IABS(x2-y2)+IABS(x3-y3)+IABS(x4-y4)+IABS(x5-y5);
		ii += lx;	kk += lx;
		if (sad > Min_FRAME)	return 32767;
	}
	return sad;
#else
	int i;
	int lx=img->width+IMG_PAD_SIZE;
	for(i=0;i<16;i+=1)
	{
		sad += (\
				 IABS(ii[ 0]-kk[ 0])
				+IABS(ii[ 1]-kk[ 1])
				+IABS(ii[ 2]-kk[ 2])
				+IABS(ii[ 3]-kk[ 3])
				+IABS(ii[ 4]-kk[ 4])
				+IABS(ii[ 5]-kk[ 5])
				+IABS(ii[ 6]-kk[ 6])
				+IABS(ii[ 7]-kk[ 7])
				+IABS(ii[ 8]-kk[ 8])
				+IABS(ii[ 9]-kk[ 9])
				+IABS(ii[10]-kk[10])
				+IABS(ii[11]-kk[11])
				+IABS(ii[12]-kk[12])
				+IABS(ii[13]-kk[13])
				+IABS(ii[14]-kk[14])
				+IABS(ii[15]-kk[15]));
		ii += lx;
		kk += lx;
		if (sad > Min_FRAME)
			return 0x7fff;
	} 
	return sad;
#endif
}

int SAD_Macroblock_sub(struct img_par *img,byte* ii, byte* kk,int Min_FRAME,int curr_sad)
{
	int sad = curr_sad;
	int i;
	int lx=img->width+IMG_PAD_SIZE;
	for(i=0;i<8;i+=1)
	{
		sad += (\
				 IABS(ii[ 0]-kk[ 0])
				+IABS(ii[ 1]-kk[ 1])
				+IABS(ii[ 2]-kk[ 2])
				+IABS(ii[ 3]-kk[ 3])
				+IABS(ii[ 4]-kk[ 4])
				+IABS(ii[ 5]-kk[ 5])
				+IABS(ii[ 6]-kk[ 6])
				+IABS(ii[ 7]-kk[ 7])
				+IABS(ii[ 8]-kk[ 8])
				+IABS(ii[ 9]-kk[ 9])
				+IABS(ii[10]-kk[10])
				+IABS(ii[11]-kk[11])
				+IABS(ii[12]-kk[12])
				+IABS(ii[13]-kk[13])
				+IABS(ii[14]-kk[14])
				+IABS(ii[15]-kk[15]));
		ii += lx;
		kk += lx;
		if (sad > Min_FRAME)
			return 0x7fff;
	} 
	return sad;
}

int SAD_Macroblock_sub2(struct img_par *img,byte* ii, byte* kk,int Min_FRAME,int curr_sad)
{
	int sad = curr_sad;
	int i;
	int lx=img->width+IMG_PAD_SIZE;
	for(i=0;i<16;i+=1)
	{
		sad += (\
				 IABS(ii[ 0]-kk[ 0])
				+IABS(ii[ 1]-kk[ 1])
				+IABS(ii[ 2]-kk[ 2])
				+IABS(ii[ 3]-kk[ 3])
				+IABS(ii[ 4]-kk[ 4])
				+IABS(ii[ 5]-kk[ 5])
				+IABS(ii[ 6]-kk[ 6])
				+IABS(ii[ 7]-kk[ 7]));
		ii += lx;
		kk += lx;
		if (sad > Min_FRAME)
			return 0x7fff;
	} 
	return sad;
}

int motion_search_16x8(struct img_par *img,int MinCost)
{
  int i,index;
  int vec0_x,vec0_y,vec1_x,vec1_y,vec2_x,vec2_y;
  int pic_block_x,pic_block_y,pic_pix_y,pic_pix_x;
  int ip0,ip1,ip2,ip3,ip4,ip5;
  int best_inter_sad[2],current_inter_sad;
  int tmp0,tmp1; 
  int lambda = QP2QUANT[img->qp-SHIFT_QP]; 
  int x,y,offset,type,pos;  
        byte* curr;
	byte* prev;
	Point* point=search;
	SAD_TABLE * sadTbl=sadTable;
	int maxX=img->width-12;
	int maxY=img->height-4;
	int sadTimes=25;
	int mvx0,mvy0,mvx1,mvy1; 
	int x_curr,y_curr;
	int width=img->width+IMG_PAD_SIZE;

#ifdef FAST_INTEGER	
	int prev_AE1[17];
	int prev_m1,prev_m2=0;
	static int prevm[4] = {2, 3, 0, 1};
	static int htp[4] = {-1, 0, 1, 0};
	static int vtp[4] = {0, 1, 0, -1};
	int j_min_now, i_min_now, i_min_next=0, j_min_next=0, sad_layr,Coffset,ioffset=0,l,m,j;
	int distortion_0, distortion_1, distortion_2;
#endif  

  for(index=0;index<2;index++)
  {
	  point=search;
	  sadTbl=sadTable;
	  pic_block_y=img->block_y+2*index;
	  pic_pix_y=img->pix_y+8*index;
	        
	  pic_block_x=img->block_x;
	  pic_pix_x=img->pix_x;
	
	  x_curr=pic_pix_x;
	  y_curr=pic_pix_y;
	                    
	  if(index==0)
	  {
		  vec0_x=pic_block_x-1;
		  vec0_y=pic_block_y;
		  vec1_x=pic_block_x;
		  vec1_y=pic_block_y-1;
		  vec2_x=pic_block_x+4;
		  vec2_y=pic_block_y-1;
		  if (pic_block_y == 0) 
          {
            vec1_x=vec0_x;
            vec2_x=vec0_x;
            vec1_y=vec0_y;
            vec2_y=vec0_y;
          }                
          if (vec2_x > (img->width>>2)-1) 
          {
            vec2_x=pic_block_x-1;
          }
		  
            ip1=tmp_mv[vec1_y][vec1_x+4][0];
			ip0=tmp_mv[vec0_y][vec0_x+4][0];
			ip2=tmp_mv[vec2_y][vec2_x+4][0];
			ip3=tmp_mv[vec0_y][vec0_x+4][1];
            ip4=tmp_mv[vec1_y][vec1_x+4][1];
            ip5=tmp_mv[vec2_y][vec2_x+4][1];
		  if(!intra_mb_pos[vec1_y][vec1_x+4])
			{
				img->mv1[index][0] = ip1;
				img->mv1[index][1] = ip4;
			}
		  else
		  {			  				
				if((!intra_mb_pos[vec0_y][vec0_x+4])&&(intra_mb_pos[vec2_y][vec2_x+4]))
				{
					img->mv1[index][0] = ip0;
					img->mv1[index][1] = ip3;
				}
				else if((intra_mb_pos[vec0_y][vec0_x+4])&&(!intra_mb_pos[vec2_y][vec2_x+4]))
				{
					img->mv1[index][0] = ip2;
					img->mv1[index][1] = ip5;
				}
				else
				{
					img->mv1[index][0]=ip0+ip1+ip2-min(min(ip0,ip1),ip2)-max(max(ip0,ip1),ip2);
					img->mv1[index][1]=ip3+ip4+ip5-min(min(ip3,ip4),ip5)-max(max(ip3,ip4),ip5);
				}
		  }
	  }
	  else
	  {
		  vec0_x=pic_block_x-1;
		  vec0_y=pic_block_y;
		  vec1_x=pic_block_x;
		  vec1_y=pic_block_y-1;
		  vec2_x=pic_block_x-1;
		  vec2_y=pic_block_y-1;
		  
            ip1=curr_mvx1[0];
            ip4=curr_mvy1[0];
            ip0=tmp_mv[vec0_y][vec0_x+4][0];
			ip2=tmp_mv[vec2_y][vec2_x+4][0];
			ip3=tmp_mv[vec0_y][vec0_x+4][1];            
			ip5=tmp_mv[vec2_y][vec2_x+4][1];
		  if(!intra_mb_pos[vec0_y][vec0_x+4])
			{
				img->mv1[index][0] = ip0;
				img->mv1[index][1] = ip3;
			}
		  else
		  {
				if(intra_mb_pos[vec2_y][vec2_x+4])
				{
					img->mv1[index][0] = ip1;
					img->mv1[index][1] = ip4;
				}
				else
				{
					img->mv1[index][0]=ip0+ip1+ip2-min(min(ip0,ip1),ip2)-max(max(ip0,ip1),ip2);
					img->mv1[index][1]=ip3+ip4+ip5-min(min(ip3,ip4),ip5)-max(max(ip3,ip4),ip5);
				}
		  }
	  }

	  
          ip0=img->mv1[index][0];
          ip1=img->mv1[index][1];
          
        
	
	offset=width*pic_pix_y+pic_pix_x; 
	curr=imgY_org + offset;
	prev=ipol[0] + offset;
#ifndef FAST_INTEGER
	    tmp0=-ip0;
	    tmp1=-ip1;
	    current_inter_sad=(lambda*(img->mv_bituse[absm(tmp0)]+img->mv_bituse[absm(tmp1)]));
	    best_inter_sad[index] = SAD_Macroblock_sub(img,curr, prev, MAX_VALUE,current_inter_sad);
	    //best_inter_sad[index] -= lambda * 16;//is 16x16block,here should not
        mvx0=0;
	    mvy0=0;        
    
 
	pos=0;
	for (i = 1; i < sadTimes; i++)
	{
		sadTbl++;
		x=x_curr+sadTbl->dx;
		y=y_curr+sadTbl->dy;
		//if((x<0)||(y<0)||(x>maxX)||(y>maxY))
		//	continue;

		tmp0=(sadTbl->dx<<2)-ip0;
		tmp1=(sadTbl->dy<<2)-ip1;
		current_inter_sad=(lambda*(img->mv_bituse[absm(tmp0)]+img->mv_bituse[absm(tmp1)]));
		//current_inter_sad+ = SAD_Macroblock_sub(img,prev+sadTbl->offset, curr, best_inter_sad[index],current_inter_sad);
		current_inter_sad = SAD_Macroblock_sub(img,prev+sadTbl->offset, curr, best_inter_sad[index],current_inter_sad);
		if (current_inter_sad < best_inter_sad[index])
		{
			pos=i;
			best_inter_sad[index] = current_inter_sad;
		}
	}
	mvx0 = sadTable[pos].dx;
	mvy0 = sadTable[pos].dy;
	offset+=sadTable[pos].offset;
	x_curr+=mvx0;
	y_curr+=mvy0;

#else
	
	i_min_now = min(maxX,max(-4, x_curr+(ip0>>2)));
	j_min_now = min(maxY,max(-4, y_curr+(ip1>>2)));
	//i_min_now = x_curr;
	//j_min_now = y_curr;
	mvx0=i_min_now-x_curr;
	mvy0=j_min_now-y_curr;
	ioffset=width*mvy0+mvx0;
	tmp0=(mvx0<<2)-ip0;
	tmp1=(mvy0<<2)-ip1;
	current_inter_sad=(lambda*(img->mv_bituse[absm(tmp0)]+img->mv_bituse[absm(tmp1)]));
	best_inter_sad[index] = SAD_Macroblock_sub(img,curr, prev+j_min_now*width+i_min_now-offset, MAX_VALUE,current_inter_sad);
  	
	distortion_0 = distortion_1 = distortion_2 = 65536;
  	prev_AE1[0] = best_inter_sad[index];
  	prev_m1 = -1;
  	for (l = 1; l <= 16; l++){
  		sad_layr = 65536;
  		for (m = 0; m < 4; m++){
  			i = i_min_now + htp[m];
  			j = j_min_now + vtp[m];
  			Coffset = i - x_curr + (j - y_curr)*width;
  			if(l>2&&prev_m1 == m)
  			{
  				current_inter_sad = prev_AE1[l-2];
  				if (current_inter_sad < sad_layr){
  					sad_layr = current_inter_sad;
  					i_min_next = i;
  					j_min_next = j;
  					prev_m2=prevm[m];
  					prev_AE1[l]=current_inter_sad;
  				}
  			}
  			else if (i >= -4 && i <= maxX && j >= -4 && j <= maxY){
  				tmp0=((i - x_curr)<<2)-ip0;
			        tmp1=((j - y_curr)<<2)-ip1;
			        current_inter_sad=(lambda*(img->mv_bituse[IABS(tmp0)]+img->mv_bituse[IABS(tmp1)]));
			        //current_inter_sad += SAD_Macroblock_sub(img,curr, prev+Coffset, sad_layr,current_inter_sad);
					current_inter_sad = SAD_Macroblock_sub(img,curr, prev+Coffset, sad_layr,current_inter_sad);
  				//current_inter_sad = SAD_Macroblock(prev+Coffset, curr, sad_layr);
  				if (current_inter_sad < best_inter_sad[index]){
  					mvx0  = i - x_curr;
  					mvy0  = j - y_curr;
  					ioffset = Coffset;
  					best_inter_sad[index] = current_inter_sad ;
  				}
  				if (current_inter_sad < sad_layr){
  					sad_layr = current_inter_sad;
  					i_min_next = i;
  					j_min_next = j;
  					prev_m2 = prevm[m];
	  				prev_AE1[l] = current_inter_sad;
  				}
  			}
  		}
  		prev_m1 = prev_m2;
  		i_min_now = i_min_next;
  		j_min_now = j_min_next;
  		distortion_2 = distortion_1;
  		distortion_1 = distortion_0;
  		distortion_0 = sad_layr;
  		if (distortion_1 <= distortion_0 && distortion_2 <= distortion_0){
  			break;  			
  		}
  	}
  	offset+=ioffset;
	x_curr+=mvx0;
	y_curr+=mvy0;
#endif
	
	pos=0;
	for (i = 1; i < 9; i++)
	{
		point++;
		x=x_curr+point->x;
		y=y_curr+point->y;
#ifdef FAST_INTEGER
		if((x<-4)||(y<-4)||(x>maxX)||(y>maxY))
			continue;
#endif
		
		tmp0=(mvx0<<2)+point->x-ip0;
	        tmp1=(mvy0<<2)+point->y-ip1;
	        current_inter_sad=(lambda*(img->mv_bituse[absm(tmp0)]+img->mv_bituse[absm(tmp1)]));
		prev=ipol[point->comp] + offset + point->offset;
//		current_inter_sad+=SAD_Macroblock_sub(img,prev,curr,best_inter_sad[index],current_inter_sad);
		current_inter_sad=SAD_Macroblock_sub(img,prev,curr,best_inter_sad[index],current_inter_sad);
		if (current_inter_sad < best_inter_sad[index])
		{
			best_inter_sad[index]=current_inter_sad;
			pos=i;
		}
	}
	mvx1 = search[pos].x;
	mvy1 = search[pos].y;
	offset+=search[pos].offset;
	type  =search[pos].type;
	x_curr+=(mvx1>>1);
	y_curr+=(mvy1>>1);
	
	point=qsearch[type];
	pos=0;
	for (i = 1; i < 9; i++)
	{
		point++;
		x=x_curr+point->x;
		y=y_curr+point->y;
#ifdef FAST_INTEGER
		if((x<-4)||(y<-4)||(x>maxX)||(y>maxY))
			continue;
#endif
			
		tmp0=(mvx0<<2)+mvx1+point->x-ip0;
        tmp1=(mvy0<<2)+mvy1+point->y-ip1;
        current_inter_sad=(lambda*(img->mv_bituse[absm(tmp0)]+img->mv_bituse[absm(tmp1)]));
		prev=ipol[point->comp] + offset + point->offset;
		current_inter_sad=SAD_Macroblock_sub(img,prev,curr,best_inter_sad[index],current_inter_sad);
		if (current_inter_sad < best_inter_sad[index])
		{
			best_inter_sad[index]=current_inter_sad;
			pos=i;
		}
	}
	offset+=qsearch[type][pos].offset;
	img->offset1[index]=offset;
	img->comp1[index]  =qsearch[type][pos].comp;
	curr_mvx1[index]=(mvx0<<2)+mvx1+qsearch[type][pos].x;
	curr_mvy1[index]=(mvy0<<2)+mvy1+qsearch[type][pos].y;
    }
	
	current_inter_sad = best_inter_sad[0]+best_inter_sad[1];
	if(current_inter_sad < MinCost) 
	{
	    img->mb_mode=2;
	    img->imod=INTRA_MB_INTER; 
		for(i=0;i<4;i++)
		{
			tmp_mv[pic_block_y-2][pic_block_x+4+i][0]=curr_mvx1[0];
			tmp_mv[pic_block_y-2][pic_block_x+4+i][1]=curr_mvy1[0];
			tmp_mv[pic_block_y-1][pic_block_x+4+i][0]=curr_mvx1[0];
			tmp_mv[pic_block_y-1][pic_block_x+4+i][1]=curr_mvy1[0];
			tmp_mv[pic_block_y][pic_block_x+4+i][0]=curr_mvx1[1];
			tmp_mv[pic_block_y][pic_block_x+4+i][1]=curr_mvy1[1];
			tmp_mv[pic_block_y+1][pic_block_x+4+i][0]=curr_mvx1[1];
			tmp_mv[pic_block_y+1][pic_block_x+4+i][1]=curr_mvy1[1];
			intra_mb_pos[pic_block_y-2][pic_block_x+4+i] = 0;
			intra_mb_pos[pic_block_y-1][pic_block_x+4+i] = 0;
			intra_mb_pos[pic_block_y][pic_block_x+4+i] = 0;
			intra_mb_pos[pic_block_y+1][pic_block_x+4+i] = 0;
		}
	    MinCost = current_inter_sad;
	} 
          	        
  return MinCost;
}

int motion_search_8x16(struct img_par *img,int MinCost)
{
  int i,index;
  int vec0_x,vec0_y,vec1_x,vec1_y,vec2_x,vec2_y;
  int pic_block_x,pic_block_y,pic_pix_y,pic_pix_x;
  int ip0,ip1,ip2,ip3,ip4,ip5;
  int best_inter_sad[2],current_inter_sad;
  int tmp0,tmp1; 
  int lambda = QP2QUANT[img->qp-SHIFT_QP]; 
  int x,y,offset,type,pos;  
        byte* curr;
	byte* prev;
	Point* point=search;
	SAD_TABLE * sadTbl=sadTable;
	int maxX=img->width-4;
	int maxY=img->height-12;
	int sadTimes=25;
	int mvx0,mvy0,mvx1,mvy1; 
	int x_curr,y_curr;
	int width=img->width+IMG_PAD_SIZE;

#ifdef FAST_INTEGER	
	int prev_AE1[17];
	int prev_m1,prev_m2=0;
	static int prevm[4] = {2, 3, 0, 1};
	static int htp[4] = {-1, 0, 1, 0};
	static int vtp[4] = {0, 1, 0, -1};
	int j_min_now, i_min_now, i_min_next=0, j_min_next=0, sad_layr,Coffset,ioffset=0,l,m,j;
	int distortion_0, distortion_1, distortion_2;
#endif
	
  for(index=0;index<2;index++)
  {
	  point=search;
	  sadTbl=sadTable;
	  pic_block_y=img->block_y;
	  pic_pix_y=img->pix_y;
	        
	  pic_block_x=img->block_x+2*index;
	  pic_pix_x=img->pix_x+8*index;
	
	  x_curr=pic_pix_x;
	  y_curr=pic_pix_y;
	                    
	  if(index==0)
	  {
		  vec0_x=pic_block_x-1;
		  vec0_y=pic_block_y;
		  vec1_x=pic_block_x;
		  vec1_y=pic_block_y-1;
		  vec2_x=pic_block_x+2;
		  vec2_y=pic_block_y-1;
		  if (pic_block_y == 0) 
	          {
	            vec1_x=vec0_x;
	            vec2_x=vec0_x;
	            vec1_y=vec0_y;
	            vec2_y=vec0_y;
	          }                
	          //if (vec2_x > (img->width>>2)-1) 
	          //{
	          //  vec2_x=pic_block_x-1;
	          //}
		  	ip0=tmp_mv[vec0_y][vec0_x+4][0];
			ip1=tmp_mv[vec1_y][vec1_x+4][0];
			ip2=tmp_mv[vec2_y][vec2_x+4][0];
			ip3=tmp_mv[vec0_y][vec0_x+4][1];
			ip4=tmp_mv[vec1_y][vec1_x+4][1];
			ip5=tmp_mv[vec2_y][vec2_x+4][1];
		  	if(!intra_mb_pos[vec0_y][vec0_x+4])
			{
				img->mv2[index][0] = ip0;
				img->mv2[index][1] = ip3;
			}
		  	else
		  	{
		  		if((!intra_mb_pos[vec1_y][vec1_x+4])&&(intra_mb_pos[vec2_y][vec2_x+4]))
				{
					img->mv2[index][0] = ip1;
					img->mv2[index][1] = ip4;
				}
				else if((intra_mb_pos[vec1_y][vec1_x+4])&&(!intra_mb_pos[vec2_y][vec2_x+4]))
				{
					img->mv2[index][0] = ip2;
					img->mv2[index][1] = ip5;
				}
				else
				{
					img->mv2[index][0]=ip0+ip1+ip2-min(min(ip0,ip1),ip2)-max(max(ip0,ip1),ip2);
					img->mv2[index][1]=ip3+ip4+ip5-min(min(ip3,ip4),ip5)-max(max(ip3,ip4),ip5);
				}			  				
		  	}
	  }
	  else
	  {
		  vec0_x=pic_block_x-1;
		  vec0_y=pic_block_y;
		  vec1_x=pic_block_x;
		  vec1_y=pic_block_y-1;
		  vec2_x=pic_block_x+2;
		  vec2_y=pic_block_y-1;
		  if (vec2_x > (img->width>>2)-1) 
		  {
			  vec2_x=pic_block_x-1;
		  }
		  if (pic_block_y == 0) 
		  {
			  //ip0=ip1=ip2=curr_mvx1[0];
			  //ip3=ip4=ip5=curr_mvy1[0];
			  img->mv2[index][0] = curr_mvx2[0];
			img->mv2[index][1] = curr_mvy2[0];
		  }
		  else
		  {                	          		  
					ip0=curr_mvx2[0];
            		ip1=tmp_mv[vec1_y][vec1_x+4][0];
					ip2=tmp_mv[vec2_y][vec2_x+4][0];
					ip3=curr_mvy2[0];
            		ip4=tmp_mv[vec1_y][vec1_x+4][1];
            		ip5=tmp_mv[vec2_y][vec2_x+4][1];
            	  
		        
				  if(!intra_mb_pos[vec2_y][vec2_x+4])
				  {
						img->mv2[index][0] = ip2;
						img->mv2[index][1] = ip5;
				  }
				  else
				  {
					  if(intra_mb_pos[vec1_y][vec1_x+4])
					  {
						img->mv2[index][0] = ip0;
						img->mv2[index][1] = ip3;
					  }
					  else
					  {
						img->mv2[index][0]=ip0+ip1+ip2-min(min(ip0,ip1),ip2)-max(max(ip0,ip1),ip2);
						img->mv2[index][1]=ip3+ip4+ip5-min(min(ip3,ip4),ip5)-max(max(ip3,ip4),ip5);
					  }
				  }
		  }
	  }
          ip0=img->mv2[index][0];
          ip1=img->mv2[index][1];
          
        
	
	offset=width*pic_pix_y+pic_pix_x;  
	curr=imgY_org + offset;
	prev=ipol[0] + offset;
#ifndef FAST_INTEGER	
	    tmp0=-ip0;
	    tmp1=-ip1;
	    current_inter_sad=(lambda*(img->mv_bituse[absm(tmp0)]+img->mv_bituse[absm(tmp1)]));
	    best_inter_sad[index] = SAD_Macroblock_sub2(img,curr, prev, MAX_VALUE,current_inter_sad);
	    //best_inter_sad[index] -= lambda * 16;//is 16x16block,here should not
		mvx0=0;
	    mvy0=0;
        
    
          
	pos=0;
	for (i = 1; i < sadTimes; i++)
	{
		sadTbl++;
		x=x_curr+sadTbl->dx;
		y=y_curr+sadTbl->dy;
		//if((x<0)||(y<0)||(x>maxX)||(y>maxY))
		//	continue;

		tmp0=(sadTbl->dx<<2)-ip0;
		tmp1=(sadTbl->dy<<2)-ip1;
		current_inter_sad=(lambda*(img->mv_bituse[absm(tmp0)]+img->mv_bituse[absm(tmp1)]));
		//current_inter_sad += SAD_Macroblock_sub2(img,prev+sadTbl->offset, curr, best_inter_sad[index],current_inter_sad);
		current_inter_sad = SAD_Macroblock_sub2(img,prev+sadTbl->offset, curr, best_inter_sad[index],current_inter_sad);
		if (current_inter_sad < best_inter_sad[index])
		{
			pos=i;
			best_inter_sad[index] = current_inter_sad;
		}
	}
	mvx0 = sadTable[pos].dx;
	mvy0 = sadTable[pos].dy;
	offset+=sadTable[pos].offset;
	x_curr+=mvx0;
	y_curr+=mvy0; 
#else
	//ioffset=0;
	i_min_now = min(maxX,max(-4, x_curr+(ip0>>2)));
	j_min_now = min(maxY,max(-4, y_curr+(ip1>>2)));
	//i_min_now = x_curr;
	//j_min_now = y_curr;
	mvx0=i_min_now-x_curr;
	mvy0=j_min_now-y_curr;
	ioffset=width*mvy0+mvx0;
	tmp0=(mvx0<<2)-ip0;
	tmp1=(mvy0<<2)-ip1;
	current_inter_sad=(lambda*(img->mv_bituse[absm(tmp0)]+img->mv_bituse[absm(tmp1)]));
	best_inter_sad[index] = SAD_Macroblock_sub2(img,curr, prev+j_min_now*width+i_min_now-offset, MAX_VALUE,current_inter_sad);

  	distortion_0 = distortion_1 = distortion_2 = 65536;
  	prev_AE1[0] = best_inter_sad[index];
  	prev_m1 = -1;
  	for (l = 1; l <= 16; l++){
  		sad_layr = 65536;
  		for (m = 0; m < 4; m++){
  			i = i_min_now + htp[m];
  			j = j_min_now + vtp[m];
  			Coffset = i - x_curr + (j - y_curr)*width;
  			if(l>2&&prev_m1 == m)
  			{
  				current_inter_sad = prev_AE1[l-2];
  				if (current_inter_sad < sad_layr){
  					sad_layr = current_inter_sad;
  					i_min_next = i;
  					j_min_next = j;
  					prev_m2=prevm[m];
  					prev_AE1[l]=current_inter_sad;
  				}
  			}
  			else if (i >= -4 && i <= maxX && j >= -4 && j <= maxY){
  				tmp0=((i - x_curr)<<2)-ip0;
			        tmp1=((j - y_curr)<<2)-ip1;
			        current_inter_sad=(lambda*(img->mv_bituse[IABS(tmp0)]+img->mv_bituse[IABS(tmp1)]));
			        //current_inter_sad += SAD_Macroblock_sub2(img,curr, prev+Coffset, sad_layr,current_inter_sad);
					current_inter_sad = SAD_Macroblock_sub2(img,curr, prev+Coffset, sad_layr,current_inter_sad);
  				//current_inter_sad = SAD_Macroblock(prev+Coffset, curr, sad_layr);
  				if (current_inter_sad < best_inter_sad[index]){
  					mvx0  = i - x_curr;
  					mvy0  = j - y_curr;
  					ioffset = Coffset;
  					best_inter_sad[index] = current_inter_sad ;
  				}
  				if (current_inter_sad < sad_layr){
  					sad_layr = current_inter_sad;
  					i_min_next = i;
  					j_min_next = j;
  					prev_m2 = prevm[m];
	  				prev_AE1[l] = current_inter_sad;
  				}
  			}
  		}
  		prev_m1 = prev_m2;
  		i_min_now = i_min_next;
  		j_min_now = j_min_next;
  		distortion_2 = distortion_1;
  		distortion_1 = distortion_0;
  		distortion_0 = sad_layr;
  		if (distortion_1 <= distortion_0 && distortion_2 <= distortion_0){
  			break;  			
  		}
  	}
  	offset+=ioffset;
	x_curr+=mvx0;
	y_curr+=mvy0;
#endif	
	
	pos=0;
	for (i = 1; i < 9; i++)
	{
		point++;
		x=x_curr+point->x;
		y=y_curr+point->y;
#ifdef FAST_INTEGER
		if((x<-4)||(y<-4)||(x>maxX)||(y>maxY))
			continue;
#endif
		
		tmp0=(mvx0<<2)+point->x-ip0;
		tmp1=(mvy0<<2)+point->y-ip1;
		current_inter_sad=(lambda*(img->mv_bituse[absm(tmp0)]+img->mv_bituse[absm(tmp1)]));
		prev=ipol[point->comp] + offset + point->offset;
		//current_inter_sad+=SAD_Macroblock_sub2(img,prev,curr,best_inter_sad[index],current_inter_sad);
		current_inter_sad = SAD_Macroblock_sub2(img,prev,curr,best_inter_sad[index],current_inter_sad);
		if (current_inter_sad < best_inter_sad[index])
		{
			best_inter_sad[index]=current_inter_sad;
			pos=i;
		}
	}
	mvx1 = search[pos].x;
	mvy1 = search[pos].y;
	offset+=search[pos].offset;
	type  =search[pos].type;
	x_curr+=(mvx1>>1);
	y_curr+=(mvy1>>1);
	
	point=qsearch[type];
	pos=0;
	for (i = 1; i < 9; i++)
	{
		point++;
		x=x_curr+point->x;
		y=y_curr+point->y;
#ifdef FAST_INTEGER
		if((x<-4)||(y<-4)||(x>maxX)||(y>maxY))
			continue;
#endif
			
		tmp0=(mvx0<<2)+mvx1+point->x-ip0;
        tmp1=(mvy0<<2)+mvy1+point->y-ip1;
        current_inter_sad=(lambda*(img->mv_bituse[absm(tmp0)]+img->mv_bituse[absm(tmp1)]));
		prev=ipol[point->comp] + offset + point->offset;
		current_inter_sad = SAD_Macroblock_sub2(img,prev,curr,best_inter_sad[index],current_inter_sad);
		if (current_inter_sad < best_inter_sad[index])
		{
			best_inter_sad[index]=current_inter_sad;
			pos=i;
		}
	}
	offset+=qsearch[type][pos].offset;
	img->offset2[index]=offset;
	img->comp2[index]  =qsearch[type][pos].comp;
	curr_mvx2[index]=(mvx0<<2)+mvx1+qsearch[type][pos].x;
	curr_mvy2[index]=(mvy0<<2)+mvy1+qsearch[type][pos].y;
    }
	
	current_inter_sad = best_inter_sad[0]+best_inter_sad[1];
	if(current_inter_sad < MinCost) 
	{
	    img->mb_mode=3;
	    img->imod=INTRA_MB_INTER; 
	   
		for(i=0;i<4;i++)
		{
			tmp_mv[pic_block_y+i][pic_block_x+2][0]=curr_mvx2[0];
			tmp_mv[pic_block_y+i][pic_block_x+2][1]=curr_mvy2[0];
			tmp_mv[pic_block_y+i][pic_block_x+3][0]=curr_mvx2[0];
			tmp_mv[pic_block_y+i][pic_block_x+3][1]=curr_mvy2[0];
			tmp_mv[pic_block_y+i][pic_block_x+4][0]=curr_mvx2[1];
			tmp_mv[pic_block_y+i][pic_block_x+4][1]=curr_mvy2[1];
			tmp_mv[pic_block_y+i][pic_block_x+5][0]=curr_mvx2[1];
			tmp_mv[pic_block_y+i][pic_block_x+5][1]=curr_mvy2[1];
			intra_mb_pos[pic_block_y+i][pic_block_x+2] = 0;
			intra_mb_pos[pic_block_y+i][pic_block_x+3] = 0;
			intra_mb_pos[pic_block_y+i][pic_block_x+4] = 0;
			intra_mb_pos[pic_block_y+i][pic_block_x+5] = 0;
		}
	    MinCost = current_inter_sad;
	} 
          	        
  return MinCost;
}

int motion_search(struct img_par *img,int tot_intra_sad)
{
  int i;
  int vec0_x,vec0_y,vec1_x,vec1_y,vec2_x,vec2_y;
  int pic_block_x,pic_block_y,pic_pix_y,pic_pix_x;
  int ip0,ip1,ip2,ip3,ip4,ip5;
  int best_inter_sad,current_inter_sad,skip_sad=65536,skip_offset;
  int tmp0,tmp1; 
  int lambda = QP2QUANT[img->qp-SHIFT_QP]; 
  int x,y,offset,type,pos,skipcomp;  
        byte* curr;
	byte* prev;
	Point* point=search;
	SAD_TABLE * sadTbl=sadTable;
	int maxX=img->width-12;
	int maxY=img->height-12;
	int sadTimes=25;
	int mvx0=0,mvy0=0,mvx1,mvy1; 
	int x_curr,y_curr;
	int width=img->width+IMG_PAD_SIZE;

#ifdef FAST_INTEGER	
	int prev_AE1[17];
	int prev_m1,prev_m2=0;
	static int prevm[4] = {2, 3, 0, 1};
	static int htp[4] = {-1, 0, 1, 0};
	static int vtp[4] = {0, 1, 0, -1};
	int j_min_now, i_min_now,i_min_next=0, j_min_next=0, Coffset,sad_layr,ioffset=0,l,m,j;
	int distortion_0, distortion_1, distortion_2;
#endif    
	
  pic_block_y=img->block_y;
  pic_pix_y=img->pix_y;
        
  pic_block_x=img->block_x;
  pic_pix_x=img->pix_x;

  x_curr=pic_pix_x;
  y_curr=pic_pix_y;
                    
  vec0_x=pic_block_x-1;		/*left block*/
  vec0_y=pic_block_y;
  vec1_x=pic_block_x;
  vec1_y=pic_block_y-1;		/*up block*/
  vec2_x=pic_block_x+4;
  vec2_y=pic_block_y-1;		/*upright block*/
                    
          if (pic_block_y == 0) /*current block has no up blocks, reset refference block*/
          {
            vec1_x=vec0_x;
            vec2_x=vec0_x;
            vec1_y=vec0_y;
            vec2_y=vec0_y;
          }                
          if (vec2_x > (img->width>>2)-1) /*current block has no upright block,reset refference block*/
          {
            vec2_x=pic_block_x-1;
          }
          
          
            ip0=tmp_mv[vec0_y][vec0_x+4][0];
            ip1=tmp_mv[vec1_y][vec1_x+4][0];
            ip2=tmp_mv[vec2_y][vec2_x+4][0];
			ip3=tmp_mv[vec0_y][vec0_x+4][1];
            ip4=tmp_mv[vec1_y][vec1_x+4][1];
            ip5=tmp_mv[vec2_y][vec2_x+4][1];
			if((!intra_mb_pos[vec0_y][vec0_x+4])&&(intra_mb_pos[vec1_y][vec1_x+4])&&(intra_mb_pos[vec2_y][vec2_x+4]))
			{
				img->mv[0] = ip0;
				img->mv[1] = ip3;
			}
			else if((intra_mb_pos[vec0_y][vec0_x+4])&&(!intra_mb_pos[vec1_y][vec1_x+4])&&(intra_mb_pos[vec2_y][vec2_x+4]))
			{
				img->mv[0] = ip1;
				img->mv[1] = ip4;
			}
			else if((intra_mb_pos[vec0_y][vec0_x+4])&&(intra_mb_pos[vec1_y][vec1_x+4])&&(!intra_mb_pos[vec2_y][vec2_x+4]))
			{
				img->mv[0] = ip2;
				img->mv[1] = ip5;
			}
			else
			{
				img->mv[0]=ip0+ip1+ip2-min(min(ip0,ip1),ip2)-max(max(ip0,ip1),ip2);
				img->mv[1]=ip3+ip4+ip5-min(min(ip3,ip4),ip5)-max(max(ip3,ip4),ip5);
			}
          ip0=img->mv[0];
          ip1=img->mv[1];

		  //by guoxq
		  if(pic_pix_y==0 || pic_pix_x==0)
		  {
			  skip_mv[0]=0;
			  skip_mv[1]=0;
		  }
		  else if(tmp_mv[vec0_y][vec0_x+4][0]==0 && tmp_mv[vec0_y][vec0_x+4][1]==0 && (!intra_mb_pos[vec0_y][vec0_x+4]))
		  {
			  skip_mv[0]=0;
			  skip_mv[1]=0;
		  }
		  else if(tmp_mv[vec1_y][vec1_x+4][0]==0 && tmp_mv[vec1_y][vec1_x+4][1]==0 && (!intra_mb_pos[vec1_y][vec1_x+4]))
		  {
			  skip_mv[0]=0;
			  skip_mv[1]=0;
		  }
		  else
		  {
			  skip_mv[0]=ip0;
			  skip_mv[1]=ip1;
		  }
    
	img->bSkipMode=0;
	offset=width*pic_pix_y+pic_pix_x;  
    curr=imgY_org + offset; 
    x=pic_pix_x+(skip_mv[0]>>2)+(((skip_mv[0]>>1)|skip_mv[0])&1);
    y=pic_pix_y+(skip_mv[1]>>2)+(((skip_mv[1]>>1)|skip_mv[1])&1);             
	if((x>=-4)&&(y>=-4)&&(x<=maxX)&&(y<=maxY))//16ʾģʽ116x16
	{
		skipcomp=(skip_mv[1]&3)*4+(skip_mv[0]&3);
		skip_offset=(pic_pix_y+(skip_mv[1]>>2))*width + pic_pix_x+(skip_mv[0]>>2);
		prev=ipol[skipcomp] + skip_offset;
		skip_sad=SAD_Macroblock(img,curr,prev,MAX_VALUE,0);
		skip_sad -= 8*lambda; 
		if (skip_sad <= 10*lambda)
		{
			img->bSkipMode=1;
			img->offset=skip_offset;
			img->comp  =skipcomp;
			best_inter_sad=skip_sad;
			curr_mvx=skip_mv[0];
			curr_mvy=skip_mv[1];
			img->mb_mode=1;
			img->imod=INTRA_MB_INTER;   
			for(i=0;i<4;i++)
			{
				tmp_mv[pic_block_y][pic_block_x+4+i][0]=curr_mvx;
				tmp_mv[pic_block_y][pic_block_x+4+i][1]=curr_mvy;
				tmp_mv[pic_block_y+1][pic_block_x+4+i][0]=curr_mvx;
				tmp_mv[pic_block_y+1][pic_block_x+4+i][1]=curr_mvy;
				tmp_mv[pic_block_y+2][pic_block_x+4+i][0]=curr_mvx;
				tmp_mv[pic_block_y+2][pic_block_x+4+i][1]=curr_mvy;
				tmp_mv[pic_block_y+3][pic_block_x+4+i][0]=curr_mvx;
				tmp_mv[pic_block_y+3][pic_block_x+4+i][1]=curr_mvy;
				intra_mb_pos[pic_block_y][pic_block_x+4+i] = 0;
				intra_mb_pos[pic_block_y+1][pic_block_x+4+i] = 0;
				intra_mb_pos[pic_block_y+2][pic_block_x+4+i] = 0;
				intra_mb_pos[pic_block_y+3][pic_block_x+4+i] = 0;
			}
			return best_inter_sad;
		}
	}    

	prev=ipol[0] + offset;  
    tmp0=-ip0;
    tmp1=-ip1;
    current_inter_sad=(lambda*(img->mv_bituse[absm(tmp0)]+img->mv_bituse[absm(tmp1)]));
    best_inter_sad = SAD_Macroblock(img,curr, prev, MAX_VALUE,current_inter_sad);
    best_inter_sad -= lambda * 16;

#ifndef FAST_INTEGER          
	pos=0;
	for (i = 1; i < sadTimes; i++)
	{
		sadTbl++;
		x=x_curr+sadTbl->dx;
		y=y_curr+sadTbl->dy;

		tmp0=((mvx0+sadTbl->dx)<<2)-ip0;
        tmp1=((mvy0+sadTbl->dy)<<2)-ip1;
        current_inter_sad=(lambda*(img->mv_bituse[absm(tmp0)]+img->mv_bituse[absm(tmp1)]));
//		current_inter_sad += SAD_Macroblock(img,prev+sadTbl->offset, curr, best_inter_sad,current_inter_sad);
		current_inter_sad = SAD_Macroblock(img,prev+sadTbl->offset, curr, best_inter_sad,current_inter_sad);
		if (current_inter_sad < best_inter_sad)
		{
			pos=i;
			best_inter_sad = current_inter_sad;
		}
	}
	mvx0 = sadTable[pos].dx;
	mvy0 = sadTable[pos].dy;
	offset+=sadTable[pos].offset;
	x_curr+=mvx0;
	y_curr+=mvy0;
	
#else
	i_min_now = min(maxX,max(-4, x_curr+(ip0>>2)));
	j_min_now = min(maxY,max(-4, y_curr+(ip1>>2)));

	/*full pixel motion estimation*/
  	distortion_0 = distortion_1 = distortion_2 = 65536;
  	prev_AE1[0] = best_inter_sad;
  	prev_m1 = -1;
  	for (l = 1; l <= 16; l++){
  		sad_layr = 65536;
  		for (m = 0; m < 4; m++){
  			i = i_min_now + htp[m];
  			j = j_min_now + vtp[m];
  			Coffset = i - x_curr + (j - y_curr)*width;
  			if(l>2&&prev_m1 == m)
  			{
  				current_inter_sad = prev_AE1[l-2];
  				if (current_inter_sad < sad_layr){
  					sad_layr = current_inter_sad;
  					i_min_next = i;
  					j_min_next = j;
  					prev_m2=prevm[m];
  					prev_AE1[l]=current_inter_sad;
  				}
  			}
  			else if (i >= -4 && i <= maxX && j >= -4 && j <= maxY){
  				tmp0=((i - x_curr)<<2)-ip0;
			        tmp1=((j - y_curr)<<2)-ip1;
			        current_inter_sad=(lambda*(img->mv_bituse[IABS(tmp0)]+img->mv_bituse[IABS(tmp1)]));
//			        current_inter_sad += SAD_Macroblock(img,curr, prev+Coffset, sad_layr,current_inter_sad);
			        current_inter_sad = SAD_Macroblock(img,curr, prev+Coffset, sad_layr,current_inter_sad);
  				if (current_inter_sad < best_inter_sad){
  					mvx0  = i - x_curr;
  					mvy0  = j - y_curr;
  					ioffset = Coffset;
  					best_inter_sad = current_inter_sad ;
  				}
  				if (current_inter_sad < sad_layr){
  					sad_layr = current_inter_sad;
  					i_min_next = i;
  					j_min_next = j;
  					prev_m2 = prevm[m];
	  				prev_AE1[l] = current_inter_sad;
  				}
  			}
  		}
  		prev_m1 = prev_m2;
  		i_min_now = i_min_next;
  		j_min_now = j_min_next;
  		distortion_2 = distortion_1;
  		distortion_1 = distortion_0;
  		distortion_0 = sad_layr;
  		if (distortion_1 <= distortion_0 && distortion_2 <= distortion_0){
  			break;  			
  		}
  	}
  	offset+=ioffset;
	x_curr+=mvx0;
	y_curr+=mvy0;

#endif	
   /*********************************
   *****                       *****
   *****  HALF-PEL REFINEMENT  *****
   *****                       *****
   *********************************/
	pos=0;
	for (i = 1; i < 9; i++)
	{
		point++;
		x=x_curr+point->x;
		y=y_curr+point->y;
#ifdef FAST_INTEGER
		if((x<-4)||(y<-4)||(x>maxX)||(y>maxY))
			continue;
#endif
		
		tmp0=(mvx0<<2)+point->x-ip0;
        tmp1=(mvy0<<2)+point->y-ip1;
        current_inter_sad=(lambda*(img->mv_bituse[absm(tmp0)]+img->mv_bituse[absm(tmp1)]));
		prev=ipol[point->comp] + offset + point->offset;
//		current_inter_sad+=SAD_Macroblock(img,prev,curr,best_inter_sad,current_inter_sad);		
		current_inter_sad=SAD_Macroblock(img,prev,curr,best_inter_sad,current_inter_sad);
		if (current_inter_sad < best_inter_sad)
		{
			best_inter_sad=current_inter_sad;
			pos=i;
		}
	}
	mvx1 = search[pos].x;
	mvy1 = search[pos].y;
	offset+=search[pos].offset;
	type  =search[pos].type;
	x_curr+=(mvx1>>1);
	y_curr+=(mvy1>>1);

	
   /************************************
   *****                          *****
   *****  QUARTER-PEL REFINEMENT  *****
   *****                          *****
   ************************************/
	point=qsearch[type];
	pos=0;
	for (i = 1; i < 9; i++)
	{
		point++;
		x=x_curr+point->x;
		y=y_curr+point->y;
#ifdef FAST_INTEGER
		if((x<-4)||(y<-4)||(x>maxX)||(y>maxY))
			continue;
#endif
			
		tmp0=(mvx0<<2)+mvx1+point->x-ip0;
        tmp1=(mvy0<<2)+mvy1+point->y-ip1;
        current_inter_sad=(lambda*(img->mv_bituse[absm(tmp0)]+img->mv_bituse[absm(tmp1)]));
		prev=ipol[point->comp] + offset + point->offset;
		current_inter_sad=SAD_Macroblock(img,prev,curr,best_inter_sad,current_inter_sad);
		if (current_inter_sad < best_inter_sad)
		{
			best_inter_sad=current_inter_sad;
			pos=i;
		}
	}
	offset+=qsearch[type][pos].offset;
	img->offset=offset;
	img->comp  =qsearch[type][pos].comp;
	curr_mvx=(mvx0<<2)+mvx1+qsearch[type][pos].x;
	curr_mvy=(mvy0<<2)+mvy1+qsearch[type][pos].y;
	
	if (skip_sad <= best_inter_sad)
	{
		img->bSkipMode=1;
		img->offset=skip_offset;
		img->comp  =skipcomp;
		best_inter_sad=skip_sad;
		curr_mvx=skip_mv[0];
		curr_mvy=skip_mv[1];
	}
	
	if (tot_intra_sad < best_inter_sad) 
	{
	    img->mb_mode=img->imod+8*img->type; 
		for(i=0;i<4;i++)
		{
			tmp_mv[pic_block_y][pic_block_x+4+i][0]=0;
			tmp_mv[pic_block_y][pic_block_x+4+i][1]=0;
			tmp_mv[pic_block_y+1][pic_block_x+4+i][0]=0;
			tmp_mv[pic_block_y+1][pic_block_x+4+i][1]=0;
			tmp_mv[pic_block_y+2][pic_block_x+4+i][0]=0;
			tmp_mv[pic_block_y+2][pic_block_x+4+i][1]=0;
			tmp_mv[pic_block_y+3][pic_block_x+4+i][0]=0;
			tmp_mv[pic_block_y+3][pic_block_x+4+i][1]=0;
			intra_mb_pos[pic_block_y][pic_block_x+4+i] = 1;
			intra_mb_pos[pic_block_y+1][pic_block_x+4+i] = 1;
			intra_mb_pos[pic_block_y+2][pic_block_x+4+i] = 1;
			intra_mb_pos[pic_block_y+3][pic_block_x+4+i] = 1;
		}
	    best_inter_sad = tot_intra_sad;
	} 
	else 
	{
		img->mb_mode=1;
	    img->imod=INTRA_MB_INTER;   
		for(i=0;i<4;i++)
		{
			tmp_mv[pic_block_y][pic_block_x+4+i][0]=curr_mvx;
			tmp_mv[pic_block_y][pic_block_x+4+i][1]=curr_mvy;
			tmp_mv[pic_block_y+1][pic_block_x+4+i][0]=curr_mvx;
			tmp_mv[pic_block_y+1][pic_block_x+4+i][1]=curr_mvy;
			tmp_mv[pic_block_y+2][pic_block_x+4+i][0]=curr_mvx;
			tmp_mv[pic_block_y+2][pic_block_x+4+i][1]=curr_mvy;
			tmp_mv[pic_block_y+3][pic_block_x+4+i][0]=curr_mvx;
			tmp_mv[pic_block_y+3][pic_block_x+4+i][1]=curr_mvy;
			intra_mb_pos[pic_block_y][pic_block_x+4+i] = 0;
			intra_mb_pos[pic_block_y+1][pic_block_x+4+i] = 0;
			intra_mb_pos[pic_block_y+2][pic_block_x+4+i] = 0;
			intra_mb_pos[pic_block_y+3][pic_block_x+4+i] = 0;
		}
	}
	return best_inter_sad;
          	        
}

int Intra_Sad(struct img_par *img,byte *ii, int *pp)
{
	int i,sad=0;
	int x0,x1,x2,x3;
	int y0,y1,y2,y3;
	int lx=img->width+IMG_PAD_SIZE;
	for(i=0;i<4;i++)
	{
		x0=ii[0];	y0=pp[0];
		x1=ii[1];	y1=pp[1];
		x2=ii[2];	y2=pp[2];
		x3=ii[3];	y3=pp[3];
		sad+=IABS(x0-y0)+IABS(x1-y1)+IABS(x2-y2)+IABS(x3-y3);
		ii+=lx;	pp+=4;
	}
	return sad;
}

int IntraMB_Sad(struct img_par *img,byte *ii, int *pp)
{
	int sad=0;
	int i,j;
	int mm,jj;
	int current_intra_sad_2;
	int lx=img->width+IMG_PAD_SIZE;
	int M1[16][16],M0[4][4][4][4],M3[4],M4[4][4];
	for (j=0;j<16;j++)
	{
		jj=j>>2;
        for (i=0;i<16;i+=4)
        {
          mm=i>>2;
          M1[i][j]  =ii[i]  -pp[i];		  M1[i+1][j]=ii[i+1]-pp[i+1];
		  M1[i+2][j]=ii[i+2]-pp[i+2];		  M1[i+3][j]=ii[i+3]-pp[i+3];
          M0[0][mm][j&0x03][jj]=  M1[i][j];
		  M0[1][mm][j&0x03][jj]=M1[i+1][j];
		  M0[2][mm][j&0x03][jj]=M1[i+2][j];
		  M0[3][mm][j&0x03][jj]=M1[i+3][j];
        }
		ii+=lx;	pp+=16;
	}

	current_intra_sad_2=0;	// no SAD start handicap here
	for (jj=0;jj<4;jj++)
	{
        for (mm=0;mm<4;mm++)
        {
			for (j=0;j<4;j++)
			{
				M3[0]=M0[0][mm][j][jj]+M0[3][mm][j][jj];
				M3[1]=M0[1][mm][j][jj]+M0[2][mm][j][jj];
				M3[2]=M0[1][mm][j][jj]-M0[2][mm][j][jj];
				M3[3]=M0[0][mm][j][jj]-M0[3][mm][j][jj];
				
				M0[0][mm][j][jj]=M3[0]+M3[1];
				M0[2][mm][j][jj]=M3[0]-M3[1];
				M0[1][mm][j][jj]=M3[2]+M3[3];
				M0[3][mm][j][jj]=M3[3]-M3[2];
			}
			
			for (i=0;i<4;i++)
			{
				M3[0]=M0[i][mm][0][jj]+M0[i][mm][3][jj];
				M3[1]=M0[i][mm][1][jj]+M0[i][mm][2][jj];
				M3[2]=M0[i][mm][1][jj]-M0[i][mm][2][jj];
				M3[3]=M0[i][mm][0][jj]-M0[i][mm][3][jj];
				
				M0[i][mm][0][jj]=M3[0]+M3[1];
				M0[i][mm][2][jj]=M3[0]-M3[1];
				M0[i][mm][1][jj]=M3[2]+M3[3];
				M0[i][mm][3][jj]=M3[3]-M3[2];
			}	
			current_intra_sad_2 +=					   abs(M0[1][mm][0][jj])
									  +abs(M0[2][mm][0][jj])+abs(M0[3][mm][0][jj])
								+abs(M0[0][mm][1][jj])+abs(M0[1][mm][1][jj])
									  +abs(M0[2][mm][1][jj])+abs(M0[3][mm][1][jj])
								+abs(M0[0][mm][2][jj])+abs(M0[1][mm][2][jj])
									  +abs(M0[2][mm][2][jj])+abs(M0[3][mm][2][jj])
								+abs(M0[0][mm][3][jj])+abs(M0[1][mm][3][jj])
									  +abs(M0[2][mm][3][jj])+abs(M0[3][mm][3][jj]);

        }
	}

	M4[0][0]=M0[0][0][0][0]/4;	M4[1][0]=M0[0][1][0][0]/4;
	M4[2][0]=M0[0][2][0][0]/4;	M4[3][0]=M0[0][3][0][0]/4;
	M4[0][1]=M0[0][0][0][1]/4;	M4[1][1]=M0[0][1][0][1]/4;
	M4[2][1]=M0[0][2][0][1]/4;	M4[3][1]=M0[0][3][0][1]/4;
	M4[0][2]=M0[0][0][0][2]/4;	M4[1][2]=M0[0][1][0][2]/4;
	M4[2][2]=M0[0][2][0][2]/4;	M4[3][2]=M0[0][3][0][2]/4;
	M4[0][3]=M0[0][0][0][3]/4;	M4[1][3]=M0[0][1][0][3]/4;
	M4[2][3]=M0[0][2][0][3]/4;	M4[3][3]=M0[0][3][0][3]/4;

    // Hadamard of DC koeff
    for (j=0;j<4;j++)
	{
          M3[0]=M4[0][j]+M4[3][j];
          M3[1]=M4[1][j]+M4[2][j];
          M3[2]=M4[1][j]-M4[2][j];
          M3[3]=M4[0][j]-M4[3][j];

          M4[0][j]=M3[0]+M3[1];
          M4[2][j]=M3[0]-M3[1];
          M4[1][j]=M3[2]+M3[3];
          M4[3][j]=M3[3]-M3[2];
	}

	for (i=0;i<4;i++)
	{
          M3[0]=M4[i][0]+M4[i][3];
          M3[1]=M4[i][1]+M4[i][2];
          M3[2]=M4[i][1]-M4[i][2];
          M3[3]=M4[i][0]-M4[i][3];

          M4[i][0]=M3[0]+M3[1];
          M4[i][2]=M3[0]-M3[1];
          M4[i][1]=M3[2]+M3[3];
          M4[i][3]=M3[3]-M3[2];
	}
	current_intra_sad_2 += abs(M4[0][0])+abs(M4[1][0])+abs(M4[2][0])+abs(M4[3][0])
							+abs(M4[0][1])+abs(M4[1][1])+abs(M4[2][1])+abs(M4[3][1])
							+abs(M4[0][2])+abs(M4[1][2])+abs(M4[2][2])+abs(M4[3][2])
							+abs(M4[0][3])+abs(M4[1][3])+abs(M4[2][3])+abs(M4[3][3]);
	return current_intra_sad_2/2;
}

/************************************************************************
*
*  Routine:     dct_luma
*
*  Description: The routine performs transform,quantization,inverse transform, adds the diff. 
*               to the prediction and writes the result to the decoded luma frame. Includes the
*               RD constrained quantization also.
*               
*               
*  Input:       block_x,block_y: Block position inside a macro block (0,4,8,12).              
*
*  Output:      nonzero: 0 if no levels are nonzero.  1 if there are nonzero levels.
*               coeff_cost: Counter for nonzero coefficients, used to discard expencive levels.
*
*                    
************************************************************************/

#define HOR_DCT(x0,x1,x2,x3)\
		D0=x0+x3;\
		D3=x0-x3;\
		D1=x1+x2;\
		D2=x1-x2;\
		x0=D0+D1;\
		x2=D0-D1;\
		x1=D2+(D3<<1);\
		x3=D3-(D2<<1);

#define VER_DCT(x0,x1,x2,x3)\
		q0=x0+x3;\
		q3=x0-x3;\
		q1=x1+x2;\
		q2=x1-x2;\
		p0=q0+q1;\
		p2=q0-q1;\
		p1=q2+(q3<<1);\
		p3=q3-(q2<<1);

#define HOR_IDCT(x0,x1,x2,x3)\
		q0=x0+x2;\
		q1=x0-x2;\
		q2=(x1>>1)-x3;\
		q3=x1+(x3>>1);\
		p0=q0+q3;\
		p3=q0-q3;\
		p1=q1+q2;\
		p2=q1-q2;

#define VER_IDCT(x0,x1,x2,x3)\
		q0=x0+x2;\
		q1=x0-x2;\
		q2=(x1>>1)-x3;\
		q3=x1+(x3>>1);\
		x0=q0+q3;\
		x3=q0-q3;\
		x1=q1+q2;\
		x2=q1-q2;

/*yummy*/
#define VER_CVT(x0,x1,x2,x3)\
		q0=x0+x3;\
		q3=x0-x3;\
		q1=x1+x2;\
		q2=x1-x2;\
		p0=q0+q1;\
		p2=q0-q1;\
		p1=q2+q3;\
		p3=q3-q2;

#define HOR_CVT(x0,x1,x2,x3)\
		D0=x0+x3;\
		D3=x0-x3;\
		D1=x1+x2;\
		D2=x1-x2;\
		x0=D0+D1;\
		x2=D0-D1;\
		x1=D2+D3;\
		x3=D3-D2;

#define HOR_ICVT(x0,x1,x2,x3)\
		q0=x0+x2;\
		q1=x0-x2;\
		q2=x1-x3;\
		q3=x1+x3;\
		p0=q0+q3;\
		p3=q0-q3;\
		p1=q1+q2;\
		p2=q1-q2;

#define VER_ICVT(x0,x1,x2,x3)\
		q0=x0+x2;\
		q1=x0-x2;\
		q2=x1-x3;\
		q3=x1+x3;\
		x0=q0+q3;\
		x3=q0-q3;\
		x1=q1+q2;\
		x2=q1-q2;

#define IDCT_SHIFT(x,y) min(255,max(0,((x)+((y) <<DQ_BITS)+DQ_ROUND)>>DQ_BITS))

int dct_luma(byte *ii, int *pp, int block_x,int block_y,struct img_par *img)
{ 
  int sign(int a,int b);
 
  int j,ilev,coeff_ctr;
  int qp_const,level,scan_pos,run;
  int nonzero;
  int qp_per,qp_rem,q_bits;

  int   pos_x   = block_x/BLOCK_SIZE;
  int   pos_y   = block_y/BLOCK_SIZE;

  int p0,p1,p2,p3;
  int q0,q1,q2,q3;
  int A0,A1,A2,A3;
  int B0,B1,B2,B3;
  int C0,C1,C2,C3;
  int D0,D1,D2,D3;
  int width=img->width+IMG_PAD_SIZE;
  int outD[16];

  qp_per    = (img->qp-MIN_QP)/6;
  qp_rem    = (img->qp-MIN_QP)%6;
  q_bits    = Q_BITS+qp_per;

  if (img->type == INTRA_IMG)
    qp_const=(1<<q_bits)/3;    // intra
  else
    qp_const=(1<<q_bits)/6;    // inter
  
  p0=ii[0]-pp[0];
  p1=ii[1]-pp[1];
  p2=ii[2]-pp[2];
  p3=ii[3]-pp[3];
  HOR_DCT(p0,p1,p2,p3);
  A0=p0;A1=p1;A2=p2;A3=p3;
  ii+=width;pp+=4;

  p0=ii[0]-pp[0];
  p1=ii[1]-pp[1];
  p2=ii[2]-pp[2];
  p3=ii[3]-pp[3];
  HOR_DCT(p0,p1,p2,p3);
  B0=p0;B1=p1;B2=p2;B3=p3;
  ii+=width;pp+=4;
  
  p0=ii[0]-pp[0];
  p1=ii[1]-pp[1];
  p2=ii[2]-pp[2];
  p3=ii[3]-pp[3];
  HOR_DCT(p0,p1,p2,p3);
  C0=p0;C1=p1;C2=p2;C3=p3;
  ii+=width;pp+=4;
  
  p0=ii[0]-pp[0];
  p1=ii[1]-pp[1];
  p2=ii[2]-pp[2];
  p3=ii[3]-pp[3];
  HOR_DCT(p0,p1,p2,p3);
  D0=p0;D1=p1;D2=p2;D3=p3;
  pp-=12;
  
  VER_DCT(A0,B0,C0,D0);
  outD[0]=p0;outD[2]=p1;outD[3]=p2;outD[9]=p3;
  VER_DCT(A1,B1,C1,D1);
  outD[1]=p0;outD[4]=p1;outD[8]=p2;outD[10]=p3;
  VER_DCT(A2,B2,C2,D2);
  outD[5]=p0;outD[7]=p1;outD[11]=p2;outD[14]=p3;
  VER_DCT(A3,B3,C3,D3);
  outD[6]=p0;outD[12]=p1;outD[13]=p2;outD[15]=p3;
  // Quant

  nonzero=FALSE;

  run=-1;
  scan_pos=0;
  
  for (coeff_ctr=0;coeff_ctr < 16;coeff_ctr++)
  { 
    p0=outD[coeff_ctr];
    run++;
    ilev=0;
    level = (abs (p0) * quant_coef1[qp_rem][coeff_ctr] + qp_const) >> q_bits;
    if (level != 0)
    {
      nonzero=TRUE;
	  img->cof[pos_x][pos_y][scan_pos][0]=sign(level,p0);
      img->cof[pos_x][pos_y][scan_pos][1]=run;
      ++scan_pos;
      run=-1;                     // reset zero level counter
      ilev=level*dequant_coef1[qp_rem][coeff_ctr]<<qp_per;
    }
    outD[coeff_ctr]=sign(ilev,p0);
  }
  img->cof[pos_x][pos_y][scan_pos][0]=0;

  p0=outD[0];p1=outD[2];p2=outD[3];p3=outD[9];
  VER_IDCT(p0,p1,p2,p3);
  A0=p0;A1=p1;A2=p2;A3=p3;
  p0=outD[1];p1=outD[4];p2=outD[8];p3=outD[10];
  VER_IDCT(p0,p1,p2,p3);
  B0=p0;B1=p1;B2=p2;B3=p3;
  p0=outD[5];p1=outD[7];p2=outD[11];p3=outD[14];
  VER_IDCT(p0,p1,p2,p3);
  C0=p0;C1=p1;C2=p2;C3=p3;
  p0=outD[6];p1=outD[12];p2=outD[13];p3=outD[15];
  VER_IDCT(p0,p1,p2,p3);
  D0=p0;D1=p1;D2=p2;D3=p3;
  
  
  HOR_IDCT(A0,B0,C0,D0);
  outD[0]=IDCT_SHIFT(p0,pp[0]);
  outD[1]=IDCT_SHIFT(p1,pp[1]);
  outD[2]=IDCT_SHIFT(p2,pp[2]);
  outD[3]=IDCT_SHIFT(p3,pp[3]);
  HOR_IDCT(A1,B1,C1,D1);
  outD[4]=IDCT_SHIFT(p0,pp[4]);
  outD[5]=IDCT_SHIFT(p1,pp[5]);
  outD[6]=IDCT_SHIFT(p2,pp[6]);
  outD[7]=IDCT_SHIFT(p3,pp[7]);
  HOR_IDCT(A2,B2,C2,D2);
  outD[8]=IDCT_SHIFT(p0,pp[8]);
  outD[9]=IDCT_SHIFT(p1,pp[9]);
  outD[10]=IDCT_SHIFT(p2,pp[10]);
  outD[11]=IDCT_SHIFT(p3,pp[11]);
  HOR_IDCT(A3,B3,C3,D3);
  outD[12]=IDCT_SHIFT(p0,pp[12]);
  outD[13]=IDCT_SHIFT(p1,pp[13]);
  outD[14]=IDCT_SHIFT(p2,pp[14]);
  outD[15]=IDCT_SHIFT(p3,pp[15]);
  
  for (j=0; j < BLOCK_SIZE; j++) 
  {
	  p0=(img->pix_y+block_y+j)*width+img->pix_x+block_x;
	  p1=j*4;
      imgY[p0  ]=outD[p1  ];
	  imgY[p0+1]=outD[p1+1];
	  imgY[p0+2]=outD[p1+2];
	  imgY[p0+3]=outD[p1+3];
  }
  return nonzero;
}

/*yummy*/
int dct_luma_16x16(byte *mm,int new_intra_mode,struct img_par *img)
{
  int lx=img->width+IMG_PAD_SIZE;
  int qp_const;
  int i,j;
  int ii,jj;
  int M1[16][16];
  int M4[4][4];
  int M0[4][4][4][4];
  int run,scan_pos,coeff_ctr,level;
  int qp_per,qp_rem,q_bits;
  int ac_coef = 0;
  int incr=1;
  int offset=0;
  int ilev,pp;

  int p0,p1,p2,p3;
  int q0,q1,q2,q3;
  int A0,A1,A2,A3;
  int B0,B1,B2,B3;
  int C0,C1,C2,C3;
  int D0,D1,D2,D3;
  int outD[16];

  qp_per    = (img->qp-MIN_QP)/6;
  qp_rem    = (img->qp-MIN_QP)%6;
  q_bits    = Q_BITS+qp_per;
  qp_const  = (1<<q_bits)/3;

  for (j=0;j<16;j++)
  {
	  jj=j>>2;
	  for (i=0;i<16;i+=4)
	  {
		  ii=i>>2;
		  M1[i][j]	=mm[i]	-img->mprr_2[new_intra_mode][i	+j*16];		  
		  M1[i+1][j]=mm[i+1]-img->mprr_2[new_intra_mode][i+1+j*16];		  
		  M1[i+2][j]=mm[i+2]-img->mprr_2[new_intra_mode][i+2+j*16];		  
		  M1[i+3][j]=mm[i+3]-img->mprr_2[new_intra_mode][i+3+j*16];
		  M0[0][ii][j&0x03][jj]=M1[i][j];		  
		  M0[1][ii][j&0x03][jj]=M1[i+1][j];		  
		  M0[2][ii][j&0x03][jj]=M1[i+2][j];		  
		  M0[3][ii][j&0x03][jj]=M1[i+3][j];
	  }
	  mm +=lx;
  }

  for (jj=0;jj<4;jj++)
  {
    for (ii=0;ii<4;ii++)
    {
      	p0=M0[0][ii][0][jj];
		p1=M0[1][ii][0][jj];
		p2=M0[2][ii][0][jj];
		p3=M0[3][ii][0][jj];
		HOR_DCT(p0,p1,p2,p3);
		A0=p0;A1=p1;A2=p2;A3=p3;

		p0=M0[0][ii][1][jj];
		p1=M0[1][ii][1][jj];
		p2=M0[2][ii][1][jj];
		p3=M0[3][ii][1][jj];
		HOR_DCT(p0,p1,p2,p3);
		B0=p0;B1=p1;B2=p2;B3=p3;

		p0=M0[0][ii][2][jj];
		p1=M0[1][ii][2][jj];
		p2=M0[2][ii][2][jj];
		p3=M0[3][ii][2][jj];
		HOR_DCT(p0,p1,p2,p3);
		C0=p0;C1=p1;C2=p2;C3=p3;

		p0=M0[0][ii][3][jj];
		p1=M0[1][ii][3][jj];
		p2=M0[2][ii][3][jj];
		p3=M0[3][ii][3][jj];
		HOR_DCT(p0,p1,p2,p3);
		D0=p0;D1=p1;D2=p2;D3=p3;

      // vertical
		VER_DCT(A0,B0,C0,D0);
		M0[0][ii][0][jj]=p0;		M0[0][ii][1][jj]=p1;
		M0[0][ii][2][jj]=p2;		M0[0][ii][3][jj]=p3;
		VER_DCT(A1,B1,C1,D1);
		M0[1][ii][0][jj]=p0;		M0[1][ii][1][jj]=p1;
		M0[1][ii][2][jj]=p2;		M0[1][ii][3][jj]=p3;
		VER_DCT(A2,B2,C2,D2);
		M0[2][ii][0][jj]=p0;		M0[2][ii][1][jj]=p1;
		M0[2][ii][2][jj]=p2;		M0[2][ii][3][jj]=p3;
		VER_DCT(A3,B3,C3,D3);
		M0[3][ii][0][jj]=p0;		M0[3][ii][1][jj]=p1;
		M0[3][ii][2][jj]=p2;		M0[3][ii][3][jj]=p3;
    }
  }

  // pick out DC coeff
  //horizontal
  p0=M0[0][0][0][0];  p1=M0[0][1][0][0];
  p2=M0[0][2][0][0];  p3=M0[0][3][0][0];
  HOR_CVT(p0,p1,p2,p3);
  A0=p0;A1=p1;A2=p2;A3=p3;
  
  p0=M0[0][0][0][1];  p1=M0[0][1][0][1];
  p2=M0[0][2][0][1];  p3=M0[0][3][0][1];
  HOR_CVT(p0,p1,p2,p3);
  B0=p0;B1=p1;B2=p2;B3=p3;

  p0=M0[0][0][0][2];  p1=M0[0][1][0][2];
  p2=M0[0][2][0][2];  p3=M0[0][3][0][2];
  HOR_CVT(p0,p1,p2,p3);
  C0=p0;C1=p1;C2=p2;C3=p3;

  p0=M0[0][0][0][3];  p1=M0[0][1][0][3];
  p2=M0[0][2][0][3];  p3=M0[0][3][0][3];
  HOR_CVT(p0,p1,p2,p3);
  D0=p0;D1=p1;D2=p2;D3=p3;

  // vertical
  VER_CVT(A0,B0,C0,D0);
  outD[0]=p0>>1;outD[2]=p1>>1;outD[3]=p2>>1;outD[9]=p3>>1;
  VER_CVT(A1,B1,C1,D1);
  outD[1]=p0>>1;outD[4]=p1>>1;outD[8]=p2>>1;outD[10]=p3>>1;
  VER_CVT(A2,B2,C2,D2);
  outD[5]=p0>>1;outD[7]=p1>>1;outD[11]=p2>>1;outD[14]=p3>>1;
  VER_CVT(A3,B3,C3,D3);
  outD[6]=p0>>1;outD[12]=p1>>1;outD[13]=p2>>1;outD[15]=p3>>1;

  // quant
  run=-1;
  scan_pos=0;
  for (coeff_ctr=0; coeff_ctr < 16; coeff_ctr++) 
  {
	  pp = outD[coeff_ctr];
	  run++;
	  ilev=0;

	  level =(abs(pp) * quant_coef[qp_rem][0][0] + 2*qp_const) >> (q_bits+1);
	  if (level  != 0) 
	  {
		  img->cofy[scan_pos][0]=sign(level ,pp);
		  img->cofy[scan_pos][1]=run;
		  scan_pos++;
		  run=-1;
	  }
	  outD[coeff_ctr]=sign(level,pp);/*level,not ilev*/
  }
  img->cofy[scan_pos][0]=0;

  // invers DC transform
  p0=outD[0];p1=outD[2];p2=outD[3];p3=outD[9];
  VER_ICVT(p0,p1,p2,p3);
  A0=p0;A1=p1;A2=p2;A3=p3;
  p0=outD[1];p1=outD[4];p2=outD[8];p3=outD[10];
  VER_ICVT(p0,p1,p2,p3);
  B0=p0;B1=p1;B2=p2;B3=p3;
  p0=outD[5];p1=outD[7];p2=outD[11];p3=outD[14];
  VER_ICVT(p0,p1,p2,p3);
  C0=p0;C1=p1;C2=p2;C3=p3;
  p0=outD[6];p1=outD[12];p2=outD[13];p3=outD[15];
  VER_ICVT(p0,p1,p2,p3);
  D0=p0;D1=p1;D2=p2;D3=p3;
  
  
  HOR_ICVT(A0,B0,C0,D0);
  M4[0][0] = p0;	M4[0][1]=p1;
  M4[0][2] = p2;	M4[0][3]=p3;
  HOR_ICVT(A1,B1,C1,D1);
  M4[1][0] = p0;	M4[1][1]=p1;
  M4[1][2] = p2;	M4[1][3]=p3;
  HOR_ICVT(A2,B2,C2,D2);
  M4[2][0] = p0;	M4[2][1]=p1;
  M4[2][2] = p2;	M4[2][3]=p3;
  HOR_ICVT(A3,B3,C3,D3);
  M4[3][0] = p0;	M4[3][1]=p1;
  M4[3][2] = p2;	M4[3][3]=p3;

  M0[0][0][0][0] = ((M4[0][0]*dequant_coef[qp_rem][0][0]<<qp_per)+2)>>2;
  M0[0][0][0][3] = ((M4[3][0]*dequant_coef[qp_rem][0][0]<<qp_per)+2)>>2;
  M0[0][0][0][1] = ((M4[1][0]*dequant_coef[qp_rem][0][0]<<qp_per)+2)>>2;
  M0[0][0][0][2] = ((M4[2][0]*dequant_coef[qp_rem][0][0]<<qp_per)+2)>>2;
  M0[0][1][0][0] = ((M4[0][1]*dequant_coef[qp_rem][0][0]<<qp_per)+2)>>2;
  M0[0][1][0][3] = ((M4[3][1]*dequant_coef[qp_rem][0][0]<<qp_per)+2)>>2;
  M0[0][1][0][1] = ((M4[1][1]*dequant_coef[qp_rem][0][0]<<qp_per)+2)>>2;
  M0[0][1][0][2] = ((M4[2][1]*dequant_coef[qp_rem][0][0]<<qp_per)+2)>>2;
  M0[0][2][0][0] = ((M4[0][2]*dequant_coef[qp_rem][0][0]<<qp_per)+2)>>2;
  M0[0][2][0][3] = ((M4[3][2]*dequant_coef[qp_rem][0][0]<<qp_per)+2)>>2;
  M0[0][2][0][1] = ((M4[1][2]*dequant_coef[qp_rem][0][0]<<qp_per)+2)>>2;
  M0[0][2][0][2] = ((M4[2][2]*dequant_coef[qp_rem][0][0]<<qp_per)+2)>>2;
  M0[0][3][0][0] = ((M4[0][3]*dequant_coef[qp_rem][0][0]<<qp_per)+2)>>2;
  M0[0][3][0][3] = ((M4[3][3]*dequant_coef[qp_rem][0][0]<<qp_per)+2)>>2;
  M0[0][3][0][1] = ((M4[1][3]*dequant_coef[qp_rem][0][0]<<qp_per)+2)>>2;
  M0[0][3][0][2] = ((M4[2][3]*dequant_coef[qp_rem][0][0]<<qp_per)+2)>>2;

  // AC invers trans/quant for MB
  for (jj=0;jj<4;jj++)
  {
    for (ii=0;ii<4;ii++)
    {
      run      = -1;
      scan_pos =  0;

	  for (coeff_ctr=1;coeff_ctr < 16;coeff_ctr++)
	  { 
		  i=SNGL_SCAN[coeff_ctr][0];
		  j=SNGL_SCAN[coeff_ctr][1];

		  pp=M0[i][ii][j][jj];
		  run++;
		  ilev=0;
		  level = (abs (pp) * quant_coef1[qp_rem][coeff_ctr] + qp_const) >> q_bits;
		  if (level != 0)
		  {
			  ac_coef = 15;
			  img->cof[jj][ii][scan_pos][0]=sign(level,pp);
			  img->cof[jj][ii][scan_pos][1]=run;
			  ++scan_pos;
			  run=-1;                     // reset zero level counter
			  ilev=level*dequant_coef1[qp_rem][coeff_ctr]<<qp_per;
		  }
		  M0[i][ii][j][jj]=sign(ilev,pp);
	  }
	  img->cof[jj][ii][scan_pos][0]=0;


	  // invers AC transform
      // IDCT horizontal
	  p0=M0[0][ii][0][jj];p1=M0[1][ii][0][jj];
	  p2=M0[2][ii][0][jj];p3=M0[3][ii][0][jj];
	  VER_IDCT(p0,p1,p2,p3);
	  A0=p0;A1=p1;A2=p2;A3=p3;
	  p0=M0[0][ii][1][jj];p1=M0[1][ii][1][jj];
	  p2=M0[2][ii][1][jj];p3=M0[3][ii][1][jj];
	  VER_IDCT(p0,p1,p2,p3);
	  B0=p0;B1=p1;B2=p2;B3=p3;
	  p0=M0[0][ii][2][jj];p1=M0[1][ii][2][jj];
	  p2=M0[2][ii][2][jj];p3=M0[3][ii][2][jj];
	  VER_IDCT(p0,p1,p2,p3);
	  C0=p0;C1=p1;C2=p2;C3=p3;
	  p0=M0[0][ii][3][jj];p1=M0[1][ii][3][jj];
	  p2=M0[2][ii][3][jj];p3=M0[3][ii][3][jj];
	  VER_IDCT(p0,p1,p2,p3);
	  D0=p0;D1=p1;D2=p2;D3=p3;
	  

	  i=ii<<2;	j=jj<<2;
	  HOR_IDCT(A0,B0,C0,D0);
	  M1[0+i][0+j] = p0;	M1[0+i][1+j] = p1;
	  M1[0+i][2+j] = p2;	M1[0+i][3+j] = p3;
	  HOR_IDCT(A1,B1,C1,D1);	  
	  M1[1+i][0+j] = p0;	M1[1+i][1+j] = p1;
	  M1[1+i][2+j] = p2;	M1[1+i][3+j] = p3;
	  HOR_IDCT(A2,B2,C2,D2);	  
	  M1[2+i][0+j] = p0;	M1[2+i][1+j] = p1;
	  M1[2+i][2+j] = p2;	M1[2+i][3+j] = p3;
	  HOR_IDCT(A3,B3,C3,D3);
	  M1[3+i][0+j] = p0;	M1[3+i][1+j] = p1;
	  M1[3+i][2+j] = p2;	M1[3+i][3+j] = p3;
    }
  }

 for (j=0;j<16;j++)
 {
	 pp=(img->pix_y+j)*lx+img->pix_x;
	 for (i=0;i<16;i+=4)
	 {
		 imgY[pp+i]   = (byte)IDCT_SHIFT(M1[i][j],  img->mprr_2[new_intra_mode][j*16+i]);
		 imgY[pp+i+1] = (byte)IDCT_SHIFT(M1[i+1][j],img->mprr_2[new_intra_mode][j*16+i+1]);
		 imgY[pp+i+2] = (byte)IDCT_SHIFT(M1[i+2][j],img->mprr_2[new_intra_mode][j*16+i+2]);
		 imgY[pp+i+3] = (byte)IDCT_SHIFT(M1[i+3][j],img->mprr_2[new_intra_mode][j*16+i+3]);
	 }
 }
    return ac_coef;
}

int inter_dct_luma(int *ii, int *coeff, int block_x,int block_y,int *coeff_cost, struct img_par *img)
{ 
  int sign(int a,int b);
 
  int ilev,coeff_ctr;
  int qp_const,level,scan_pos,run;
  int nonzero;
  int qp_per,qp_rem,q_bits;

  int   pos_x   = block_x/BLOCK_SIZE;
  int   pos_y   = block_y/BLOCK_SIZE;

  int p0,p1,p2,p3;
  int q0,q1,q2,q3;
  int A0,A1,A2,A3;
  int B0,B1,B2,B3;
  int C0,C1,C2,C3;
  int D0,D1,D2,D3;
  //int width=img->width+IMG_PAD_SIZE;
  int outD[16];

  qp_per    = (img->qp-MIN_QP)/6;
  qp_rem    = (img->qp-MIN_QP)%6;
  q_bits    = Q_BITS+qp_per;

  if (img->type == INTRA_IMG)
    qp_const=(1<<q_bits)/3;    // intra
  else
    qp_const=(1<<q_bits)/6;    // inter
  
  p0=ii[0];
  p1=ii[1];
  p2=ii[2];
  p3=ii[3];
  HOR_DCT(p0,p1,p2,p3);
  A0=p0;A1=p1;A2=p2;A3=p3;
  ii+=16;

  p0=ii[0];
  p1=ii[1];
  p2=ii[2];
  p3=ii[3];
  HOR_DCT(p0,p1,p2,p3);
  B0=p0;B1=p1;B2=p2;B3=p3;
  ii+=16;
  
  p0=ii[0];
  p1=ii[1];
  p2=ii[2];
  p3=ii[3];
  HOR_DCT(p0,p1,p2,p3);
  C0=p0;C1=p1;C2=p2;C3=p3;
  ii+=16;
  
  p0=ii[0];
  p1=ii[1];
  p2=ii[2];
  p3=ii[3];
  HOR_DCT(p0,p1,p2,p3);
  D0=p0;D1=p1;D2=p2;D3=p3;
  
  VER_DCT(A0,B0,C0,D0);
  outD[0]=p0;outD[2]=p1;outD[3]=p2;outD[9]=p3;
  VER_DCT(A1,B1,C1,D1);
  outD[1]=p0;outD[4]=p1;outD[8]=p2;outD[10]=p3;
  VER_DCT(A2,B2,C2,D2);
  outD[5]=p0;outD[7]=p1;outD[11]=p2;outD[14]=p3;
  VER_DCT(A3,B3,C3,D3);
  outD[6]=p0;outD[12]=p1;outD[13]=p2;outD[15]=p3;
  // Quant

  nonzero=FALSE;

  run=-1;
  scan_pos=0;
  
  for (coeff_ctr=0;coeff_ctr < 16;coeff_ctr++)
  { 
    p0=outD[coeff_ctr];
    run++;
    ilev=0;
    level = (abs (p0) * quant_coef1[qp_rem][coeff_ctr] + qp_const) >> q_bits;
    if (level != 0)
    {
      nonzero=TRUE;
      if (level > 1)
        *coeff_cost += MAX_VALUE;
      else
        *coeff_cost += COEFF_COST[run];
      img->cof[pos_x][pos_y][scan_pos][0]=sign(level,p0);
      img->cof[pos_x][pos_y][scan_pos][1]=run;
      ++scan_pos;
      run=-1;
      ilev=level*dequant_coef1[qp_rem][coeff_ctr]<<qp_per;
    }
    coeff[coeff_ctr]=sign(ilev,p0);
  }
  img->cof[pos_x][pos_y][scan_pos][0]=0;

  cbp_blk[img->block_y+pos_y][img->block_x+pos_x]=nonzero;
  return nonzero;
}

void inter_idct_luma(int *outD, byte *pp, int block_x,int block_y,struct img_par *img,int width)
{ 
  int sign(int a,int b);
 
  int j;
  //int qp_const,level,scan_pos,run;
  //int qp_per,qp_rem,q_bits;

  int   pos_x   = block_x/BLOCK_SIZE;
  int   pos_y   = block_y/BLOCK_SIZE;

  int p0,p1,p2,p3;
  int q0,q1,q2,q3;
  int A0,A1,A2,A3;
  int B0,B1,B2,B3;
  int C0,C1,C2,C3;
  int D0,D1,D2,D3;
  //int width=img->width;
  int out[256];
  
  p0=outD[0];p1=outD[2];p2=outD[3];p3=outD[9];
  VER_IDCT(p0,p1,p2,p3);
  A0=p0;A1=p1;A2=p2;A3=p3;
  p0=outD[1];p1=outD[4];p2=outD[8];p3=outD[10];
  VER_IDCT(p0,p1,p2,p3);
  B0=p0;B1=p1;B2=p2;B3=p3;
  p0=outD[5];p1=outD[7];p2=outD[11];p3=outD[14];
  VER_IDCT(p0,p1,p2,p3);
  C0=p0;C1=p1;C2=p2;C3=p3;
  p0=outD[6];p1=outD[12];p2=outD[13];p3=outD[15];
  VER_IDCT(p0,p1,p2,p3);
  D0=p0;D1=p1;D2=p2;D3=p3;
  
  
  HOR_IDCT(A0,B0,C0,D0);
  out[0]=IDCT_SHIFT(p0,pp[0]);
  out[1]=IDCT_SHIFT(p1,pp[1]);
  out[2]=IDCT_SHIFT(p2,pp[2]);
  out[3]=IDCT_SHIFT(p3,pp[3]);
  pp+=width;
  HOR_IDCT(A1,B1,C1,D1);
  out[4]=IDCT_SHIFT(p0,pp[0]);
  out[5]=IDCT_SHIFT(p1,pp[1]);
  out[6]=IDCT_SHIFT(p2,pp[2]);
  out[7]=IDCT_SHIFT(p3,pp[3]);
  pp+=width;
  HOR_IDCT(A2,B2,C2,D2);
  out[8]=IDCT_SHIFT(p0,pp[0]);
  out[9]=IDCT_SHIFT(p1,pp[1]);
  out[10]=IDCT_SHIFT(p2,pp[2]);
  out[11]=IDCT_SHIFT(p3,pp[3]);
  pp+=width;
  HOR_IDCT(A3,B3,C3,D3);
  out[12]=IDCT_SHIFT(p0,pp[0]);
  out[13]=IDCT_SHIFT(p1,pp[1]);
  out[14]=IDCT_SHIFT(p2,pp[2]);
  out[15]=IDCT_SHIFT(p3,pp[3]);
  
  for (j=0; j < BLOCK_SIZE; j++) 
  {
	  p0=(img->pix_y+block_y+j)*(img->width+IMG_PAD_SIZE)+img->pix_x+block_x;
	  p1=j*4;
	  imgY[p0  ]=out[p1  ];
	  imgY[p0+1]=out[p1+1];
	  imgY[p0+2]=out[p1+2];
	  imgY[p0+3]=out[p1+3];
  }
}

#define UVFILTER1(f0,f1,x0,x1) (((f0)*(x0)+(f1)*(x1)+4)>>3)
#define UVFILTER2(if0,if1,jf0,jf1,x0,y0,x1,y1)\
	 (((if0)*(jf0)*(x0)+(if1)*(jf0)*(x1)+(if0)*(jf1)*(y0)+(if1)*(jf1)*(y1)+32)>>6)

void Predict(struct img_par *img)
{
	int m,n;
	int x = img->pix_x;
	int y = img->pix_y;
	int mvx = curr_mvx;
	int mvy = curr_mvy;
	int*  Cb;
	int*  Cr;
	int offset1;
	int offset2;
	int cc;
	byte*  iCb1;
	byte*  iCb2;
	byte*  iCr1;
	byte*  iCr2;
	//byte*  cCr;
	//byte*  cCb;

	int lx=img->width+IMG_PAD_SIZE;
	byte*  curr=imgY_org + x+y*lx;
	byte*  prev=ipol[img->comp] + img->offset;
	int*  error=MbLum;
	
	int if0,if1,jf0,jf1;

	int x0,x1,x2,x3,x4,x5,x6,x7,x8;
	int y0,y1,y2,y3,y4,y5,y6,y7,y8;
	int z0,z1,z2,z3,z4,z5,z6,z7;

	for (n = 0; n < MB_BLOCK_SIZE; n++)
	{
		z0=curr[ 0];		y0=prev[ 0];
		z1=curr[ 1];		y1=prev[ 1];
		z2=curr[ 2];		y2=prev[ 2];
		z3=curr[ 3];		y3=prev[ 3];
		z4=curr[ 4];		y4=prev[ 4];
		z5=curr[ 5];		y5=prev[ 5];
		z6=curr[ 6];		y6=prev[ 6];
		z7=curr[ 7];		y7=prev[ 7];
		error[ 0]=z0-y0;
		error[ 1]=z1-y1;
		error[ 2]=z2-y2;
		error[ 3]=z3-y3;
		error[ 4]=z4-y4;
		error[ 5]=z5-y5;
		error[ 6]=z6-y6;
		error[ 7]=z7-y7;
		z0=curr[ 8];		y0=prev[ 8];
		z1=curr[ 9];		y1=prev[ 9];
		z2=curr[10];		y2=prev[10];
		z3=curr[11];		y3=prev[11];
		z4=curr[12];		y4=prev[12];
		z5=curr[13];		y5=prev[13];
		z6=curr[14];		y6=prev[14];
		z7=curr[15];		y7=prev[15];
		error[ 8]=z0-y0;
		error[ 9]=z1-y1;
		error[10]=z2-y2;
		error[11]=z3-y3;
		error[12]=z4-y4;
		error[13]=z5-y5;
		error[14]=z6-y6;
		error[15]=z7-y7;
		error+=16;
		curr+=lx;
		prev+=lx;
	}
	
  x = img->pix_c_x;
  y = img->pix_c_y;
  m=( (mvx&7)==0 ? 0:1);
  n=( (mvy&7)==0 ? 0:1);
  cc=m+(n<<1);
  lx = img->width_cr+IMG_PAD_SIZE;
  Cb=MbCb;
  Cr=MbCr;
  offset1=x + (mvx>>3) + (y + (mvy>>3))*lx;
  offset2=x + y*lx;	
  iCb1=mcef[0]+offset1;
  iCb2=iCb1+lx;
  iCr1=mcef[1]+offset1;
  iCr2=iCr1+lx;

  switch(cc)
  {
  	case 0:
  		for (n = 0; n < 8; n++)
		{
			x0=iCb1[0];
			x1=iCb1[1];
			x2=iCb1[2];
			x3=iCb1[3];
			x4=iCb1[4];
			x5=iCb1[5];
			x6=iCb1[6];
			x7=iCb1[7];
			Cb[0]=x0;
			Cb[1]=x1;
			Cb[2]=x2;
			Cb[3]=x3;
			Cb[4]=x4;
			Cb[5]=x5;
			Cb[6]=x6;
			Cb[7]=x7;
			x0=iCr1[0];
			x1=iCr1[1];
			x2=iCr1[2];
			x3=iCr1[3];
			x4=iCr1[4];
			x5=iCr1[5];
			x6=iCr1[6];
			x7=iCr1[7];
			Cr[0]=x0;
			Cr[1]=x1;
			Cr[2]=x2;
			Cr[3]=x3;
			Cr[4]=x4;
			Cr[5]=x5;
			Cr[6]=x6;
			Cr[7]=x7;
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
		}
		break;
	case 1:
		if1=(mvx & 7);
	        if0=8-if1;	 
		for (n = 0; n < 8; n++)
		{
			x0=iCb1[0];
			x1=iCb1[1];
			x2=iCb1[2];
			x3=iCb1[3];
			x4=iCb1[4];
			x5=iCb1[5];
			x6=iCb1[6];
			x7=iCb1[7];
			x8=iCb1[8];
			Cb[0]=UVFILTER1(if0,if1,x0,x1);
			Cb[1]=UVFILTER1(if0,if1,x1,x2);
			Cb[2]=UVFILTER1(if0,if1,x2,x3);
			Cb[3]=UVFILTER1(if0,if1,x3,x4);
			Cb[4]=UVFILTER1(if0,if1,x4,x5);
			Cb[5]=UVFILTER1(if0,if1,x5,x6);
			Cb[6]=UVFILTER1(if0,if1,x6,x7);
			Cb[7]=UVFILTER1(if0,if1,x7,x8);
			x0=iCr1[0];
			x1=iCr1[1];
			x2=iCr1[2];
			x3=iCr1[3];
			x4=iCr1[4];
			x5=iCr1[5];
			x6=iCr1[6];
			x7=iCr1[7];
			x8=iCr1[8];
			Cr[0]=UVFILTER1(if0,if1,x0,x1);
			Cr[1]=UVFILTER1(if0,if1,x1,x2);
			Cr[2]=UVFILTER1(if0,if1,x2,x3);
			Cr[3]=UVFILTER1(if0,if1,x3,x4);
			Cr[4]=UVFILTER1(if0,if1,x4,x5);
			Cr[5]=UVFILTER1(if0,if1,x5,x6);
			Cr[6]=UVFILTER1(if0,if1,x6,x7);
			Cr[7]=UVFILTER1(if0,if1,x7,x8);
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
		}
		break;
	case 2:
		jf1=(mvy & 7);
	        jf0=8-jf1;
		for (n = 0; n < 8; n++)
		{
			x0=iCb1[0];		y0=iCb2[0];
			x1=iCb1[1];		y1=iCb2[1];
			x2=iCb1[2];		y2=iCb2[2];
			x3=iCb1[3];		y3=iCb2[3];
			x4=iCb1[4];		y4=iCb2[4];
			x5=iCb1[5];		y5=iCb2[5];
			x6=iCb1[6];		y6=iCb2[6];
			x7=iCb1[7];		y7=iCb2[7];
			Cb[0]=UVFILTER1(jf0,jf1,x0,y0);
			Cb[1]=UVFILTER1(jf0,jf1,x1,y1);
			Cb[2]=UVFILTER1(jf0,jf1,x2,y2);
			Cb[3]=UVFILTER1(jf0,jf1,x3,y3);
			Cb[4]=UVFILTER1(jf0,jf1,x4,y4);
			Cb[5]=UVFILTER1(jf0,jf1,x5,y5);
			Cb[6]=UVFILTER1(jf0,jf1,x6,y6);
			Cb[7]=UVFILTER1(jf0,jf1,x7,y7);
			x0=iCr1[0];		y0=iCr2[0];
			x1=iCr1[1];		y1=iCr2[1];
			x2=iCr1[2];		y2=iCr2[2];
			x3=iCr1[3];		y3=iCr2[3];
			x4=iCr1[4];		y4=iCr2[4];
			x5=iCr1[5];		y5=iCr2[5];
			x6=iCr1[6];		y6=iCr2[6];
			x7=iCr1[7];		y7=iCr2[7];
			Cr[0]=UVFILTER1(jf0,jf1,x0,y0);
			Cr[1]=UVFILTER1(jf0,jf1,x1,y1);
			Cr[2]=UVFILTER1(jf0,jf1,x2,y2);
			Cr[3]=UVFILTER1(jf0,jf1,x3,y3);
			Cr[4]=UVFILTER1(jf0,jf1,x4,y4);
			Cr[5]=UVFILTER1(jf0,jf1,x5,y5);
			Cr[6]=UVFILTER1(jf0,jf1,x6,y6);
			Cr[7]=UVFILTER1(jf0,jf1,x7,y7);
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
			iCb2+=lx;		iCr2+=lx;
		}
		break;
	default:
		if1=(mvx & 7);
	        if0=8-if1;
	        jf1=(mvy & 7);
	        jf0=8-jf1;
		for (n = 0; n < 8; n++)
		{
			x0=iCb1[0];		y0=iCb2[0];
			x1=iCb1[1];		y1=iCb2[1];
			x2=iCb1[2];		y2=iCb2[2];
			x3=iCb1[3];		y3=iCb2[3];
			x4=iCb1[4];		y4=iCb2[4];
			x5=iCb1[5];		y5=iCb2[5];
			x6=iCb1[6];		y6=iCb2[6];
			x7=iCb1[7];		y7=iCb2[7];
			x8=iCb1[8];		y8=iCb2[8];
			Cb[0]=UVFILTER2(if0,if1,jf0,jf1,x0,y0,x1,y1);
			Cb[1]=UVFILTER2(if0,if1,jf0,jf1,x1,y1,x2,y2);
			Cb[2]=UVFILTER2(if0,if1,jf0,jf1,x2,y2,x3,y3);
			Cb[3]=UVFILTER2(if0,if1,jf0,jf1,x3,y3,x4,y4);
			Cb[4]=UVFILTER2(if0,if1,jf0,jf1,x4,y4,x5,y5);
			Cb[5]=UVFILTER2(if0,if1,jf0,jf1,x5,y5,x6,y6);
			Cb[6]=UVFILTER2(if0,if1,jf0,jf1,x6,y6,x7,y7);
			Cb[7]=UVFILTER2(if0,if1,jf0,jf1,x7,y7,x8,y8);
			x0=iCr1[0];		y0=iCr2[0];
			x1=iCr1[1];		y1=iCr2[1];
			x2=iCr1[2];		y2=iCr2[2];
			x3=iCr1[3];		y3=iCr2[3];
			x4=iCr1[4];		y4=iCr2[4];
			x5=iCr1[5];		y5=iCr2[5];
			x6=iCr1[6];		y6=iCr2[6];
			x7=iCr1[7];		y7=iCr2[7];
			x8=iCr1[8];		y8=iCr2[8];
			Cr[0]=UVFILTER2(if0,if1,jf0,jf1,x0,y0,x1,y1);
			Cr[1]=UVFILTER2(if0,if1,jf0,jf1,x1,y1,x2,y2);
			Cr[2]=UVFILTER2(if0,if1,jf0,jf1,x2,y2,x3,y3);
			Cr[3]=UVFILTER2(if0,if1,jf0,jf1,x3,y3,x4,y4);
			Cr[4]=UVFILTER2(if0,if1,jf0,jf1,x4,y4,x5,y5);
			Cr[5]=UVFILTER2(if0,if1,jf0,jf1,x5,y5,x6,y6);
			Cr[6]=UVFILTER2(if0,if1,jf0,jf1,x6,y6,x7,y7);
			Cr[7]=UVFILTER2(if0,if1,jf0,jf1,x7,y7,x8,y8);
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
			iCb2+=lx;		iCr2+=lx;
		}
		break;
	}
}

void Predict_16x8(struct img_par *img, int index)
{
	int m,n;
	int x = img->pix_x;
	int y = img->pix_y+(index<<3);
	int mvx = curr_mvx1[index];
	int mvy = curr_mvy1[index];
	int*  Cb;
	int*  Cr;
	int offset1;
	int offset2;
	int cc;
	byte*  iCb1;
	byte*  iCb2;
	byte*  iCr1;
	byte*  iCr2;

	int lx=img->width+IMG_PAD_SIZE;
	byte*  curr=imgY_org + x+y*lx;
	byte*  prev=ipol[img->comp1[index]] + img->offset1[index];
	int*  error=MbLum+(index<<7);
	
	int if0,if1,jf0,jf1;

	int x0,x1,x2,x3,x4,x5,x6,x7,x8;
	int y0,y1,y2,y3,y4,y5,y6,y7,y8;
	int z0,z1,z2,z3,z4,z5,z6,z7;

	for (n = 0; n < 8; n++)
	{
		z0=curr[ 0];		y0=prev[ 0];
		z1=curr[ 1];		y1=prev[ 1];
		z2=curr[ 2];		y2=prev[ 2];
		z3=curr[ 3];		y3=prev[ 3];
		z4=curr[ 4];		y4=prev[ 4];
		z5=curr[ 5];		y5=prev[ 5];
		z6=curr[ 6];		y6=prev[ 6];
		z7=curr[ 7];		y7=prev[ 7];
		error[ 0]=z0-y0;
		error[ 1]=z1-y1;
		error[ 2]=z2-y2;
		error[ 3]=z3-y3;
		error[ 4]=z4-y4;
		error[ 5]=z5-y5;
		error[ 6]=z6-y6;
		error[ 7]=z7-y7;
		z0=curr[ 8];		y0=prev[ 8];
		z1=curr[ 9];		y1=prev[ 9];
		z2=curr[10];		y2=prev[10];
		z3=curr[11];		y3=prev[11];
		z4=curr[12];		y4=prev[12];
		z5=curr[13];		y5=prev[13];
		z6=curr[14];		y6=prev[14];
		z7=curr[15];		y7=prev[15];
		error[ 8]=z0-y0;
		error[ 9]=z1-y1;
		error[10]=z2-y2;
		error[11]=z3-y3;
		error[12]=z4-y4;
		error[13]=z5-y5;
		error[14]=z6-y6;
		error[15]=z7-y7;
		error+=16;
		curr+=lx;
		prev+=lx;
	}
	
  x = img->pix_c_x;
  y = img->pix_c_y+(index<<2);
  m=( (mvx&7)==0 ? 0:1);
  n=( (mvy&7)==0 ? 0:1);
  cc=m+(n<<1);
  lx = img->width_cr+IMG_PAD_SIZE;
  Cb=MbCb+(index<<5);
  Cr=MbCr+(index<<5);
  offset1=x + (mvx>>3) + (y + (mvy>>3))*lx;
  offset2=x + y*lx;	
  iCb1=mcef[0]+offset1;
  iCb2=iCb1+lx;
  iCr1=mcef[1]+offset1;
  iCr2=iCr1+lx;

  switch(cc)
  {
  	case 0:
  		for (n = 0; n < 4; n++)
		{
			x0=iCb1[0];
			x1=iCb1[1];
			x2=iCb1[2];
			x3=iCb1[3];
			x4=iCb1[4];
			x5=iCb1[5];
			x6=iCb1[6];
			x7=iCb1[7];
			Cb[0]=x0;
			Cb[1]=x1;
			Cb[2]=x2;
			Cb[3]=x3;
			Cb[4]=x4;
			Cb[5]=x5;
			Cb[6]=x6;
			Cb[7]=x7;
			x0=iCr1[0];
			x1=iCr1[1];
			x2=iCr1[2];
			x3=iCr1[3];
			x4=iCr1[4];
			x5=iCr1[5];
			x6=iCr1[6];
			x7=iCr1[7];
			Cr[0]=x0;
			Cr[1]=x1;
			Cr[2]=x2;
			Cr[3]=x3;
			Cr[4]=x4;
			Cr[5]=x5;
			Cr[6]=x6;
			Cr[7]=x7;
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
		}
		break;
	case 1:
		if1=(mvx & 7);
	        if0=8-if1;	 
		for (n = 0; n < 4; n++)
		{
			x0=iCb1[0];
			x1=iCb1[1];
			x2=iCb1[2];
			x3=iCb1[3];
			x4=iCb1[4];
			x5=iCb1[5];
			x6=iCb1[6];
			x7=iCb1[7];
			x8=iCb1[8];
			Cb[0]=UVFILTER1(if0,if1,x0,x1);
			Cb[1]=UVFILTER1(if0,if1,x1,x2);
			Cb[2]=UVFILTER1(if0,if1,x2,x3);
			Cb[3]=UVFILTER1(if0,if1,x3,x4);
			Cb[4]=UVFILTER1(if0,if1,x4,x5);
			Cb[5]=UVFILTER1(if0,if1,x5,x6);
			Cb[6]=UVFILTER1(if0,if1,x6,x7);
			Cb[7]=UVFILTER1(if0,if1,x7,x8);
			x0=iCr1[0];
			x1=iCr1[1];
			x2=iCr1[2];
			x3=iCr1[3];
			x4=iCr1[4];
			x5=iCr1[5];
			x6=iCr1[6];
			x7=iCr1[7];
			x8=iCr1[8];
			Cr[0]=UVFILTER1(if0,if1,x0,x1);
			Cr[1]=UVFILTER1(if0,if1,x1,x2);
			Cr[2]=UVFILTER1(if0,if1,x2,x3);
			Cr[3]=UVFILTER1(if0,if1,x3,x4);
			Cr[4]=UVFILTER1(if0,if1,x4,x5);
			Cr[5]=UVFILTER1(if0,if1,x5,x6);
			Cr[6]=UVFILTER1(if0,if1,x6,x7);
			Cr[7]=UVFILTER1(if0,if1,x7,x8);
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
		}
		break;
	case 2:
		jf1=(mvy & 7);
	        jf0=8-jf1;
		for (n = 0; n < 4; n++)
		{
			x0=iCb1[0];		y0=iCb2[0];
			x1=iCb1[1];		y1=iCb2[1];
			x2=iCb1[2];		y2=iCb2[2];
			x3=iCb1[3];		y3=iCb2[3];
			x4=iCb1[4];		y4=iCb2[4];
			x5=iCb1[5];		y5=iCb2[5];
			x6=iCb1[6];		y6=iCb2[6];
			x7=iCb1[7];		y7=iCb2[7];
			Cb[0]=UVFILTER1(jf0,jf1,x0,y0);
			Cb[1]=UVFILTER1(jf0,jf1,x1,y1);
			Cb[2]=UVFILTER1(jf0,jf1,x2,y2);
			Cb[3]=UVFILTER1(jf0,jf1,x3,y3);
			Cb[4]=UVFILTER1(jf0,jf1,x4,y4);
			Cb[5]=UVFILTER1(jf0,jf1,x5,y5);
			Cb[6]=UVFILTER1(jf0,jf1,x6,y6);
			Cb[7]=UVFILTER1(jf0,jf1,x7,y7);
			x0=iCr1[0];		y0=iCr2[0];
			x1=iCr1[1];		y1=iCr2[1];
			x2=iCr1[2];		y2=iCr2[2];
			x3=iCr1[3];		y3=iCr2[3];
			x4=iCr1[4];		y4=iCr2[4];
			x5=iCr1[5];		y5=iCr2[5];
			x6=iCr1[6];		y6=iCr2[6];
			x7=iCr1[7];		y7=iCr2[7];
			Cr[0]=UVFILTER1(jf0,jf1,x0,y0);
			Cr[1]=UVFILTER1(jf0,jf1,x1,y1);
			Cr[2]=UVFILTER1(jf0,jf1,x2,y2);
			Cr[3]=UVFILTER1(jf0,jf1,x3,y3);
			Cr[4]=UVFILTER1(jf0,jf1,x4,y4);
			Cr[5]=UVFILTER1(jf0,jf1,x5,y5);
			Cr[6]=UVFILTER1(jf0,jf1,x6,y6);
			Cr[7]=UVFILTER1(jf0,jf1,x7,y7);
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
			iCb2+=lx;		iCr2+=lx;
		}
		break;
	default:
		if1=(mvx & 7);
	        if0=8-if1;
	        jf1=(mvy & 7);
	        jf0=8-jf1;
		for (n = 0; n < 4; n++)
		{
			x0=iCb1[0];		y0=iCb2[0];
			x1=iCb1[1];		y1=iCb2[1];
			x2=iCb1[2];		y2=iCb2[2];
			x3=iCb1[3];		y3=iCb2[3];
			x4=iCb1[4];		y4=iCb2[4];
			x5=iCb1[5];		y5=iCb2[5];
			x6=iCb1[6];		y6=iCb2[6];
			x7=iCb1[7];		y7=iCb2[7];
			x8=iCb1[8];		y8=iCb2[8];
			Cb[0]=UVFILTER2(if0,if1,jf0,jf1,x0,y0,x1,y1);
			Cb[1]=UVFILTER2(if0,if1,jf0,jf1,x1,y1,x2,y2);
			Cb[2]=UVFILTER2(if0,if1,jf0,jf1,x2,y2,x3,y3);
			Cb[3]=UVFILTER2(if0,if1,jf0,jf1,x3,y3,x4,y4);
			Cb[4]=UVFILTER2(if0,if1,jf0,jf1,x4,y4,x5,y5);
			Cb[5]=UVFILTER2(if0,if1,jf0,jf1,x5,y5,x6,y6);
			Cb[6]=UVFILTER2(if0,if1,jf0,jf1,x6,y6,x7,y7);
			Cb[7]=UVFILTER2(if0,if1,jf0,jf1,x7,y7,x8,y8);
			x0=iCr1[0];		y0=iCr2[0];
			x1=iCr1[1];		y1=iCr2[1];
			x2=iCr1[2];		y2=iCr2[2];
			x3=iCr1[3];		y3=iCr2[3];
			x4=iCr1[4];		y4=iCr2[4];
			x5=iCr1[5];		y5=iCr2[5];
			x6=iCr1[6];		y6=iCr2[6];
			x7=iCr1[7];		y7=iCr2[7];
			x8=iCr1[8];		y8=iCr2[8];
			Cr[0]=UVFILTER2(if0,if1,jf0,jf1,x0,y0,x1,y1);
			Cr[1]=UVFILTER2(if0,if1,jf0,jf1,x1,y1,x2,y2);
			Cr[2]=UVFILTER2(if0,if1,jf0,jf1,x2,y2,x3,y3);
			Cr[3]=UVFILTER2(if0,if1,jf0,jf1,x3,y3,x4,y4);
			Cr[4]=UVFILTER2(if0,if1,jf0,jf1,x4,y4,x5,y5);
			Cr[5]=UVFILTER2(if0,if1,jf0,jf1,x5,y5,x6,y6);
			Cr[6]=UVFILTER2(if0,if1,jf0,jf1,x6,y6,x7,y7);
			Cr[7]=UVFILTER2(if0,if1,jf0,jf1,x7,y7,x8,y8);
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
			iCb2+=lx;		iCr2+=lx;
		}
		break;
	}
}

void Predict_8x16(struct img_par *img, int index)
{
	int m,n;
	int x = img->pix_x+(index<<3);
	int y = img->pix_y;
	int mvx = curr_mvx2[index];
	int mvy = curr_mvy2[index];
	int*  Cb;
	int*  Cr;
	int offset1;
	int offset2;
	int cc;
	byte*  iCb1;
	byte*  iCb2;
	byte*  iCr1;
	byte*  iCr2;

	int lx=img->width+IMG_PAD_SIZE;
	byte*  curr=imgY_org + x+y*lx;
	byte*  prev=ipol[img->comp2[index]] + img->offset2[index];
	int*  error=MbLum+8*index;
	
	int if0,if1,jf0,jf1;

	int x0,x1,x2,x3,x4;
	int y0,y1,y2,y3,y4,y5,y6,y7;
	int z0,z1,z2,z3,z4,z5,z6,z7;

	for (n = 0; n < 16; n++)
	{
		z0=curr[ 0];		y0=prev[ 0];
		z1=curr[ 1];		y1=prev[ 1];
		z2=curr[ 2];		y2=prev[ 2];
		z3=curr[ 3];		y3=prev[ 3];
		z4=curr[ 4];		y4=prev[ 4];
		z5=curr[ 5];		y5=prev[ 5];
		z6=curr[ 6];		y6=prev[ 6];
		z7=curr[ 7];		y7=prev[ 7];
		error[ 0]=z0-y0;
		error[ 1]=z1-y1;
		error[ 2]=z2-y2;
		error[ 3]=z3-y3;
		error[ 4]=z4-y4;
		error[ 5]=z5-y5;
		error[ 6]=z6-y6;
		error[ 7]=z7-y7;
		
		error+=16;
		curr+=lx;
		prev+=lx;
	}
	
  x = img->pix_c_x+(index<<2);
  y = img->pix_c_y;
  m=( (mvx&7)==0 ? 0:1);
  n=( (mvy&7)==0 ? 0:1);
  cc=m+(n<<1);
  lx = img->width_cr+IMG_PAD_SIZE;
  Cb=MbCb+index*4;
  Cr=MbCr+index*4;
  offset1=x + (mvx>>3) + (y + (mvy>>3))*lx;
  offset2=x + y*lx;	
  iCb1=mcef[0]+offset1;
  iCb2=iCb1+lx;
  iCr1=mcef[1]+offset1;
  iCr2=iCr1+lx;

  switch(cc)
  {
  	case 0:
  		for (n = 0; n < 8; n++)
		{
			x0=iCb1[0];
			x1=iCb1[1];
			x2=iCb1[2];
			x3=iCb1[3];
			
			Cb[0]=x0;
			Cb[1]=x1;
			Cb[2]=x2;
			Cb[3]=x3;
			
			x0=iCr1[0];
			x1=iCr1[1];
			x2=iCr1[2];
			x3=iCr1[3];
			
			Cr[0]=x0;
			Cr[1]=x1;
			Cr[2]=x2;
			Cr[3]=x3;
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
		}
		break;
	case 1:
		if1=(mvx & 7);
	        if0=8-if1;	 
		for (n = 0; n < 8; n++)
		{
			x0=iCb1[0];
			x1=iCb1[1];
			x2=iCb1[2];
			x3=iCb1[3];
			x4=iCb1[4];
			
			Cb[0]=UVFILTER1(if0,if1,x0,x1);
			Cb[1]=UVFILTER1(if0,if1,x1,x2);
			Cb[2]=UVFILTER1(if0,if1,x2,x3);
			Cb[3]=UVFILTER1(if0,if1,x3,x4);
			
			x0=iCr1[0];
			x1=iCr1[1];
			x2=iCr1[2];
			x3=iCr1[3];
			x4=iCr1[4];
			
			Cr[0]=UVFILTER1(if0,if1,x0,x1);
			Cr[1]=UVFILTER1(if0,if1,x1,x2);
			Cr[2]=UVFILTER1(if0,if1,x2,x3);
			Cr[3]=UVFILTER1(if0,if1,x3,x4);
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
		}
		break;
	case 2:
		jf1=(mvy & 7);
	        jf0=8-jf1;
		for (n = 0; n < 8; n++)
		{
			x0=iCb1[0];		y0=iCb2[0];
			x1=iCb1[1];		y1=iCb2[1];
			x2=iCb1[2];		y2=iCb2[2];
			x3=iCb1[3];		y3=iCb2[3];
			
			Cb[0]=UVFILTER1(jf0,jf1,x0,y0);
			Cb[1]=UVFILTER1(jf0,jf1,x1,y1);
			Cb[2]=UVFILTER1(jf0,jf1,x2,y2);
			Cb[3]=UVFILTER1(jf0,jf1,x3,y3);
			
			x0=iCr1[0];		y0=iCr2[0];
			x1=iCr1[1];		y1=iCr2[1];
			x2=iCr1[2];		y2=iCr2[2];
			x3=iCr1[3];		y3=iCr2[3];
			
			Cr[0]=UVFILTER1(jf0,jf1,x0,y0);
			Cr[1]=UVFILTER1(jf0,jf1,x1,y1);
			Cr[2]=UVFILTER1(jf0,jf1,x2,y2);
			Cr[3]=UVFILTER1(jf0,jf1,x3,y3);
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
			iCb2+=lx;		iCr2+=lx;
		}
		break;
	default:
		if1=(mvx & 7);
	        if0=8-if1;
	        jf1=(mvy & 7);
	        jf0=8-jf1;
		for (n = 0; n < 8; n++)
		{
			x0=iCb1[0];		y0=iCb2[0];
			x1=iCb1[1];		y1=iCb2[1];
			x2=iCb1[2];		y2=iCb2[2];
			x3=iCb1[3];		y3=iCb2[3];
			x4=iCb1[4];		y4=iCb2[4];
			
			Cb[0]=UVFILTER2(if0,if1,jf0,jf1,x0,y0,x1,y1);
			Cb[1]=UVFILTER2(if0,if1,jf0,jf1,x1,y1,x2,y2);
			Cb[2]=UVFILTER2(if0,if1,jf0,jf1,x2,y2,x3,y3);
			Cb[3]=UVFILTER2(if0,if1,jf0,jf1,x3,y3,x4,y4);
			
			x0=iCr1[0];		y0=iCr2[0];
			x1=iCr1[1];		y1=iCr2[1];
			x2=iCr1[2];		y2=iCr2[2];
			x3=iCr1[3];		y3=iCr2[3];
			x4=iCr1[4];		y4=iCr2[4];
			
			Cr[0]=UVFILTER2(if0,if1,jf0,jf1,x0,y0,x1,y1);
			Cr[1]=UVFILTER2(if0,if1,jf0,jf1,x1,y1,x2,y2);
			Cr[2]=UVFILTER2(if0,if1,jf0,jf1,x2,y2,x3,y3);
			Cr[3]=UVFILTER2(if0,if1,jf0,jf1,x3,y3,x4,y4);
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
			iCb2+=lx;		iCr2+=lx;
		}
		break;
	}
}

/************************************************************************
*
*  Routine :    dct_chroma
*
*  Description:    
*               Transform,quantization,inverse transform for chroma.  
*               The main reason why this is done in a separate routine is the 
*               additional 2x2 transform of DC-coeffs. This routine is called 
*               ones for each of the chroma components.
*
*  Input:       uv    : Make difference between the U and V chroma component  
*               cr_cbp: chroma coded block pattern              
*
*  Output:      cr_cbp: Updated chroma coded block pattern.
*               
*
************************************************************************/
void dct_chroma4x4(int *pp, int *outD, byte *ii ,struct img_par *img)
{
	int p0,p1,p2,p3;
  int q0,q1,q2,q3;
  int A0,A1,A2,A3;
  int B0,B1,B2,B3;
  int C0,C1,C2,C3;
  int D0,D1,D2,D3;
  int width=img->width_cr+IMG_PAD_SIZE;
  
  p0=ii[0]-pp[0];
  p1=ii[1]-pp[1];
  p2=ii[2]-pp[2];
  p3=ii[3]-pp[3];
  HOR_DCT(p0,p1,p2,p3);
  A0=p0;A1=p1;A2=p2;A3=p3;
  ii+=width;pp+=8;

  p0=ii[0]-pp[0];
  p1=ii[1]-pp[1];
  p2=ii[2]-pp[2];
  p3=ii[3]-pp[3];
  HOR_DCT(p0,p1,p2,p3);
  B0=p0;B1=p1;B2=p2;B3=p3;
  ii+=width;pp+=8;
  
  p0=ii[0]-pp[0];
  p1=ii[1]-pp[1];
  p2=ii[2]-pp[2];
  p3=ii[3]-pp[3];
  HOR_DCT(p0,p1,p2,p3);
  C0=p0;C1=p1;C2=p2;C3=p3;
  ii+=width;pp+=8;
  
  p0=ii[0]-pp[0];
  p1=ii[1]-pp[1];
  p2=ii[2]-pp[2];
  p3=ii[3]-pp[3];
  HOR_DCT(p0,p1,p2,p3);
  D0=p0;D1=p1;D2=p2;D3=p3;
  
  VER_DCT(A0,B0,C0,D0);
  outD[0]=p0;outD[2]=p1;outD[3]=p2;outD[9]=p3;
  VER_DCT(A1,B1,C1,D1);
  outD[1]=p0;outD[4]=p1;outD[8]=p2;outD[10]=p3;
  VER_DCT(A2,B2,C2,D2);
  outD[5]=p0;outD[7]=p1;outD[11]=p2;outD[14]=p3;
  VER_DCT(A3,B3,C3,D3);
  outD[6]=p0;outD[12]=p1;outD[13]=p2;outD[15]=p3;
}

void idct_chroma4x4(int *pp, int *outD, struct img_par *img)
{
	int p0,p1,p2,p3;
  int q0,q1,q2,q3;
  int A0,A1,A2,A3;
  int B0,B1,B2,B3;
  int C0,C1,C2,C3;
  int D0,D1,D2,D3;
  //int width=img->width_cr+IMG_PAD_SIZE;
  
  p0=outD[0];p1=outD[2];p2=outD[3];p3=outD[9];
  VER_IDCT(p0,p1,p2,p3);
  A0=p0;A1=p1;A2=p2;A3=p3;
  p0=outD[1];p1=outD[4];p2=outD[8];p3=outD[10];
  VER_IDCT(p0,p1,p2,p3);
  B0=p0;B1=p1;B2=p2;B3=p3;
  p0=outD[5];p1=outD[7];p2=outD[11];p3=outD[14];
  VER_IDCT(p0,p1,p2,p3);
  C0=p0;C1=p1;C2=p2;C3=p3;
  p0=outD[6];p1=outD[12];p2=outD[13];p3=outD[15];
  VER_IDCT(p0,p1,p2,p3);
  D0=p0;D1=p1;D2=p2;D3=p3;
  
  
  HOR_IDCT(A0,B0,C0,D0);
  outD[0]=IDCT_SHIFT(p0,pp[0]);
  outD[1]=IDCT_SHIFT(p1,pp[1]);
  outD[2]=IDCT_SHIFT(p2,pp[2]);
  outD[3]=IDCT_SHIFT(p3,pp[3]);
  pp+=8;
  HOR_IDCT(A1,B1,C1,D1);
  outD[4]=IDCT_SHIFT(p0,pp[0]);
  outD[5]=IDCT_SHIFT(p1,pp[1]);
  outD[6]=IDCT_SHIFT(p2,pp[2]);
  outD[7]=IDCT_SHIFT(p3,pp[3]);
  pp+=8;
  HOR_IDCT(A2,B2,C2,D2);
  outD[8]=IDCT_SHIFT(p0,pp[0]);
  outD[9]=IDCT_SHIFT(p1,pp[1]);
  outD[10]=IDCT_SHIFT(p2,pp[2]);
  outD[11]=IDCT_SHIFT(p3,pp[3]);
  pp+=8;
  HOR_IDCT(A3,B3,C3,D3);
  outD[12]=IDCT_SHIFT(p0,pp[0]);
  outD[13]=IDCT_SHIFT(p1,pp[1]);
  outD[14]=IDCT_SHIFT(p2,pp[2]);
  outD[15]=IDCT_SHIFT(p3,pp[3]);
}  
  
int intra_dct_chroma(int uv,int cr_cbp,byte *in,int *pp, struct img_par *img) 
{
	int i,j,ilev,n2,n1,coeff_ctr,qp_const,pos_x,pos_y,level ,scan_pos,run;
  int m1[BLOCK_SIZE];
  int coeff_cost;
  int cr_cbp_tmp;
  int nn0,nn1;
  int DCcoded=0 ;
  //Macroblock *currMB = &img->mb_data[img->current_mb_nr];

  int qp_per,qp_rem,q_bits;
  
  int p0,p1,p2,p3;
  //int q0,q1,q2,q3;
  int width=img->width_cr+IMG_PAD_SIZE;
  int outD[64];
  int *out;
  byte *ii=in;
  int s0=img->s[0];
  int s1=img->s[1];
  int s2=img->s[2];
  int s3=img->s[3];
  
  qp_per    = QP_SCALE_CR[img->qp-MIN_QP]/6;
  qp_rem    = QP_SCALE_CR[img->qp-MIN_QP]%6;
  q_bits    = Q_BITS+qp_per;

  if (img->type == INTRA_IMG)
    qp_const=(1<<q_bits)/3;    // intra
  else
    qp_const=(1<<q_bits)/6;    // inter
  
  dct_chroma4x4(pp, outD, ii ,img);
  dct_chroma4x4(pp+4, outD+16, ii+4 ,img);
  dct_chroma4x4(pp+32, outD+32, ii+width*4 ,img);
  dct_chroma4x4(pp+36, outD+48, ii+width*4+4 ,img);

  p0=outD[0];	p1=outD[16];	p2=outD[32];	p3=outD[48];
  m1[0]=(p0+p1+p2+p3);
  m1[1]=(p0-p1+p2-p3);
  m1[2]=(p0+p1-p2-p3);
  m1[3]=(p0-p1-p2+p3);
  
  run=-1;
  scan_pos=0;

  for (coeff_ctr=0; coeff_ctr < 4; coeff_ctr++) 
  {
    run++;
    ilev=0;

    level =(abs(m1[coeff_ctr]) * quant_coef[qp_rem][0][0] + 2*qp_const) >> (q_bits+1);
    if (level  != 0) 
    {
      cr_cbp=max(1,cr_cbp);
	  DCcoded = 1 ;
      img->cofu[scan_pos][0][uv]=sign(level ,m1[coeff_ctr]);
      img->cofu[scan_pos][1][uv]=run;
      scan_pos++;
      run=-1;
      ilev=level*dequant_coef[qp_rem][0][0]<<qp_per;
    }
    m1[coeff_ctr]=sign(ilev,m1[coeff_ctr]);
  }
  img->cofu[scan_pos][0][uv]=0;
  
  p0=m1[0];	p1=m1[1];	p2=m1[2];	p3=m1[3];
  outD[ 0]=(p0+p1+p2+p3)>>1;
  outD[16]=(p0-p1+p2-p3)>>1;
  outD[32]=(p0+p1-p2-p3)>>1;
  outD[48]=(p0-p1-p2+p3)>>1;
  
  //     Quant of chroma AC-coeffs.
  coeff_cost=0;
  cr_cbp_tmp=0;
  out=outD;

  for (n2=0; n2 <= BLOCK_SIZE; n2 += BLOCK_SIZE)
  {
    for (n1=0; n1 <= BLOCK_SIZE; n1 += BLOCK_SIZE)
    {
      pos_x=n1/BLOCK_SIZE + 2*uv;
      pos_y=n2/BLOCK_SIZE + BLOCK_SIZE;
      run=-1;
      scan_pos=0;

      for (coeff_ctr=1; coeff_ctr < 16; coeff_ctr++)
      {        
        ++run;
        ilev=0;

        level=(abs(out[coeff_ctr])*quant_coef1[qp_rem][coeff_ctr]+qp_const)>>q_bits;

        if (level  != 0)
        {
          if (level > 1)
            coeff_cost += MAX_VALUE;                // set high cost, shall not be discarded
          else
            coeff_cost += COEFF_COST[run];

          cr_cbp_tmp=2;
          img->cof[pos_x][pos_y][scan_pos][0]=sign(level,out[coeff_ctr]);
          img->cof[pos_x][pos_y][scan_pos][1]=run;
          ++scan_pos;
          run=-1;
          ilev=level*dequant_coef1[qp_rem][coeff_ctr]<<qp_per;
        }
        out[coeff_ctr]=sign(ilev,out[coeff_ctr]); // for use in IDCT
      }
      img->cof[pos_x][pos_y][scan_pos][0]=0;
      out+=16;
    }
  }

  // * reset chroma coeffs
  if(coeff_cost < _CHROMA_COEFF_COST_)
  {
    cr_cbp_tmp = 0 ;
    out=outD;
    for (n2=0; n2 <= BLOCK_SIZE; n2 += BLOCK_SIZE)
    {
      for (n1=0; n1 <= BLOCK_SIZE; n1 += BLOCK_SIZE)
      {
        pos_x=n1/BLOCK_SIZE + 2*uv;
        pos_y=n2/BLOCK_SIZE + BLOCK_SIZE;

        nn0 = (n1>>2) + (uv<<1);
        nn1 = 4 + (n2>>2) ;
        img->cof[pos_x][pos_y][0][0]=0;
        for (coeff_ctr=1; coeff_ctr < 16; coeff_ctr++)// ac coeff
        { 
          out[coeff_ctr]=0;
          img->cof[pos_x][pos_y][coeff_ctr][0]=0;
        }
        out+=16;
      }
    }
  }
  if(cr_cbp_tmp==2)
    cr_cbp = 2;

  idct_chroma4x4(pp, outD, img);
  idct_chroma4x4(pp+4, outD+16, img);
  idct_chroma4x4(pp+32, outD+32, img);
  idct_chroma4x4(pp+36, outD+48, img);
  out=outD;
  for (j=0; j < BLOCK_SIZE; j++) 
  {
  	i=(img->pix_c_y+j)*width+img->pix_c_x;
      imgUV[uv][i  ]= out[0];
      imgUV[uv][i+1]= out[1];
      imgUV[uv][i+2]= out[2];
      imgUV[uv][i+3]= out[3];
      imgUV[uv][i+4]= out[16];
      imgUV[uv][i+5]= out[17];
      imgUV[uv][i+6]= out[18];
      imgUV[uv][i+7]= out[19];
      out+=4;
  }
  out+=16;
  for (j=0; j < BLOCK_SIZE; j++) 
  {
  	i=(img->pix_c_y+j+4)*width+img->pix_c_x;
      imgUV[uv][i  ]= out[0];
      imgUV[uv][i+1]= out[1];
      imgUV[uv][i+2]= out[2];
      imgUV[uv][i+3]= out[3];
      imgUV[uv][i+4]= out[16];
      imgUV[uv][i+5]= out[17];
      imgUV[uv][i+6]= out[18];
      imgUV[uv][i+7]= out[19];
	  out+=4;
  }   
    return cr_cbp;
}

/* control the sign of a with b */
int sign(int a,int b)
{
  int x;
  x=absm(a); 
  if (b >= 0)
    return x;
  else
    return -x;
}

void Init_intra_mb_pos()
{
	int j;
	for(j=0;j<72;j++)
		memset(intra_mb_pos1[j]+4,0,88);
}

int SAD_Macroblock_sub3(struct img_par *img,byte* ii, byte* kk,int Min_FRAME,int curr_sad,int BlockType)
{
	int sad = curr_sad;
	int i;
	int lx=img->width+IMG_PAD_SIZE;
	int lines = 8>>(BlockType&1);
	
	switch(BlockType)
	{
		case 0:
		case 1:
		for(i=0;i<lines;i+=1)
		{
			sad += (\
					 IABS(ii[ 0]-kk[ 0])
					+IABS(ii[ 1]-kk[ 1])
					+IABS(ii[ 2]-kk[ 2])
					+IABS(ii[ 3]-kk[ 3])
					+IABS(ii[ 4]-kk[ 4])
					+IABS(ii[ 5]-kk[ 5])
					+IABS(ii[ 6]-kk[ 6])
					+IABS(ii[ 7]-kk[ 7]));
			ii += lx;
			kk += lx;
			if (sad > Min_FRAME)
				return 0x7fff;
		}
		break;
		
		case 2:
		case 3:
		for(i=0;i<lines;i+=1)
		{
			sad += (\
					 IABS(ii[ 0]-kk[ 0])
					+IABS(ii[ 1]-kk[ 1])
					+IABS(ii[ 2]-kk[ 2])
					+IABS(ii[ 3]-kk[ 3]));
			ii += lx;
			kk += lx;
			if (sad > Min_FRAME)
				return 0x7fff;
		}
		break;
	}
	return sad;
}

//BlockType: 0:8x8,1:8x4,2:4x8,3:4x4
int motion_search_8x8(struct img_par *img,int BlockType, int index, int MB_x, int MB_y)
{
  int i,j;
  int vec0_x,vec0_y,vec1_x,vec1_y,vec2_x,vec2_y;
  int pic_block_x,pic_block_y,pic_pix_y,pic_pix_x;
  int ip0,ip1,ip2,ip3,ip4,ip5;
  int best_inter_sad,current_inter_sad;
  int tmp0,tmp1; 
  int lambda = QP2QUANT[img->qp-SHIFT_QP]; 
  int x,y,offset,type,pos;  
        byte* curr;
	byte* prev;
	Point* point=search;
	SAD_TABLE * sadTbl=sadTable;
	int maxX=img->width-(8>>(BlockType>>1))+4;
	int maxY=img->height-(8>>(BlockType&1))+4;
	int sadTimes=25;
	int mvx0=0,mvy0=0,mvx1,mvy1; 
	int x_curr,y_curr;
	int blockshape_x,blockshape_y;
	int width=img->width+IMG_PAD_SIZE;

#ifdef FAST_INTEGER	
	int prev_AE1[17];
	int prev_m1,prev_m2=0;
	static int prevm[4] = {2, 3, 0, 1};
	static int htp[4] = {-1, 0, 1, 0};
	static int vtp[4] = {0, 1, 0, -1};
	int j_min_now, i_min_now, i_min_next=0, j_min_next=0, sad_layr,Coffset,ioffset=0,l,m;
	int distortion_0, distortion_1, distortion_2;
#endif    
	
  blockshape_x = 8>>(BlockType>>1);
  blockshape_y = 8>>(BlockType&1);
  maxX=img->width-blockshape_x;
  maxY=img->height-blockshape_y;
  
  pic_block_y=img->block_y+(MB_y>>2);
  pic_pix_y=img->pix_y+MB_y;
        
  pic_block_x=img->block_x+(MB_x>>2);
  pic_pix_x=img->pix_x+MB_x;

  x_curr=pic_pix_x;
  y_curr=pic_pix_y;
                    
  vec0_x=pic_block_x-1;
  vec0_y=pic_block_y;
  vec1_x=pic_block_x;
  vec1_y=pic_block_y-1;
  vec2_x=pic_block_x+(blockshape_x>>2);
  vec2_y=pic_block_y-1;
                    
          if (pic_block_y == 0) 
          {
            vec1_x=vec0_x;
            vec2_x=vec0_x;
            vec1_y=vec0_y;
            vec2_y=vec0_y;
          }                
          if (vec2_x > (img->width>>2)-1) 
          {
            vec2_x=pic_block_x-1;
          }
          if (MB_y > 0)
		  {
			  if (MB_x < 8)  // first column of 8x8 blocks
			  {
				  if (MB_y==8)
				  {
					  if (blockshape_x == 16)      vec2_x=pic_block_x-1;
				  }
				  else
				  {
					  if (MB_x+blockshape_x == 8)  vec2_x=pic_block_x-1;					  
				  }
			  }
			  else
			  {
				  if (MB_x+blockshape_x == 16)   vec2_x=pic_block_x-1;
			  }
		  }
          
          
            ip0=tmp_mv1[BlockType][vec0_y][vec0_x+4][0];
            ip1=tmp_mv1[BlockType][vec1_y][vec1_x+4][0];
            ip2=tmp_mv1[BlockType][vec2_y][vec2_x+4][0];
			ip3=tmp_mv1[BlockType][vec0_y][vec0_x+4][1];
            ip4=tmp_mv1[BlockType][vec1_y][vec1_x+4][1];
            ip5=tmp_mv1[BlockType][vec2_y][vec2_x+4][1];
			if((!intra_mb_pos1[vec0_y][vec0_x+4])&&(intra_mb_pos1[vec1_y][vec1_x+4])&&(intra_mb_pos1[vec2_y][vec2_x+4]))
			{
				img->mv3[BlockType][index][0] = ip0;
				img->mv3[BlockType][index][1] = ip3;
			}
			else if((intra_mb_pos1[vec0_y][vec0_x+4])&&(!intra_mb_pos1[vec1_y][vec1_x+4])&&(intra_mb_pos1[vec2_y][vec2_x+4]))
			{
				img->mv3[BlockType][index][0] = ip1;
				img->mv3[BlockType][index][1] = ip4;
			}
			else if((intra_mb_pos1[vec0_y][vec0_x+4])&&(intra_mb_pos1[vec1_y][vec1_x+4])&&(!intra_mb_pos1[vec2_y][vec2_x+4]))
			{
				img->mv3[BlockType][index][0] = ip2;
				img->mv3[BlockType][index][1] = ip5;
			}
			else
			{
				img->mv3[BlockType][index][0]=ip0+ip1+ip2-min(min(ip0,ip1),ip2)-max(max(ip0,ip1),ip2);
				img->mv3[BlockType][index][1]=ip3+ip4+ip5-min(min(ip3,ip4),ip5)-max(max(ip3,ip4),ip5);
			}
          ip0=img->mv3[BlockType][index][0];
          ip1=img->mv3[BlockType][index][1];

		 
    
	  
	offset=width*pic_pix_y+pic_pix_x;  
    curr=imgY_org + offset;
	prev=ipol[0] + offset;

#ifndef FAST_INTEGER 	
    tmp0=-ip0;
    tmp1=-ip1;
    current_inter_sad=(lambda*(img->mv_bituse[absm(tmp0)]+img->mv_bituse[absm(tmp1)]));
    best_inter_sad = SAD_Macroblock_sub3(img,curr, prev, MAX_VALUE,current_inter_sad,BlockType);
         
	pos=0;
	for (i = 1; i < sadTimes; i++)
	{
		sadTbl++;
		x=x_curr+sadTbl->dx;
		y=y_curr+sadTbl->dy;
		//if((x<0)||(y<0)||(x>maxX)||(y>maxY))
		//	continue;

		tmp0=(sadTbl->dx<<2)-ip0;
        tmp1=(sadTbl->dy<<2)-ip1;
        current_inter_sad=(lambda*(img->mv_bituse[absm(tmp0)]+img->mv_bituse[absm(tmp1)]));
		//current_inter_sad += SAD_Macroblock_sub3(img, curr,prev+sadTbl->offset, best_inter_sad,current_inter_sad,BlockType);
		current_inter_sad = SAD_Macroblock_sub3(img, curr,prev+sadTbl->offset, best_inter_sad,current_inter_sad,BlockType);
		if (current_inter_sad < best_inter_sad)
		{
			pos=i;
			best_inter_sad = current_inter_sad;
		}
	}
	mvx0 = sadTable[pos].dx;
	mvy0 = sadTable[pos].dy;
	offset+=sadTable[pos].offset;
	x_curr+=mvx0;
	y_curr+=mvy0;
#else
	i_min_now = min(maxX,max(-4, x_curr+(ip0>>2)));
	j_min_now = min(maxY,max(-4, y_curr+(ip1>>2)));
	//i_min_now = x_curr;
	//j_min_now = y_curr;
	mvx0=i_min_now-x_curr;
	mvy0=j_min_now-y_curr;
	ioffset=width*mvy0+mvx0;
	tmp0=(mvx0<<2)-ip0;
	tmp1=(mvy0<<2)-ip1;
	current_inter_sad=(lambda*(img->mv_bituse[absm(tmp0)]+img->mv_bituse[absm(tmp1)]));
	best_inter_sad = SAD_Macroblock_sub3(img,curr, prev+j_min_now*width+i_min_now-offset, MAX_VALUE,current_inter_sad,BlockType);

  	distortion_0 = distortion_1 = distortion_2 = 65536;
  	prev_AE1[0] = best_inter_sad;
  	prev_m1 = -1;
  	for (l = 1; l <= 16; l++){
  		sad_layr = 65536;
  		for (m = 0; m < 4; m++){
  			i = i_min_now + htp[m];
  			j = j_min_now + vtp[m];
  			Coffset = i - x_curr + (j - y_curr)*width;
  			if(l>2&&prev_m1 == m)
  			{
  				current_inter_sad = prev_AE1[l-2];
  				if (current_inter_sad < sad_layr){
  					sad_layr = current_inter_sad;
  					i_min_next = i;
  					j_min_next = j;
  					prev_m2=prevm[m];
  					prev_AE1[l]=current_inter_sad;
  				}
  			}
  			else if (i >= -4 && i <= maxX && j >= -4 && j <= maxY){
  				tmp0=((i - x_curr)<<2)-ip0;
			        tmp1=((j - y_curr)<<2)-ip1;
			        current_inter_sad=(lambda*(img->mv_bituse[IABS(tmp0)]+img->mv_bituse[IABS(tmp1)]));
			        //current_inter_sad += SAD_Macroblock_sub3(img,curr, prev+Coffset, sad_layr,current_inter_sad,BlockType);
					current_inter_sad = SAD_Macroblock_sub3(img,curr, prev+Coffset, sad_layr,current_inter_sad,BlockType);
  				//current_inter_sad = SAD_Macroblock(prev+Coffset, curr, sad_layr);
  				if (current_inter_sad < best_inter_sad){
  					mvx0  = i - x_curr;
  					mvy0  = j - y_curr;
  					ioffset = Coffset;
  					best_inter_sad = current_inter_sad ;
  				}
  				if (current_inter_sad < sad_layr){
  					sad_layr = current_inter_sad;
  					i_min_next = i;
  					j_min_next = j;
  					prev_m2 = prevm[m];
	  				prev_AE1[l] = current_inter_sad;
  				}
  			}
  		}
  		prev_m1 = prev_m2;
  		i_min_now = i_min_next;
  		j_min_now = j_min_next;
  		distortion_2 = distortion_1;
  		distortion_1 = distortion_0;
  		distortion_0 = sad_layr;
  		if (distortion_1 <= distortion_0 && distortion_2 <= distortion_0){
  			break;  			
  		}
  	}
  	offset+=ioffset;
	x_curr+=mvx0;
	y_curr+=mvy0;
#endif	
	
	pos=0;
	for (i = 1; i < 9; i++)
	{
		point++;
		x=x_curr+point->x;
		y=y_curr+point->y;
#ifdef FAST_INTEGER
		if((x<-4)||(y<-4)||(x>maxX)||(y>maxY))
			continue;
#endif
		
		tmp0=(mvx0<<2)+point->x-ip0;
        tmp1=(mvy0<<2)+point->y-ip1;
        current_inter_sad=(lambda*(img->mv_bituse[absm(tmp0)]+img->mv_bituse[absm(tmp1)]));
		prev=ipol[point->comp] + offset + point->offset;
		//current_inter_sad+=SAD_Macroblock_sub3(img,curr,prev,best_inter_sad,current_inter_sad,BlockType);
		current_inter_sad=SAD_Macroblock_sub3(img,curr,prev,best_inter_sad,current_inter_sad,BlockType);
		if (current_inter_sad < best_inter_sad)
		{
			best_inter_sad=current_inter_sad;
			pos=i;
		}
	}
	mvx1 = search[pos].x;
	mvy1 = search[pos].y;
	offset+=search[pos].offset;

	type  =search[pos].type;
	x_curr+=(mvx1>>1);
	y_curr+=(mvy1>>1);
	
	point=qsearch[type];
	pos=0;
	for (i = 1; i < 9; i++)
	{
		point++;
		x=x_curr+point->x;
		y=y_curr+point->y;
#ifdef FAST_INTEGER
		if((x<-4)||(y<-4)||(x>maxX)||(y>maxY))
			continue;
#endif
			
		tmp0=(mvx0<<2)+mvx1+point->x-ip0;
        tmp1=(mvy0<<2)+mvy1+point->y-ip1;
        current_inter_sad=(lambda*(img->mv_bituse[absm(tmp0)]+img->mv_bituse[absm(tmp1)]));
		prev=ipol[point->comp] + offset + point->offset;
		current_inter_sad=SAD_Macroblock_sub3(img,curr,prev,best_inter_sad,current_inter_sad,BlockType);
		if (current_inter_sad < best_inter_sad)
		{
			best_inter_sad=current_inter_sad;
			pos=i;
		}
	}
	offset+=qsearch[type][pos].offset;
	img->offset3[BlockType][index]=offset;
	img->comp3[BlockType][index]  =qsearch[type][pos].comp;
	ip0=curr_mvx3[BlockType][index]=(mvx0<<2)+mvx1+qsearch[type][pos].x;
	ip1=curr_mvy3[BlockType][index]=(mvy0<<2)+mvy1+qsearch[type][pos].y;

	for(j=0;j<(blockshape_y>>2);j++)
	for(i=0;i<(blockshape_x>>2);i++)
	{
		tmp_mv1[BlockType][pic_block_y+j][pic_block_x+4+i][0] = ip0;
		tmp_mv1[BlockType][pic_block_y+j][pic_block_x+4+i][1] = ip1;
	}
	return best_inter_sad;
}

void Copy_tmpmv(struct img_par *img)
{
	int i,t;
	int pic_block_x,pic_block_y;
	
	pic_block_x=img->block_x+4;
	pic_block_y=img->block_y; 
	
	for(t=0;t<4;t++)
	for(i=0;i<4;i++)
	{
		tmp_mv1[t][pic_block_y+i][pic_block_x  ][0] = tmp_mv[pic_block_y+i][pic_block_x  ][0];
		tmp_mv1[t][pic_block_y+i][pic_block_x  ][1] = tmp_mv[pic_block_y+i][pic_block_x  ][1];
		tmp_mv1[t][pic_block_y+i][pic_block_x+1][0] = tmp_mv[pic_block_y+i][pic_block_x+1][0];
		tmp_mv1[t][pic_block_y+i][pic_block_x+1][1] = tmp_mv[pic_block_y+i][pic_block_x+1][1];
		tmp_mv1[t][pic_block_y+i][pic_block_x+2][0] = tmp_mv[pic_block_y+i][pic_block_x+2][0];
		tmp_mv1[t][pic_block_y+i][pic_block_x+2][1] = tmp_mv[pic_block_y+i][pic_block_x+2][1];
		tmp_mv1[t][pic_block_y+i][pic_block_x+3][0] = tmp_mv[pic_block_y+i][pic_block_x+3][0];
		tmp_mv1[t][pic_block_y+i][pic_block_x+3][1] = tmp_mv[pic_block_y+i][pic_block_x+3][1];
	}
}

void Copy_tmpmv1(struct img_par *img, int index_8x8, int SubBlockType, int pic_block_x, int pic_block_y)
{
	int t,index;
	
	switch(SubBlockType)
	{
		case 0:
		index=index_8x8;
		for(t=0;t<4;t++)
		{
			tmp_mv1[t][pic_block_y  ][pic_block_x  ][0] = curr_mvx3[0][index];
			tmp_mv1[t][pic_block_y  ][pic_block_x  ][1] = curr_mvy3[0][index];
			tmp_mv1[t][pic_block_y  ][pic_block_x+1][0] = curr_mvx3[0][index];
			tmp_mv1[t][pic_block_y  ][pic_block_x+1][1] = curr_mvy3[0][index];
			tmp_mv1[t][pic_block_y+1][pic_block_x  ][0] = curr_mvx3[0][index];
			tmp_mv1[t][pic_block_y+1][pic_block_x  ][1] = curr_mvy3[0][index];
			tmp_mv1[t][pic_block_y+1][pic_block_x+1][0] = curr_mvx3[0][index];
			tmp_mv1[t][pic_block_y+1][pic_block_x+1][1] = curr_mvy3[0][index];
		}
		break;
		
		case 1:
		index=index_8x8<<1;
		for(t=0;t<4;t++)
		{
			tmp_mv1[t][pic_block_y  ][pic_block_x  ][0] = curr_mvx3[1][index];
			tmp_mv1[t][pic_block_y  ][pic_block_x  ][1] = curr_mvy3[1][index];
			tmp_mv1[t][pic_block_y  ][pic_block_x+1][0] = curr_mvx3[1][index];
			tmp_mv1[t][pic_block_y  ][pic_block_x+1][1] = curr_mvy3[1][index];
			tmp_mv1[t][pic_block_y+1][pic_block_x  ][0] = curr_mvx3[1][index+1];
			tmp_mv1[t][pic_block_y+1][pic_block_x  ][1] = curr_mvy3[1][index+1];
			tmp_mv1[t][pic_block_y+1][pic_block_x+1][0] = curr_mvx3[1][index+1];
			tmp_mv1[t][pic_block_y+1][pic_block_x+1][1] = curr_mvy3[1][index+1];
		}
		break;
		
		case 2:
		index=index_8x8<<1;
		for(t=0;t<4;t++)
		{
			tmp_mv1[t][pic_block_y  ][pic_block_x  ][0] = curr_mvx3[2][index];
			tmp_mv1[t][pic_block_y  ][pic_block_x  ][1] = curr_mvy3[2][index];
			tmp_mv1[t][pic_block_y  ][pic_block_x+1][0] = curr_mvx3[2][index+1];
			tmp_mv1[t][pic_block_y  ][pic_block_x+1][1] = curr_mvy3[2][index+1];
			tmp_mv1[t][pic_block_y+1][pic_block_x  ][0] = curr_mvx3[2][index];
			tmp_mv1[t][pic_block_y+1][pic_block_x  ][1] = curr_mvy3[2][index];
			tmp_mv1[t][pic_block_y+1][pic_block_x+1][0] = curr_mvx3[2][index+1];
			tmp_mv1[t][pic_block_y+1][pic_block_x+1][1] = curr_mvy3[2][index+1];
		}
		break;
		
		case 3:
		index=index_8x8<<2;
		for(t=0;t<4;t++)
		{
			tmp_mv1[t][pic_block_y  ][pic_block_x  ][0] = curr_mvx3[3][index];
			tmp_mv1[t][pic_block_y  ][pic_block_x  ][1] = curr_mvy3[3][index];
			tmp_mv1[t][pic_block_y  ][pic_block_x+1][0] = curr_mvx3[3][index+1];
			tmp_mv1[t][pic_block_y  ][pic_block_x+1][1] = curr_mvy3[3][index+1];
			tmp_mv1[t][pic_block_y+1][pic_block_x  ][0] = curr_mvx3[3][index+2];
			tmp_mv1[t][pic_block_y+1][pic_block_x  ][1] = curr_mvy3[3][index+2];
			tmp_mv1[t][pic_block_y+1][pic_block_x+1][0] = curr_mvx3[3][index+3];
			tmp_mv1[t][pic_block_y+1][pic_block_x+1][1] = curr_mvy3[3][index+3];
		}
		break;
	}
}

int SmallBlock_ME(struct img_par *img, int Min_Cost)
{
	int i;
	int MinCost,CurrCost,MBCost;
	int pic_block_x,pic_block_y;
	
	pic_block_x = img->block_x+4;
	pic_block_y = img->block_y;
	//first block
	MinCost  = motion_search_8x8(img,0, 0, 0, 0);
	CurrCost = motion_search_8x8(img,1, 0, 0, 0);
	CurrCost+= motion_search_8x8(img,1, 1, 0, 4);
	if(CurrCost<MinCost)
	{
		img->SubMBType1=1;
		MinCost = CurrCost;
	}
	else
	{
		img->SubMBType1=0;
	}
	
	CurrCost = motion_search_8x8(img,2, 0, 0, 0);
	CurrCost+= motion_search_8x8(img,2, 1, 4, 0);
	if(CurrCost<MinCost)
	{
		img->SubMBType1=2;
		MinCost = CurrCost;
	}
	
	CurrCost = motion_search_8x8(img,3, 0, 0, 0);
	CurrCost+= motion_search_8x8(img,3, 1, 4, 0);
	CurrCost+= motion_search_8x8(img,3, 2, 0, 4);
	CurrCost+= motion_search_8x8(img,3, 3, 4, 4);
	if(CurrCost<MinCost)
	{
		img->SubMBType1=3;
		MinCost = CurrCost;
	}
	MBCost = MinCost;
	Copy_tmpmv1(img,0,img->SubMBType1,pic_block_x,pic_block_y);
	
	//second blcok
	MinCost  = motion_search_8x8(img,0, 1, 8, 0);
	CurrCost = motion_search_8x8(img,1, 2, 8, 0);
	CurrCost+= motion_search_8x8(img,1, 3, 8, 4);
	if(CurrCost<MinCost)
	{
		img->SubMBType2=1;
		MinCost = CurrCost;
	}
	else
	{
		img->SubMBType2=0;
	}
	
	CurrCost = motion_search_8x8(img,2, 2, 8, 0);
	CurrCost+= motion_search_8x8(img,2, 3,12, 0);
	if(CurrCost<MinCost)
	{
		img->SubMBType2=2;
		MinCost = CurrCost;
	}
	
	CurrCost = motion_search_8x8(img,3, 4, 8, 0);
	CurrCost+= motion_search_8x8(img,3, 5,12, 0);
	CurrCost+= motion_search_8x8(img,3, 6, 8, 4);
	CurrCost+= motion_search_8x8(img,3, 7,12, 4);
	if(CurrCost<MinCost)
	{
		img->SubMBType2=3;
		MinCost = CurrCost;
	}
	MBCost += MinCost;
	Copy_tmpmv1(img,1,img->SubMBType2,pic_block_x+2,pic_block_y);
	
	//third block
	MinCost  = motion_search_8x8(img,0, 2, 0, 8);
	CurrCost = motion_search_8x8(img,1, 4, 0, 8);
	CurrCost+= motion_search_8x8(img,1, 5, 0,12);
	if(CurrCost<MinCost)
	{
		img->SubMBType3=1;
		MinCost = CurrCost;
	}
	else
	{
		img->SubMBType3=0;
	}
	
	CurrCost = motion_search_8x8(img,2, 4, 0, 8);
	CurrCost+= motion_search_8x8(img,2, 5, 4, 8);
	if(CurrCost<MinCost)
	{
		img->SubMBType3=2;
		MinCost = CurrCost;
	}
	
	CurrCost = motion_search_8x8(img,3, 8, 0, 8);
	CurrCost+= motion_search_8x8(img,3, 9, 4, 8);
	CurrCost+= motion_search_8x8(img,3,10, 0,12);
	CurrCost+= motion_search_8x8(img,3,11, 4,12);
	if(CurrCost<MinCost)
	{
		img->SubMBType3=3;
		MinCost = CurrCost;
	}
	MBCost += MinCost;
	Copy_tmpmv1(img,2,img->SubMBType3,pic_block_x,pic_block_y+2);
	
	//fourth blcok
	MinCost  = motion_search_8x8(img,0, 3, 8, 8);				
	CurrCost = motion_search_8x8(img,1, 6, 8, 8);
	CurrCost+= motion_search_8x8(img,1, 7, 8,12);
	if(CurrCost<MinCost)
	{
		img->SubMBType4=1;
		MinCost = CurrCost;
	}
	else
	{
		img->SubMBType4=0;
	}				
	CurrCost = motion_search_8x8(img,2, 6, 8, 8);
	CurrCost+= motion_search_8x8(img,2, 7,12, 8);
	if(CurrCost<MinCost)
	{
		img->SubMBType4=2;
		MinCost = CurrCost;
	}				
	CurrCost = motion_search_8x8(img,3,12, 8, 8);
	CurrCost+= motion_search_8x8(img,3,13,12, 8);
	CurrCost+= motion_search_8x8(img,3,14, 8,12);
	CurrCost+= motion_search_8x8(img,3,15,12,12);
	if(CurrCost<MinCost)
	{
		img->SubMBType4=3;
		MinCost = CurrCost;
	}
	MBCost += MinCost;
	Copy_tmpmv1(img,3,img->SubMBType4,pic_block_x+2,pic_block_y+2);

	if(MBCost<Min_Cost)
	{
		img->imod = INTRA_MB_INTER;
		switch(img->SubMBType1)
		{
			case 0:
			tmp_mv[pic_block_y  ][pic_block_x  ][0] = curr_mvx3[0][0];
			tmp_mv[pic_block_y  ][pic_block_x  ][1] = curr_mvy3[0][0];
			tmp_mv[pic_block_y  ][pic_block_x+1][0] = curr_mvx3[0][0];
			tmp_mv[pic_block_y  ][pic_block_x+1][1] = curr_mvy3[0][0];
			tmp_mv[pic_block_y+1][pic_block_x  ][0] = curr_mvx3[0][0];
			tmp_mv[pic_block_y+1][pic_block_x  ][1] = curr_mvy3[0][0];
			tmp_mv[pic_block_y+1][pic_block_x+1][0] = curr_mvx3[0][0];
			tmp_mv[pic_block_y+1][pic_block_x+1][1] = curr_mvy3[0][0];
			break;
			
			case 1:
			tmp_mv[pic_block_y  ][pic_block_x  ][0] = curr_mvx3[1][0];
			tmp_mv[pic_block_y  ][pic_block_x  ][1] = curr_mvy3[1][0];
			tmp_mv[pic_block_y  ][pic_block_x+1][0] = curr_mvx3[1][0];
			tmp_mv[pic_block_y  ][pic_block_x+1][1] = curr_mvy3[1][0];
			tmp_mv[pic_block_y+1][pic_block_x  ][0] = curr_mvx3[1][1];
			tmp_mv[pic_block_y+1][pic_block_x  ][1] = curr_mvy3[1][1];
			tmp_mv[pic_block_y+1][pic_block_x+1][0] = curr_mvx3[1][1];
			tmp_mv[pic_block_y+1][pic_block_x+1][1] = curr_mvy3[1][1];
			break;
			
			case 2:
			tmp_mv[pic_block_y  ][pic_block_x  ][0] = curr_mvx3[2][0];
			tmp_mv[pic_block_y  ][pic_block_x  ][1] = curr_mvy3[2][0];
			tmp_mv[pic_block_y  ][pic_block_x+1][0] = curr_mvx3[2][1];
			tmp_mv[pic_block_y  ][pic_block_x+1][1] = curr_mvy3[2][1];
			tmp_mv[pic_block_y+1][pic_block_x  ][0] = curr_mvx3[2][0];
			tmp_mv[pic_block_y+1][pic_block_x  ][1] = curr_mvy3[2][0];
			tmp_mv[pic_block_y+1][pic_block_x+1][0] = curr_mvx3[2][1];
			tmp_mv[pic_block_y+1][pic_block_x+1][1] = curr_mvy3[2][1];
			break;
			
			case 3:
			tmp_mv[pic_block_y  ][pic_block_x  ][0] = curr_mvx3[3][0];
			tmp_mv[pic_block_y  ][pic_block_x  ][1] = curr_mvy3[3][0];
			tmp_mv[pic_block_y  ][pic_block_x+1][0] = curr_mvx3[3][1];
			tmp_mv[pic_block_y  ][pic_block_x+1][1] = curr_mvy3[3][1];
			tmp_mv[pic_block_y+1][pic_block_x  ][0] = curr_mvx3[3][2];
			tmp_mv[pic_block_y+1][pic_block_x  ][1] = curr_mvy3[3][2];
			tmp_mv[pic_block_y+1][pic_block_x+1][0] = curr_mvx3[3][3];
			tmp_mv[pic_block_y+1][pic_block_x+1][1] = curr_mvy3[3][3];
			break;
		}
		
		switch(img->SubMBType2)
		{
			case 0:
			tmp_mv[pic_block_y  ][pic_block_x+2][0] = curr_mvx3[0][1];
			tmp_mv[pic_block_y  ][pic_block_x+2][1] = curr_mvy3[0][1];
			tmp_mv[pic_block_y  ][pic_block_x+3][0] = curr_mvx3[0][1];
			tmp_mv[pic_block_y  ][pic_block_x+3][1] = curr_mvy3[0][1];
			tmp_mv[pic_block_y+1][pic_block_x+2][0] = curr_mvx3[0][1];
			tmp_mv[pic_block_y+1][pic_block_x+2][1] = curr_mvy3[0][1];
			tmp_mv[pic_block_y+1][pic_block_x+3][0] = curr_mvx3[0][1];
			tmp_mv[pic_block_y+1][pic_block_x+3][1] = curr_mvy3[0][1];
			break;
			
			case 1:
			tmp_mv[pic_block_y  ][pic_block_x+2][0] = curr_mvx3[1][2];
			tmp_mv[pic_block_y  ][pic_block_x+2][1] = curr_mvy3[1][2];
			tmp_mv[pic_block_y  ][pic_block_x+3][0] = curr_mvx3[1][2];
			tmp_mv[pic_block_y  ][pic_block_x+3][1] = curr_mvy3[1][2];
			tmp_mv[pic_block_y+1][pic_block_x+2][0] = curr_mvx3[1][3];
			tmp_mv[pic_block_y+1][pic_block_x+2][1] = curr_mvy3[1][3];
			tmp_mv[pic_block_y+1][pic_block_x+3][0] = curr_mvx3[1][3];
			tmp_mv[pic_block_y+1][pic_block_x+3][1] = curr_mvy3[1][3];
			break;
			
			case 2:
			tmp_mv[pic_block_y  ][pic_block_x+2][0] = curr_mvx3[2][2];
			tmp_mv[pic_block_y  ][pic_block_x+2][1] = curr_mvy3[2][2];
			tmp_mv[pic_block_y  ][pic_block_x+3][0] = curr_mvx3[2][3];
			tmp_mv[pic_block_y  ][pic_block_x+3][1] = curr_mvy3[2][3];
			tmp_mv[pic_block_y+1][pic_block_x+2][0] = curr_mvx3[2][2];
			tmp_mv[pic_block_y+1][pic_block_x+2][1] = curr_mvy3[2][2];
			tmp_mv[pic_block_y+1][pic_block_x+3][0] = curr_mvx3[2][3];
			tmp_mv[pic_block_y+1][pic_block_x+3][1] = curr_mvy3[2][3];
			break;
			
			case 3:
			tmp_mv[pic_block_y  ][pic_block_x+2][0] = curr_mvx3[3][4];
			tmp_mv[pic_block_y  ][pic_block_x+2][1] = curr_mvy3[3][4];
			tmp_mv[pic_block_y  ][pic_block_x+3][0] = curr_mvx3[3][5];
			tmp_mv[pic_block_y  ][pic_block_x+3][1] = curr_mvy3[3][5];
			tmp_mv[pic_block_y+1][pic_block_x+2][0] = curr_mvx3[3][6];
			tmp_mv[pic_block_y+1][pic_block_x+2][1] = curr_mvy3[3][6];
			tmp_mv[pic_block_y+1][pic_block_x+3][0] = curr_mvx3[3][7];
			tmp_mv[pic_block_y+1][pic_block_x+3][1] = curr_mvy3[3][7];
			break;
		}
		
		switch(img->SubMBType3)
		{
			case 0:
			tmp_mv[pic_block_y+2][pic_block_x  ][0] = curr_mvx3[0][2];
			tmp_mv[pic_block_y+2][pic_block_x  ][1] = curr_mvy3[0][2];
			tmp_mv[pic_block_y+2][pic_block_x+1][0] = curr_mvx3[0][2];
			tmp_mv[pic_block_y+2][pic_block_x+1][1] = curr_mvy3[0][2];
			tmp_mv[pic_block_y+3][pic_block_x  ][0] = curr_mvx3[0][2];
			tmp_mv[pic_block_y+3][pic_block_x  ][1] = curr_mvy3[0][2];
			tmp_mv[pic_block_y+3][pic_block_x+1][0] = curr_mvx3[0][2];
			tmp_mv[pic_block_y+3][pic_block_x+1][1] = curr_mvy3[0][2];
			break;
			
			case 1:
			tmp_mv[pic_block_y+2][pic_block_x  ][0] = curr_mvx3[1][4];
			tmp_mv[pic_block_y+2][pic_block_x  ][1] = curr_mvy3[1][4];
			tmp_mv[pic_block_y+2][pic_block_x+1][0] = curr_mvx3[1][4];
			tmp_mv[pic_block_y+2][pic_block_x+1][1] = curr_mvy3[1][4];
			tmp_mv[pic_block_y+3][pic_block_x  ][0] = curr_mvx3[1][5];
			tmp_mv[pic_block_y+3][pic_block_x  ][1] = curr_mvy3[1][5];
			tmp_mv[pic_block_y+3][pic_block_x+1][0] = curr_mvx3[1][5];
			tmp_mv[pic_block_y+3][pic_block_x+1][1] = curr_mvy3[1][5];
			break;
			
			case 2:
			tmp_mv[pic_block_y+2][pic_block_x  ][0] = curr_mvx3[2][4];
			tmp_mv[pic_block_y+2][pic_block_x  ][1] = curr_mvy3[2][4];
			tmp_mv[pic_block_y+2][pic_block_x+1][0] = curr_mvx3[2][5];
			tmp_mv[pic_block_y+2][pic_block_x+1][1] = curr_mvy3[2][5];
			tmp_mv[pic_block_y+3][pic_block_x  ][0] = curr_mvx3[2][4];
			tmp_mv[pic_block_y+3][pic_block_x  ][1] = curr_mvy3[2][4];
			tmp_mv[pic_block_y+3][pic_block_x+1][0] = curr_mvx3[2][5];
			tmp_mv[pic_block_y+3][pic_block_x+1][1] = curr_mvy3[2][5];
			break;
			
			case 3:
			tmp_mv[pic_block_y+2][pic_block_x  ][0] = curr_mvx3[3][8 ];
			tmp_mv[pic_block_y+2][pic_block_x  ][1] = curr_mvy3[3][8 ];
			tmp_mv[pic_block_y+2][pic_block_x+1][0] = curr_mvx3[3][9 ];
			tmp_mv[pic_block_y+2][pic_block_x+1][1] = curr_mvy3[3][9 ];
			tmp_mv[pic_block_y+3][pic_block_x  ][0] = curr_mvx3[3][10];
			tmp_mv[pic_block_y+3][pic_block_x  ][1] = curr_mvy3[3][10];
			tmp_mv[pic_block_y+3][pic_block_x+1][0] = curr_mvx3[3][11];
			tmp_mv[pic_block_y+3][pic_block_x+1][1] = curr_mvy3[3][11];
			break;
		}
		
		switch(img->SubMBType4)
		{
			case 0:
			tmp_mv[pic_block_y+2][pic_block_x+2][0] = curr_mvx3[0][3];
			tmp_mv[pic_block_y+2][pic_block_x+2][1] = curr_mvy3[0][3];
			tmp_mv[pic_block_y+2][pic_block_x+3][0] = curr_mvx3[0][3];
			tmp_mv[pic_block_y+2][pic_block_x+3][1] = curr_mvy3[0][3];
			tmp_mv[pic_block_y+3][pic_block_x+2][0] = curr_mvx3[0][3];
			tmp_mv[pic_block_y+3][pic_block_x+2][1] = curr_mvy3[0][3];
			tmp_mv[pic_block_y+3][pic_block_x+3][0] = curr_mvx3[0][3];
			tmp_mv[pic_block_y+3][pic_block_x+3][1] = curr_mvy3[0][3];
			break;
			
			case 1:
			tmp_mv[pic_block_y+2][pic_block_x+2][0] = curr_mvx3[1][6];
			tmp_mv[pic_block_y+2][pic_block_x+2][1] = curr_mvy3[1][6];
			tmp_mv[pic_block_y+2][pic_block_x+3][0] = curr_mvx3[1][6];
			tmp_mv[pic_block_y+2][pic_block_x+3][1] = curr_mvy3[1][6];
			tmp_mv[pic_block_y+3][pic_block_x+2][0] = curr_mvx3[1][7];
			tmp_mv[pic_block_y+3][pic_block_x+2][1] = curr_mvy3[1][7];
			tmp_mv[pic_block_y+3][pic_block_x+3][0] = curr_mvx3[1][7];
			tmp_mv[pic_block_y+3][pic_block_x+3][1] = curr_mvy3[1][7];
			break;
			
			case 2:
			tmp_mv[pic_block_y+2][pic_block_x+2][0] = curr_mvx3[2][6];
			tmp_mv[pic_block_y+2][pic_block_x+2][1] = curr_mvy3[2][6];
			tmp_mv[pic_block_y+2][pic_block_x+3][0] = curr_mvx3[2][7];
			tmp_mv[pic_block_y+2][pic_block_x+3][1] = curr_mvy3[2][7];
			tmp_mv[pic_block_y+3][pic_block_x+2][0] = curr_mvx3[2][6];
			tmp_mv[pic_block_y+3][pic_block_x+2][1] = curr_mvy3[2][6];
			tmp_mv[pic_block_y+3][pic_block_x+3][0] = curr_mvx3[2][7];
			tmp_mv[pic_block_y+3][pic_block_x+3][1] = curr_mvy3[2][7];
			break;
			
			case 3:
			tmp_mv[pic_block_y+2][pic_block_x+2][0] = curr_mvx3[3][12];
			tmp_mv[pic_block_y+2][pic_block_x+2][1] = curr_mvy3[3][12];
			tmp_mv[pic_block_y+2][pic_block_x+3][0] = curr_mvx3[3][13];
			tmp_mv[pic_block_y+2][pic_block_x+3][1] = curr_mvy3[3][13];
			tmp_mv[pic_block_y+3][pic_block_x+2][0] = curr_mvx3[3][14];
			tmp_mv[pic_block_y+3][pic_block_x+2][1] = curr_mvy3[3][14];
			tmp_mv[pic_block_y+3][pic_block_x+3][0] = curr_mvx3[3][15];
			tmp_mv[pic_block_y+3][pic_block_x+3][1] = curr_mvy3[3][15];
			break;
		}
		for(i=0;i<4;i++)
		{
			intra_mb_pos[pic_block_y+i][pic_block_x] = 0;
			intra_mb_pos[pic_block_y+i][pic_block_x+1] = 0;
			intra_mb_pos[pic_block_y+i][pic_block_x+2] = 0;
			intra_mb_pos[pic_block_y+i][pic_block_x+3] = 0;
		}
		Min_Cost=MBCost;
	}

	return Min_Cost;
}
	
void Predict_8x8(struct img_par *img, int index, int MB_x, int MB_y)
{
	int m,n;
	int x = img->pix_x+MB_x;
	int y = img->pix_y+MB_y;
	int mvx = curr_mvx3[0][index];
	int mvy = curr_mvy3[0][index];
	int*  Cb;
	int*  Cr;
	int offset1;
	int offset2;
	int cc;
	byte*  iCb1;
	byte*  iCb2;
	byte*  iCr1;
	byte*  iCr2;

	int lx=img->width+IMG_PAD_SIZE;
	byte*  curr=imgY_org + x+y*lx;
	byte*  prev=ipol[img->comp3[0][index]] + img->offset3[0][index];
	byte*  pred=LumPred+MB_y*16+MB_x;
	int*  error=MbLum+16*MB_y+MB_x;
	
	int if0,if1,jf0,jf1;

	int x0,x1,x2,x3,x4;
	int y0,y1,y2,y3,y4,y5,y6,y7;
	int z0,z1,z2,z3,z4,z5,z6,z7;

	for (n = 0; n < 8; n++)
	{
		z0=curr[ 0];		y0=pred[0]=prev[ 0];
		z1=curr[ 1];		y1=pred[1]=prev[ 1];
		z2=curr[ 2];		y2=pred[2]=prev[ 2];
		z3=curr[ 3];		y3=pred[3]=prev[ 3];
		z4=curr[ 4];		y4=pred[4]=prev[ 4];
		z5=curr[ 5];		y5=pred[5]=prev[ 5];
		z6=curr[ 6];		y6=pred[6]=prev[ 6];
		z7=curr[ 7];		y7=pred[7]=prev[ 7];
		error[ 0]=z0-y0;
		error[ 1]=z1-y1;
		error[ 2]=z2-y2;
		error[ 3]=z3-y3;
		error[ 4]=z4-y4;
		error[ 5]=z5-y5;
		error[ 6]=z6-y6;
		error[ 7]=z7-y7;
		
		error+=16;
		curr+=lx;
		prev+=lx;
		pred+=16;
	}
	
  x = img->pix_c_x+(MB_x>>1);
  y = img->pix_c_y+(MB_y>>1);
  m=( (mvx&7)==0 ? 0:1);
  n=( (mvy&7)==0 ? 0:1);
  cc=m+(n<<1);
  lx = img->width_cr+IMG_PAD_SIZE;
  Cb=MbCb+MB_y*4+(MB_x>>1);
  Cr=MbCr+MB_y*4+(MB_x>>1);
  offset1=x + (mvx>>3) + (y + (mvy>>3))*lx;
  offset2=x + y*lx;	
  iCb1=mcef[0]+offset1;
  iCb2=iCb1+lx;
  iCr1=mcef[1]+offset1;
  iCr2=iCr1+lx;
  switch(cc)
  {
  	case 0:
  		for (n = 0; n < 4; n++)
		{
			x0=iCb1[0];
			x1=iCb1[1];
			x2=iCb1[2];
			x3=iCb1[3];
			
			Cb[0]=x0;
			Cb[1]=x1;
			Cb[2]=x2;
			Cb[3]=x3;
			
			x0=iCr1[0];
			x1=iCr1[1];
			x2=iCr1[2];
			x3=iCr1[3];
			
			Cr[0]=x0;
			Cr[1]=x1;
			Cr[2]=x2;
			Cr[3]=x3;
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
		}
		break;
	case 1:
		if1=(mvx & 7);
	        if0=8-if1;	 
		for (n = 0; n < 4; n++)
		{
			x0=iCb1[0];
			x1=iCb1[1];
			x2=iCb1[2];
			x3=iCb1[3];
			x4=iCb1[4];
			
			Cb[0]=UVFILTER1(if0,if1,x0,x1);
			Cb[1]=UVFILTER1(if0,if1,x1,x2);
			Cb[2]=UVFILTER1(if0,if1,x2,x3);
			Cb[3]=UVFILTER1(if0,if1,x3,x4);
			
			x0=iCr1[0];
			x1=iCr1[1];
			x2=iCr1[2];
			x3=iCr1[3];
			x4=iCr1[4];
			
			Cr[0]=UVFILTER1(if0,if1,x0,x1);
			Cr[1]=UVFILTER1(if0,if1,x1,x2);
			Cr[2]=UVFILTER1(if0,if1,x2,x3);
			Cr[3]=UVFILTER1(if0,if1,x3,x4);
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
		}
		break;
	case 2:
		jf1=(mvy & 7);
	        jf0=8-jf1;
		for (n = 0; n < 4; n++)
		{
			x0=iCb1[0];		y0=iCb2[0];
			x1=iCb1[1];		y1=iCb2[1];
			x2=iCb1[2];		y2=iCb2[2];
			x3=iCb1[3];		y3=iCb2[3];
			
			Cb[0]=UVFILTER1(jf0,jf1,x0,y0);
			Cb[1]=UVFILTER1(jf0,jf1,x1,y1);
			Cb[2]=UVFILTER1(jf0,jf1,x2,y2);
			Cb[3]=UVFILTER1(jf0,jf1,x3,y3);
			
			x0=iCr1[0];		y0=iCr2[0];
			x1=iCr1[1];		y1=iCr2[1];
			x2=iCr1[2];		y2=iCr2[2];
			x3=iCr1[3];		y3=iCr2[3];
			
			Cr[0]=UVFILTER1(jf0,jf1,x0,y0);
			Cr[1]=UVFILTER1(jf0,jf1,x1,y1);
			Cr[2]=UVFILTER1(jf0,jf1,x2,y2);
			Cr[3]=UVFILTER1(jf0,jf1,x3,y3);
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
			iCb2+=lx;		iCr2+=lx;
		}
		break;
	default:
		if1=(mvx & 7);
	        if0=8-if1;
	        jf1=(mvy & 7);
	        jf0=8-jf1;
		for (n = 0; n < 4; n++)
		{
			x0=iCb1[0];		y0=iCb2[0];
			x1=iCb1[1];		y1=iCb2[1];
			x2=iCb1[2];		y2=iCb2[2];
			x3=iCb1[3];		y3=iCb2[3];
			x4=iCb1[4];		y4=iCb2[4];
			
			Cb[0]=UVFILTER2(if0,if1,jf0,jf1,x0,y0,x1,y1);
			Cb[1]=UVFILTER2(if0,if1,jf0,jf1,x1,y1,x2,y2);
			Cb[2]=UVFILTER2(if0,if1,jf0,jf1,x2,y2,x3,y3);
			Cb[3]=UVFILTER2(if0,if1,jf0,jf1,x3,y3,x4,y4);
			
			x0=iCr1[0];		y0=iCr2[0];
			x1=iCr1[1];		y1=iCr2[1];
			x2=iCr1[2];		y2=iCr2[2];
			x3=iCr1[3];		y3=iCr2[3];
			x4=iCr1[4];		y4=iCr2[4];
			
			Cr[0]=UVFILTER2(if0,if1,jf0,jf1,x0,y0,x1,y1);
			Cr[1]=UVFILTER2(if0,if1,jf0,jf1,x1,y1,x2,y2);
			Cr[2]=UVFILTER2(if0,if1,jf0,jf1,x2,y2,x3,y3);
			Cr[3]=UVFILTER2(if0,if1,jf0,jf1,x3,y3,x4,y4);
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
			iCb2+=lx;		iCr2+=lx;
		}
		break;
	}
}

void Predict_8x4(struct img_par *img, int index, int MB_x, int MB_y)
{
	int m,n;
	int x = img->pix_x+MB_x;
	int y = img->pix_y+MB_y;
	int mvx = curr_mvx3[1][index];
	int mvy = curr_mvy3[1][index];
	int*  Cb;
	int*  Cr;
	int offset1;
	int offset2;
	int cc;
	byte*  iCb1;
	byte*  iCb2;
	byte*  iCr1;
	byte*  iCr2;

	int lx=img->width+IMG_PAD_SIZE;
	byte*  curr=imgY_org + x+y*lx;
	byte*  prev=ipol[img->comp3[1][index]] + img->offset3[1][index];
	byte*  pred=LumPred+MB_y*16+MB_x;
	int*  error=MbLum+16*MB_y+MB_x;
	
	int if0,if1,jf0,jf1;

	int x0,x1,x2,x3,x4;
	int y0,y1,y2,y3,y4,y5,y6,y7;
	int z0,z1,z2,z3,z4,z5,z6,z7;

	for (n = 0; n < 4; n++)
	{
		z0=curr[ 0];		y0=pred[0]=prev[ 0];
		z1=curr[ 1];		y1=pred[1]=prev[ 1];
		z2=curr[ 2];		y2=pred[2]=prev[ 2];
		z3=curr[ 3];		y3=pred[3]=prev[ 3];
		z4=curr[ 4];		y4=pred[4]=prev[ 4];
		z5=curr[ 5];		y5=pred[5]=prev[ 5];
		z6=curr[ 6];		y6=pred[6]=prev[ 6];
		z7=curr[ 7];		y7=pred[7]=prev[ 7];
		error[ 0]=z0-y0;
		error[ 1]=z1-y1;
		error[ 2]=z2-y2;
		error[ 3]=z3-y3;
		error[ 4]=z4-y4;
		error[ 5]=z5-y5;
		error[ 6]=z6-y6;
		error[ 7]=z7-y7;
		
		error+=16;
		curr+=lx;
		prev+=lx;
		pred+=16;
	}
	
  x = img->pix_c_x+(MB_x>>1);
  y = img->pix_c_y+(MB_y>>1);
  m=( (mvx&7)==0 ? 0:1);
  n=( (mvy&7)==0 ? 0:1);
  cc=m+(n<<1);
  lx = img->width_cr+IMG_PAD_SIZE;
  Cb=MbCb+MB_y*4+(MB_x>>1);
  Cr=MbCr+MB_y*4+(MB_x>>1);
  offset1=x + (mvx>>3) + (y + (mvy>>3))*lx;
  offset2=x + y*lx;	
  iCb1=mcef[0]+offset1;
  iCb2=iCb1+lx;
  iCr1=mcef[1]+offset1;
  iCr2=iCr1+lx;
  
  switch(cc)
  {
  	case 0:
  		for (n = 0; n < 2; n++)
		{
			x0=iCb1[0];
			x1=iCb1[1];
			x2=iCb1[2];
			x3=iCb1[3];
			
			Cb[0]=x0;
			Cb[1]=x1;
			Cb[2]=x2;
			Cb[3]=x3;
			
			x0=iCr1[0];
			x1=iCr1[1];
			x2=iCr1[2];
			x3=iCr1[3];
			
			Cr[0]=x0;
			Cr[1]=x1;
			Cr[2]=x2;
			Cr[3]=x3;
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
		}
		break;
	case 1:
		if1=(mvx & 7);
	        if0=8-if1;	 
		for (n = 0; n < 2; n++)
		{
			x0=iCb1[0];
			x1=iCb1[1];
			x2=iCb1[2];
			x3=iCb1[3];
			x4=iCb1[4];
			
			Cb[0]=UVFILTER1(if0,if1,x0,x1);
			Cb[1]=UVFILTER1(if0,if1,x1,x2);
			Cb[2]=UVFILTER1(if0,if1,x2,x3);
			Cb[3]=UVFILTER1(if0,if1,x3,x4);
			
			x0=iCr1[0];
			x1=iCr1[1];
			x2=iCr1[2];
			x3=iCr1[3];
			x4=iCr1[4];
			
			Cr[0]=UVFILTER1(if0,if1,x0,x1);
			Cr[1]=UVFILTER1(if0,if1,x1,x2);
			Cr[2]=UVFILTER1(if0,if1,x2,x3);
			Cr[3]=UVFILTER1(if0,if1,x3,x4);
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
		}
		break;
	case 2:
		jf1=(mvy & 7);
	        jf0=8-jf1;
		for (n = 0; n < 2; n++)
		{
			x0=iCb1[0];		y0=iCb2[0];
			x1=iCb1[1];		y1=iCb2[1];
			x2=iCb1[2];		y2=iCb2[2];
			x3=iCb1[3];		y3=iCb2[3];
			
			Cb[0]=UVFILTER1(jf0,jf1,x0,y0);
			Cb[1]=UVFILTER1(jf0,jf1,x1,y1);
			Cb[2]=UVFILTER1(jf0,jf1,x2,y2);
			Cb[3]=UVFILTER1(jf0,jf1,x3,y3);
			
			x0=iCr1[0];		y0=iCr2[0];
			x1=iCr1[1];		y1=iCr2[1];
			x2=iCr1[2];		y2=iCr2[2];
			x3=iCr1[3];		y3=iCr2[3];
			
			Cr[0]=UVFILTER1(jf0,jf1,x0,y0);
			Cr[1]=UVFILTER1(jf0,jf1,x1,y1);
			Cr[2]=UVFILTER1(jf0,jf1,x2,y2);
			Cr[3]=UVFILTER1(jf0,jf1,x3,y3);
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
			iCb2+=lx;		iCr2+=lx;
		}
		break;
	default:
		if1=(mvx & 7);
	        if0=8-if1;
	        jf1=(mvy & 7);
	        jf0=8-jf1;
		for (n = 0; n < 2; n++)
		{
			x0=iCb1[0];		y0=iCb2[0];
			x1=iCb1[1];		y1=iCb2[1];
			x2=iCb1[2];		y2=iCb2[2];
			x3=iCb1[3];		y3=iCb2[3];
			x4=iCb1[4];		y4=iCb2[4];
			
			Cb[0]=UVFILTER2(if0,if1,jf0,jf1,x0,y0,x1,y1);
			Cb[1]=UVFILTER2(if0,if1,jf0,jf1,x1,y1,x2,y2);
			Cb[2]=UVFILTER2(if0,if1,jf0,jf1,x2,y2,x3,y3);
			Cb[3]=UVFILTER2(if0,if1,jf0,jf1,x3,y3,x4,y4);
			
			x0=iCr1[0];		y0=iCr2[0];
			x1=iCr1[1];		y1=iCr2[1];
			x2=iCr1[2];		y2=iCr2[2];
			x3=iCr1[3];		y3=iCr2[3];
			x4=iCr1[4];		y4=iCr2[4];
			
			Cr[0]=UVFILTER2(if0,if1,jf0,jf1,x0,y0,x1,y1);
			Cr[1]=UVFILTER2(if0,if1,jf0,jf1,x1,y1,x2,y2);
			Cr[2]=UVFILTER2(if0,if1,jf0,jf1,x2,y2,x3,y3);
			Cr[3]=UVFILTER2(if0,if1,jf0,jf1,x3,y3,x4,y4);
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
			iCb2+=lx;		iCr2+=lx;
		}
		break;
	}
}

void Predict_4x8(struct img_par *img, int index, int MB_x, int MB_y)
{
	int m,n;
	int x = img->pix_x+MB_x;
	int y = img->pix_y+MB_y;
	int mvx = curr_mvx3[2][index];
	int mvy = curr_mvy3[2][index];
	int*  Cb;
	int*  Cr;
	int offset1;
	int offset2;
	int cc;
	byte*  iCb1;
	byte*  iCb2;
	byte*  iCr1;
	byte*  iCr2;

	int lx=img->width+IMG_PAD_SIZE;
	byte*  curr=imgY_org + x+y*lx;
	byte*  prev=ipol[img->comp3[2][index]] + img->offset3[2][index];
	byte*  pred=LumPred+MB_y*16+MB_x;
	int*  error=MbLum+16*MB_y+MB_x;
	
	int if0,if1,jf0,jf1;

	int x0,x1,x2;
	int y0,y1,y2,y3;
	int z0,z1,z2,z3;

	for (n = 0; n < 8; n++)
	{
		z0=curr[ 0];		y0=pred[0]=prev[ 0];
		z1=curr[ 1];		y1=pred[1]=prev[ 1];
		z2=curr[ 2];		y2=pred[2]=prev[ 2];
		z3=curr[ 3];		y3=pred[3]=prev[ 3];
		
		error[ 0]=z0-y0;
		error[ 1]=z1-y1;
		error[ 2]=z2-y2;
		error[ 3]=z3-y3;
		
		error+=16;
		curr+=lx;
		prev+=lx;
		pred+=16;
	}
	
  x = img->pix_c_x+(MB_x>>1);
  y = img->pix_c_y+(MB_y>>1);
  m=( (mvx&7)==0 ? 0:1);
  n=( (mvy&7)==0 ? 0:1);
  cc=m+(n<<1);
  lx = img->width_cr+IMG_PAD_SIZE;
  Cb=MbCb+MB_y*4+(MB_x>>1);
  Cr=MbCr+MB_y*4+(MB_x>>1);
  offset1=x + (mvx>>3) + (y + (mvy>>3))*lx;
  offset2=x + y*lx;	
  iCb1=mcef[0]+offset1;
  iCb2=iCb1+lx;
  iCr1=mcef[1]+offset1;
  iCr2=iCr1+lx;
  
  switch(cc)
  {
  	case 0:
  		for (n = 0; n < 4; n++)
		{
			x0=iCb1[0];
			x1=iCb1[1];
			
			Cb[0]=x0;
			Cb[1]=x1;
			
			x0=iCr1[0];
			x1=iCr1[1];
			
			Cr[0]=x0;
			Cr[1]=x1;
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
		}
		break;
	case 1:
		if1=(mvx & 7);
	        if0=8-if1;	 
		for (n = 0; n < 4; n++)
		{
			x0=iCb1[0];
			x1=iCb1[1];
			x2=iCb1[2];
			
			Cb[0]=UVFILTER1(if0,if1,x0,x1);
			Cb[1]=UVFILTER1(if0,if1,x1,x2);
			
			x0=iCr1[0];
			x1=iCr1[1];
			x2=iCr1[2];
			
			Cr[0]=UVFILTER1(if0,if1,x0,x1);
			Cr[1]=UVFILTER1(if0,if1,x1,x2);
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
		}
		break;
	case 2:
		jf1=(mvy & 7);
	        jf0=8-jf1;
		for (n = 0; n < 4; n++)
		{
			x0=iCb1[0];		y0=iCb2[0];
			x1=iCb1[1];		y1=iCb2[1];
			
			Cb[0]=UVFILTER1(jf0,jf1,x0,y0);
			Cb[1]=UVFILTER1(jf0,jf1,x1,y1);
			
			x0=iCr1[0];		y0=iCr2[0];
			x1=iCr1[1];		y1=iCr2[1];
			
			Cr[0]=UVFILTER1(jf0,jf1,x0,y0);
			Cr[1]=UVFILTER1(jf0,jf1,x1,y1);
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
			iCb2+=lx;		iCr2+=lx;
		}
		break;
	default:
		if1=(mvx & 7);
	        if0=8-if1;
	        jf1=(mvy & 7);
	        jf0=8-jf1;
		for (n = 0; n < 4; n++)
		{
			x0=iCb1[0];		y0=iCb2[0];
			x1=iCb1[1];		y1=iCb2[1];
			x2=iCb1[2];		y2=iCb2[2];
			
			Cb[0]=UVFILTER2(if0,if1,jf0,jf1,x0,y0,x1,y1);
			Cb[1]=UVFILTER2(if0,if1,jf0,jf1,x1,y1,x2,y2);
			
			x0=iCr1[0];		y0=iCr2[0];
			x1=iCr1[1];		y1=iCr2[1];
			x2=iCr1[2];		y2=iCr2[2];
			
			Cr[0]=UVFILTER2(if0,if1,jf0,jf1,x0,y0,x1,y1);
			Cr[1]=UVFILTER2(if0,if1,jf0,jf1,x1,y1,x2,y2);
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
			iCb2+=lx;		iCr2+=lx;
		}
		break;
	}
}

void Predict_4x4(struct img_par *img, int index, int MB_x, int MB_y)
{
	int m,n;
	int x = img->pix_x+MB_x;
	int y = img->pix_y+MB_y;
	int mvx = curr_mvx3[3][index];
	int mvy = curr_mvy3[3][index];
	int*  Cb;
	int*  Cr;
	int offset1;
	int offset2;
	int cc;
	byte*  iCb1;
	byte*  iCb2;
	byte*  iCr1;
	byte*  iCr2;

	int lx=img->width+IMG_PAD_SIZE;
	byte*  curr=imgY_org + x+y*lx;
	byte*  prev=ipol[img->comp3[3][index]] + img->offset3[3][index];
	byte*  pred=LumPred+MB_y*16+MB_x;
	int*  error=MbLum+16*MB_y+MB_x;
	
	int if0,if1,jf0,jf1;

	int x0,x1,x2;
	int y0,y1,y2,y3;
	int z0,z1,z2,z3;

	for (n = 0; n < 4; n++)
	{
		z0=curr[ 0];		y0=pred[0]=prev[ 0];
		z1=curr[ 1];		y1=pred[1]=prev[ 1];
		z2=curr[ 2];		y2=pred[2]=prev[ 2];
		z3=curr[ 3];		y3=pred[3]=prev[ 3];
		
		error[ 0]=z0-y0;
		error[ 1]=z1-y1;
		error[ 2]=z2-y2;
		error[ 3]=z3-y3;
		
		error+=16;
		curr+=lx;
		prev+=lx;
		pred+=16;
	}
	
  x = img->pix_c_x+(MB_x>>1);
  y = img->pix_c_y+(MB_y>>1);
  m=( (mvx&7)==0 ? 0:1);
  n=( (mvy&7)==0 ? 0:1);
  cc=m+(n<<1);
  lx = img->width_cr+IMG_PAD_SIZE;
  Cb=MbCb+MB_y*4+(MB_x>>1);
  Cr=MbCr+MB_y*4+(MB_x>>1);
  offset1=x + (mvx>>3) + (y + (mvy>>3))*lx;
  offset2=x + y*lx;	
  iCb1=mcef[0]+offset1;
  iCb2=iCb1+lx;
  iCr1=mcef[1]+offset1;
  iCr2=iCr1+lx;
  
  switch(cc)
  {
  	case 0:
  		for (n = 0; n < 2; n++)
		{
			x0=iCb1[0];
			x1=iCb1[1];
			
			Cb[0]=x0;
			Cb[1]=x1;
			
			x0=iCr1[0];
			x1=iCr1[1];
			
			Cr[0]=x0;
			Cr[1]=x1;
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
		}
		break;
	case 1:
		if1=(mvx & 7);
	        if0=8-if1;	 
		for (n = 0; n < 2; n++)
		{
			x0=iCb1[0];
			x1=iCb1[1];
			x2=iCb1[2];
			
			Cb[0]=UVFILTER1(if0,if1,x0,x1);
			Cb[1]=UVFILTER1(if0,if1,x1,x2);
			
			x0=iCr1[0];
			x1=iCr1[1];
			x2=iCr1[2];
			
			Cr[0]=UVFILTER1(if0,if1,x0,x1);
			Cr[1]=UVFILTER1(if0,if1,x1,x2);
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
		}
		break;
	case 2:
		jf1=(mvy & 7);
	        jf0=8-jf1;
		for (n = 0; n < 2; n++)
		{
			x0=iCb1[0];		y0=iCb2[0];
			x1=iCb1[1];		y1=iCb2[1];
			
			Cb[0]=UVFILTER1(jf0,jf1,x0,y0);
			Cb[1]=UVFILTER1(jf0,jf1,x1,y1);
			
			x0=iCr1[0];		y0=iCr2[0];
			x1=iCr1[1];		y1=iCr2[1];
			
			Cr[0]=UVFILTER1(jf0,jf1,x0,y0);
			Cr[1]=UVFILTER1(jf0,jf1,x1,y1);
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
			iCb2+=lx;		iCr2+=lx;
		}
		break;
	default:
		if1=(mvx & 7);
	        if0=8-if1;
	        jf1=(mvy & 7);
	        jf0=8-jf1;
		for (n = 0; n < 2; n++)
		{
			x0=iCb1[0];		y0=iCb2[0];
			x1=iCb1[1];		y1=iCb2[1];
			x2=iCb1[2];		y2=iCb2[2];
			
			Cb[0]=UVFILTER2(if0,if1,jf0,jf1,x0,y0,x1,y1);
			Cb[1]=UVFILTER2(if0,if1,jf0,jf1,x1,y1,x2,y2);
			
			x0=iCr1[0];		y0=iCr2[0];
			x1=iCr1[1];		y1=iCr2[1];
			x2=iCr1[2];		y2=iCr2[2];
			
			Cr[0]=UVFILTER2(if0,if1,jf0,jf1,x0,y0,x1,y1);
			Cr[1]=UVFILTER2(if0,if1,jf0,jf1,x1,y1,x2,y2);
			
			Cb+=8;			Cr+=8;
			iCb1+=lx;		iCr1+=lx;
			iCb2+=lx;		iCr2+=lx;
		}
		break;
	}
}

void macroblock_inter_SmallBlock(struct img_par *img)
{
	int i;
	int cbp,cr_cbp=0;
	int cbptmp=0;
	int sum_cnt_nonz,coeff_cost;
	int *Lum=MbLum;
	int *LumTmp=MbLumTmp;
	byte *curr,*pre;
	int width=img->width+IMG_PAD_SIZE;
	int width_cr=img->width_cr+IMG_PAD_SIZE;
	int pic_block_x=img->block_x;
	int	pic_block_y=img->block_y;
	
	for(i=0;i<4;i++)
	{
		img->ipredmode[pic_block_x+i+1][pic_block_y+1]=DC_PRED;
		img->ipredmode[pic_block_x+i+1][pic_block_y+2]=DC_PRED;
		img->ipredmode[pic_block_x+i+1][pic_block_y+3]=DC_PRED;
		img->ipredmode[pic_block_x+i+1][pic_block_y+4]=DC_PRED;
	}
	switch(img->SubMBType1)
	{
		case 0:
		Predict_8x8(img,0,0,0);
		break;
		
		case 1:
		Predict_8x4(img,0,0,0);
		Predict_8x4(img,1,0,4);
		break;
		
		case 2:
		Predict_4x8(img,0,0,0);
		Predict_4x8(img,1,4,0);
		break;
		
		case 3:
		Predict_4x4(img,0,0,0);
		Predict_4x4(img,1,4,0);
		Predict_4x4(img,2,0,4);
		Predict_4x4(img,3,4,4);
		break;
	}
	
	switch(img->SubMBType2)
	{
		case 0:
		Predict_8x8(img,1,8,0);
		break;
		
		case 1:
		Predict_8x4(img,2,8,0);
		Predict_8x4(img,3,8,4);
		break;
		
		case 2:
		Predict_4x8(img,2, 8,0);
		Predict_4x8(img,3,12,0);
		break;
		
		case 3:
		Predict_4x4(img,4, 8,0);
		Predict_4x4(img,5,12,0);
		Predict_4x4(img,6, 8,4);
		Predict_4x4(img,7,12,4);
		break;
	}
	
	switch(img->SubMBType3)
	{
		case 0:
		Predict_8x8(img,2,0,8);
		break;
		
		case 1:
		Predict_8x4(img,4,0, 8);
		Predict_8x4(img,5,0,12);
		break;
		
		case 2:
		Predict_4x8(img,4,0,8);
		Predict_4x8(img,5,4,8);
		break;
		
		case 3:
		Predict_4x4(img, 8,0, 8);
		Predict_4x4(img, 9,4, 8);
		Predict_4x4(img,10,0,12);
		Predict_4x4(img,11,4,12);
		break;
	}
	
	switch(img->SubMBType4)
	{
		case 0:
		Predict_8x8(img,3,8,8);
		break;
		
		case 1:
		Predict_8x4(img,6,8, 8);
		Predict_8x4(img,7,8,12);
		break;
		
		case 2:
		Predict_4x8(img,6, 8,8);
		Predict_4x8(img,7,12,8);
		break;
		
		case 3:
		Predict_4x4(img,12, 8, 8);
		Predict_4x4(img,13,12, 8);
		Predict_4x4(img,14, 8,12);
		Predict_4x4(img,15,12,12);
		break;
	}

    	cbp=0;
    	sum_cnt_nonz=0;
    	
    	coeff_cost=0;
    	cbptmp|=inter_dct_luma(Lum,   LumTmp   ,0,0,&coeff_cost,img);
    	cbptmp|=inter_dct_luma(Lum+4, LumTmp+16,4,0,&coeff_cost,img);
    	cbptmp|=inter_dct_luma(Lum+64,LumTmp+32,0,4,&coeff_cost,img);
    	cbptmp|=inter_dct_luma(Lum+68,LumTmp+48,4,4,&coeff_cost,img);
    	if (coeff_cost > _LUMA_COEFF_COST_) 
        {
        	cbp=cbptmp;
          	sum_cnt_nonz += coeff_cost;
        }
        else //discard
        {
		curr= imgY+img->pix_y*width+img->pix_x;
		pre = LumPred;

		for (i=0; i < 8; i++)
		{
          		memcpy(curr+i*width,pre+i*16,8);
		}
		cbp_blk[pic_block_y  ][pic_block_x  ]=0;
			cbp_blk[pic_block_y  ][pic_block_x+1]=0;
			cbp_blk[pic_block_y+1][pic_block_x  ]=0;
			cbp_blk[pic_block_y+1][pic_block_x+1]=0;
        }
		cbptmp=0;
        Lum+=8;
        LumTmp+=64;
        
        coeff_cost=0;
    	cbptmp|=inter_dct_luma(Lum,   LumTmp   ,8, 0,&coeff_cost,img);
    	cbptmp|=inter_dct_luma(Lum+4, LumTmp+16,12,0,&coeff_cost,img);
    	cbptmp|=inter_dct_luma(Lum+64,LumTmp+32,8, 4,&coeff_cost,img);
    	cbptmp|=inter_dct_luma(Lum+68,LumTmp+48,12,4,&coeff_cost,img);
    	if (coeff_cost > _LUMA_COEFF_COST_) 
        {
        	cbp|=(cbptmp<<1);
          	sum_cnt_nonz += coeff_cost;
        }
        else //discard
        {
		curr= imgY+img->pix_y*width+img->pix_x+8;				
		pre = LumPred+8;
			  
		for (i=0; i < 8; i++)
		{
          		memcpy(curr+i*width,pre+i*16,8);
		}
		cbp_blk[pic_block_y  ][pic_block_x+2]=0;
		cbp_blk[pic_block_y  ][pic_block_x+3]=0;
		cbp_blk[pic_block_y+1][pic_block_x+2]=0;
		cbp_blk[pic_block_y+1][pic_block_x+3]=0;
        }
		cbptmp=0;
        Lum+=120;//128-8;
        LumTmp+=64;
        
        coeff_cost=0;
    	cbptmp|=inter_dct_luma(Lum,   LumTmp   ,0,8 ,&coeff_cost,img);
    	cbptmp|=inter_dct_luma(Lum+4, LumTmp+16,4,8 ,&coeff_cost,img);
    	cbptmp|=inter_dct_luma(Lum+64,LumTmp+32,0,12,&coeff_cost,img);
    	cbptmp|=inter_dct_luma(Lum+68,LumTmp+48,4,12,&coeff_cost,img);
    	if (coeff_cost > _LUMA_COEFF_COST_) 
        {
        	cbp|=(cbptmp<<2);
          sum_cnt_nonz += coeff_cost;
        }
        else //discard
        {
		curr= imgY+img->pix_y*width+img->pix_x+8*width;
		pre = LumPred+128;
			  
		for (i=0; i < 8; i++)
		{
          		memcpy(curr+i*width,pre+i*16,8);
		}
		cbp_blk[pic_block_y+2][pic_block_x  ]=0;
		cbp_blk[pic_block_y+2][pic_block_x+1]=0;
		cbp_blk[pic_block_y+3][pic_block_x  ]=0;
		cbp_blk[pic_block_y+3][pic_block_x+1]=0;
        }
		cbptmp=0;
        Lum+=8;
        LumTmp+=64;
        
        coeff_cost=0;
    	cbptmp|=inter_dct_luma(Lum,   LumTmp   ,8 ,8 ,&coeff_cost,img);
    	cbptmp|=inter_dct_luma(Lum+4, LumTmp+16,12,8 ,&coeff_cost,img);
    	cbptmp|=inter_dct_luma(Lum+64,LumTmp+32,8 ,12,&coeff_cost,img);
    	cbptmp|=inter_dct_luma(Lum+68,LumTmp+48,12,12,&coeff_cost,img);
    	if (coeff_cost > _LUMA_COEFF_COST_) 
        {
        	cbp|=(cbptmp<<3);
          sum_cnt_nonz += coeff_cost;
        }
        else //discard
        {
		curr= imgY+img->pix_y*width+img->pix_x+8*width+8;
		pre = LumPred+136;
			  
		for (i=0; i < 8; i++)
		{
          		memcpy(curr+i*width,pre+i*16,8);
		}
		cbp_blk[pic_block_y+2][pic_block_x+2]=0;
			cbp_blk[pic_block_y+2][pic_block_x+3]=0;
			cbp_blk[pic_block_y+3][pic_block_x+2]=0;
			cbp_blk[pic_block_y+3][pic_block_x+3]=0;
        }
        
        if (sum_cnt_nonz <= 5 ) 
		{
		  cbp=cbp & 48; // mask bit 4 and 5
		  
		  curr= imgY+img->pix_y*width+img->pix_x;
		  pre = LumPred;
		  for (i=0; i < 16; i++)
		  {
			memcpy(curr+i*width,pre+i*16,16);
		  }
				cbp_blk[pic_block_y  ][pic_block_x  ]=0;
				cbp_blk[pic_block_y  ][pic_block_x+1]=0;
				cbp_blk[pic_block_y  ][pic_block_x+2]=0;
				cbp_blk[pic_block_y  ][pic_block_x+3]=0;
				cbp_blk[pic_block_y+1][pic_block_x  ]=0;
				cbp_blk[pic_block_y+1][pic_block_x+1]=0;
				cbp_blk[pic_block_y+1][pic_block_x+2]=0;
				cbp_blk[pic_block_y+1][pic_block_x+3]=0;
				cbp_blk[pic_block_y+2][pic_block_x  ]=0;
				cbp_blk[pic_block_y+2][pic_block_x+1]=0;
				cbp_blk[pic_block_y+2][pic_block_x+2]=0;
				cbp_blk[pic_block_y+2][pic_block_x+3]=0;
				cbp_blk[pic_block_y+3][pic_block_x  ]=0;
				cbp_blk[pic_block_y+3][pic_block_x+1]=0;
				cbp_blk[pic_block_y+3][pic_block_x+2]=0;
				cbp_blk[pic_block_y+3][pic_block_x+3]=0;
	}
	
		if(cbp&1)
		{
			LumTmp=MbLumTmp;
			pre=LumPred;
			inter_idct_luma(LumTmp   ,pre   ,0,0,img,16);
			inter_idct_luma(LumTmp+16,pre+4 ,4,0,img,16);
			inter_idct_luma(LumTmp+32,pre+64,0,4,img,16);
			inter_idct_luma(LumTmp+48,pre+68,4,4,img,16);
		}
		if(cbp&2)
		{
			LumTmp=MbLumTmp+64;
			pre=LumPred+8;
			inter_idct_luma(LumTmp   ,pre   ,8 ,0,img,16);
			inter_idct_luma(LumTmp+16,pre+4 ,12,0,img,16);
			inter_idct_luma(LumTmp+32,pre+64,8 ,4,img,16);
			inter_idct_luma(LumTmp+48,pre+68,12,4,img,16);
		}
		if(cbp&4)
		{
			LumTmp=MbLumTmp+128;
			pre=LumPred+128;
			inter_idct_luma(LumTmp   ,pre   ,0,8 ,img,16);
			inter_idct_luma(LumTmp+16,pre+4 ,4,8 ,img,16);
			inter_idct_luma(LumTmp+32,pre+64,0,12,img,16);
			inter_idct_luma(LumTmp+48,pre+68,4,12,img,16);
		}
		if(cbp&8)
		{
			LumTmp=MbLumTmp+192;
			pre=LumPred+136;
			inter_idct_luma(LumTmp   ,pre   ,8 ,8 ,img,16);
			inter_idct_luma(LumTmp+16,pre+4 ,12,8 ,img,16);
			inter_idct_luma(LumTmp+32,pre+64,8 ,12,img,16);
			inter_idct_luma(LumTmp+48,pre+68,12,12,img,16);
		}
		
		
		cr_cbp=intra_dct_chroma(0,cr_cbp,imgUV_org[0]+img->pix_c_y*width_cr+img->pix_c_x,MbCb,img);
		cr_cbp=intra_dct_chroma(1,cr_cbp,imgUV_org[1]+img->pix_c_y*width_cr+img->pix_c_x,MbCr,img);
		cbp += cr_cbp*16;
		img->cbp=cbp;
}
